@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2008, 2017 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -306,6 +309,19 @@ private String getDeclaredType(NamedNodeMap map, Node location) {
 
 	// strip any _Nullable and _Nonnull annotations
 	value = value.replace("_Nullable", "").replace("_Nonnull", "").replace("_Null_unspecified", "");
+	
+	// strip greater-than (>) sign
+	value = value.replace(">", "");
+
+	// also strip __kindof keyword
+	value = value.replace("__kindof", "");
+
+	// anther generics thing is "ObjectType", however, those names can be arbitrary
+	// let's to a catch-all for common names here and handle other cases individually via mapping in extras file
+	value = value.replace("ObjectType", "id").replace("KeyType", "id");
+
+	// also invalid: 'struct FSRef*' should be 'FSRef*'
+	value = value.replace("struct ", "");
 
 	// also remove any white spaces
 	value = value.chars().filter((c)->!Character.isWhitespace(c)).mapToObj(c -> String.valueOf((char)c)).collect(Collectors.joining());
