@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2016 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -114,7 +117,13 @@ public class Accessible {
 	Accessible (Control control) {
 		super ();
 		this.control = control;
-		AccessibleFactory.registerAccessible (this);
+		if (GTK.GTK3) {
+			long /*int*/ type = OS.G_OBJECT_TYPE (getControlHandle());
+			accessibleObject = new AccessibleObject (type, getControlHandle(), this, false);
+			addRelations();
+		} else {
+			AccessibleFactory.registerAccessible (this);
+		}
 	}
 
 	/**
@@ -411,7 +420,6 @@ public class Accessible {
 	 *
 	 * @param type an <code>ACC</code> constant beginning with RELATION_* indicating the type of relation
 	 * @param target the accessible that is the target for this relation
-	 *
 	 * @exception IllegalArgumentException ERROR_NULL_ARGUMENT - if the Accessible target is null
 	 * @since 3.6
 	 */
@@ -475,11 +483,23 @@ public class Accessible {
 
 	AccessibleObject getAccessibleObject () {
 		if (accessibleObject == null) {
-			if (parent == null) {
-				AccessibleFactory.createAccessible(this);
+			if (GTK.GTK3) {
+				long /*int*/ widget = this.getControlHandle();
+				long /*int*/ type = OS.G_OBJECT_TYPE (widget);
+				if (parent == null) {
+					accessibleObject = new AccessibleObject (type, widget, this, false);
+				} else {
+					accessibleObject = new AccessibleObject (type, 0, this, true);
+					accessibleObject.parent = parent.getAccessibleObject();
+				}
 			} else {
-				accessibleObject = AccessibleFactory.createChildAccessible(this, ACC.CHILDID_SELF);
-				accessibleObject.parent = parent.getAccessibleObject();
+				if (parent == null) {
+					AccessibleFactory.createAccessible(this);
+				} else {
+					accessibleObject = AccessibleFactory.createChildAccessible(this, ACC.CHILDID_SELF);
+					accessibleObject.parent = parent.getAccessibleObject();
+				}
+
 			}
 		}
 		return accessibleObject;
@@ -488,12 +508,12 @@ public class Accessible {
 	long /*int*/ getControlHandle () {
 		long /*int*/ result = control.handle;
 		if (control instanceof Label) {
-			long /*int*/ list = OS.gtk_container_get_children (result);
+			long /*int*/ list = GTK.gtk_container_get_children (result);
 			if (list != 0) {
 				long /*int*/ temp = list;
 				while (temp != 0) {
 					long /*int*/ widget = OS.g_list_data( temp);
-					if (OS.gtk_widget_get_visible (widget)) {
+					if (GTK.gtk_widget_get_visible (widget)) {
 						result = widget;
 						break;
 					}
@@ -518,7 +538,7 @@ public class Accessible {
 	 * @noreference This method is not intended to be referenced by clients.
 	 */
 	public void internal_dispose_Accessible() {
-		AccessibleFactory.unregisterAccessible (Accessible.this);
+		if (!GTK.GTK3) AccessibleFactory.unregisterAccessible (Accessible.this);
 		release ();
 	}
 
