@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2016 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2017 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -12,10 +15,11 @@ package org.eclipse.swt.accessibility;
 
 
 import java.util.*;
+
+import org.eclipse.swt.*;
 import org.eclipse.swt.internal.*;
 import org.eclipse.swt.internal.accessibility.gtk.*;
 import org.eclipse.swt.internal.gtk.*;
-import org.eclipse.swt.*;
 
 class AccessibleFactory {
 	static final Map<LONG, Accessible> Accessibles = new HashMap<> (9);
@@ -303,10 +307,10 @@ class AccessibleFactory {
 
 	static String getTypeName (long /*int*/ type) {
 		long /*int*/ typeName = OS.g_type_name (type);
-		int widgetTypeNameLength = OS.strlen (typeName);
+		int widgetTypeNameLength = C.strlen (typeName);
 		byte[] buffer = new byte [widgetTypeNameLength];
-		OS.memmove (buffer, typeName, widgetTypeNameLength);
-		return new String(Converter.mbcsToWcs(null, buffer));
+		C.memmove (buffer, typeName, widgetTypeNameLength);
+		return new String(Converter.mbcsToWcs(buffer));
 	}
 
 	static long /*int*/ getParentType (long /*int*/ widgetType) {
@@ -331,26 +335,26 @@ class AccessibleFactory {
 		}
 		/* if an atk object has already been created for this widget then just return it */
 		if (accessible.accessibleObject != null) {
-			return accessible.accessibleObject.handle;
+			return accessible.accessibleObject.atkHandle;
 		}
 		long /*int*/ widgetType = OS.G_OBJECT_TYPE (widget);
 		long /*int*/ parentType = getParentType (widgetType);
-		if (parentType == 0) parentType = ATK.GTK_TYPE_ACCESSIBLE();
+		if (parentType == 0) parentType = GTK.GTK_TYPE_ACCESSIBLE();
 		long /*int*/ type = getType (getTypeName(widgetType), accessible, parentType, ACC.CHILDID_SELF);
 		AccessibleObject object = new AccessibleObject (type, widget, accessible, false);
 		accessible.accessibleObject = object;
 		accessible.addRelations ();
-		return object.handle;
+		return object.atkHandle;
 	}
 
 	static AccessibleObject createChildAccessible (Accessible accessible, int childId) {
-		long /*int*/ childType = getType (CHILD_TYPENAME, accessible, ATK.GTK_TYPE_ACCESSIBLE(), childId);
+		long /*int*/ childType = getType (CHILD_TYPENAME, accessible, GTK.GTK_TYPE_ACCESSIBLE(), childId);
 		return new AccessibleObject(childType, 0, accessible, true);
 	}
 
 	static void createAccessible (Accessible accessible) {
 		long /*int*/ controlHandle = accessible.getControlHandle ();
-		OS.gtk_widget_get_accessible(controlHandle);
+		GTK.gtk_widget_get_accessible(controlHandle);
 	}
 
 	static long /*int*/ getType (String widgetTypeName, Accessible accessible, long /*int*/ parentType, int childId) {
@@ -417,7 +421,7 @@ class AccessibleFactory {
 		if (text) swtTypeName += "Text"; //$NON-NLS-1$
 		if (value) swtTypeName += "Value"; //$NON-NLS-1$
 
-		byte[] nameBytes = Converter.wcsToMbcs(null, swtTypeName, true);
+		byte[] nameBytes = Converter.wcsToMbcs(swtTypeName, true);
 		long /*int*/ type = OS.g_type_from_name(nameBytes);
 		if (type == 0) {
 			if (AccessibleObject.DEBUG) AccessibleObject.print("-->New Type=" + swtTypeName); //$NON-NLS-1$
@@ -605,7 +609,7 @@ class AccessibleFactory {
 		/* If NO_OP factory is registered then OS accessibility is not active */
 		if (ATK.ATK_IS_NO_OP_OBJECT_FACTORY(factory)) return;
 		String name = FACTORY_TYPENAME + getTypeName(widgetType);
-		byte[] factoryName = Converter.wcsToMbcs(null, name, true);
+		byte[] factoryName = Converter.wcsToMbcs(name, true);
 		if (OS.g_type_from_name (factoryName) == 0) {
 			if (AccessibleObject.DEBUG) AccessibleObject.print("-->New Factory=" + name); //$NON-NLS-1$
 			/* register the factory */
