@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2003, 2012 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2003, 2018 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -23,12 +26,12 @@ import org.eclipse.swt.widgets.*;
  * </p>
  * <dl>
  * <dt><b>Styles:</b></dt>
- * <dd>MOZILLA, WEBKIT</dd>
+ * <dd>NONE, WEBKIT</dd>
  * <dt><b>Events:</b></dt>
  * <dd>CloseWindowListener, LocationListener, OpenWindowListener, ProgressListener, StatusTextListener, TitleListener, VisibilityWindowListener</dd>
  * </dl>
  * <p>
- * Note: At most one of the styles MOZILLA and WEBKIT may be specified.
+ * Note: MOZILLA is deprecated and is no longer supported.
  * </p>
  * <p>
  * IMPORTANT: This class is <em>not</em> intended to be subclassed.
@@ -100,7 +103,29 @@ public Browser (Composite parent, int style) {
 		return;
 	}
 	dispose ();
-	SWT.error (SWT.ERROR_NO_HANDLES);
+
+	String errMsg = " because no underlying browser available.\n";
+	switch (SWT.getPlatform()) {
+	case "gtk":
+		String gtkVersion = System.getProperty("org.eclipse.swt.internal.gtk.version");
+		boolean isGtk2 = Integer.parseInt(gtkVersion.substring(0,1)) == 2 ? true : false;
+		if (isGtk2) {
+			errMsg = errMsg + "   SWT on GTK 2.x detected. It is reccomended to use SWT on GTK 3.x and Webkit2 API.\n";
+		} else { // Gtk3 & above.
+			errMsg = errMsg + "   Please ensure Webkit with its Gtk 3.x bindings installed. Webkit2 API level preferred.\n";
+		}
+		break;
+	case "cocoa":
+		errMsg = errMsg + " SWT failed to load webkit library.\n";
+		break;
+	case "win32":
+		errMsg = errMsg + " SWT uses either IE or Webkit. Either SWT.WEBKIT flag is passed and Webkit library was not "
+				+ "loaded properly by SWT or SWT failed to load IE.\n";
+		break;
+	default:
+		break;
+	}
+	SWT.error (SWT.ERROR_NO_HANDLES, null, errMsg);
 }
 
 static Composite checkParent (Composite parent) {
@@ -124,18 +149,19 @@ static Composite checkParent (Composite parent) {
 	return parent;
 }
 
+@SuppressWarnings("deprecation")
 static int checkStyle(int style) {
 	String platform = SWT.getPlatform ();
 	if (DefaultType == SWT.DEFAULT) {
 		/*
-		* Some Browser clients that explicitly specify the native renderer to use
-		* (by creating a Browser with style SWT.MOZILLA or SWT.WEBKIT) may also
-		* need to specify that all "default" Browser instances (those created with
-		* style SWT.NONE) should use this renderer as well.  This may be needed in
-		* order to avoid incompatibilities that can arise from having multiple
-		* native renderers loaded within the same process.  A client can do this by
-		* setting the "org.eclipse.swt.browser.DefaultType" java system property to
-		* a value like "mozilla" or "webkit".
+		* Some Browser clients that explicitly specify the native renderer to use (by
+		* creating a Browser with SWT.WEBKIT) may also need to specify that all
+		* "default" Browser instances (those created with style SWT.NONE) should use
+		* this renderer as well. This may be needed in order to avoid incompatibilities
+		* that can arise from having multiple native renderers loaded within the same
+		* process. A client can do this by setting the
+		* "org.eclipse.swt.browser.DefaultType" java system property to a value like
+		* "ie" or "webkit". Value "mozilla" is ignored now.
 		*/
 
 		/*
@@ -163,10 +189,7 @@ static int checkStyle(int style) {
 					newIndex = length;
 				}
 				String current = value.substring(index, newIndex).trim();
-				if (current.equalsIgnoreCase ("mozilla")) { //$NON-NLS-1$
-					DefaultType = SWT.MOZILLA;
-					break;
-				} else if (current.equalsIgnoreCase ("webkit")) { //$NON-NLS-1$
+				if (current.equalsIgnoreCase ("webkit")) { //$NON-NLS-1$
 					DefaultType = SWT.WEBKIT;
 					break;
 				} else if (current.equalsIgnoreCase ("ie") && "win32".equals (platform)) { //$NON-NLS-1$ //$NON-NLS-2$
@@ -181,17 +204,20 @@ static int checkStyle(int style) {
 		}
 	}
 
-	if ((style & (SWT.MOZILLA | SWT.WEBKIT)) == 0) {
+	/* remove SWT.MOZILLA style if specified */
+	if ((style & SWT.MOZILLA) != 0) {
+		System.err.println ("Unsupported Browser Type: SWT.MOZILLA style is deprecated.\n" //$NON-NLS-1$
+				+ "It'll be removed from the user specified style. Browser will be created with the modified style " //$NON-NLS-1$
+				+ "and if no other style bit is specified, browser with SWT.NONE style will be created"); //$NON-NLS-1$
+		style &= ~SWT.MOZILLA;
+	}
+
+	if ((style & SWT.WEBKIT) == 0) {
 		style |= DefaultType;
 	}
-
-	if ((style & (SWT.MOZILLA | SWT.WEBKIT)) == (SWT.MOZILLA | SWT.WEBKIT)) {
-		style &= ~SWT.WEBKIT;
-	}
-	if ((style & SWT.MOZILLA) != 0 || (style & SWT.WEBKIT) != 0) {
+	if ((style & SWT.WEBKIT) != 0) {
 		return style;
 	}
-
 	if ("win32".equals (platform)) { //$NON-NLS-1$
 		/*
 		* For IE on win32 the border is supplied by the embedded browser, so remove
@@ -731,7 +757,12 @@ public int getStyle () {
  * Returns a string with HTML that represents the content of the current page.
  *
  * @return HTML representing the current page or an empty <code>String</code>
- * if this is empty
+ * if this is empty.<br>
+ * <p> Note, the exact return value is platform dependent.
+ * For example on Windows, the returned string is the proccessed webpage
+ * with javascript executed and missing html tags added.
+ * On Linux and OS X, this returns the original HTML before the browser has
+ * processed it.</p>
  *
  * @exception SWTException <ul>
  *    <li>ERROR_THREAD_INVALID_ACCESS when called from the wrong thread</li>
@@ -776,7 +807,9 @@ public String getUrl () {
  * @return the receiver's JavaXPCOM <code>nsIWebBrowser</code> or <code>null</code>
  *
  * @since 3.3
+ * @deprecated SWT.MOZILLA is deprecated and XULRunner as a browser renderer is no longer supported.
  */
+@Deprecated
 public Object getWebBrowser () {
 	checkWidget();
 	return webBrowser.getWebBrowser ();
