@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2016 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -11,10 +14,10 @@
 package org.eclipse.swt.custom;
 
 import org.eclipse.swt.*;
-import org.eclipse.swt.widgets.*;
-import org.eclipse.swt.graphics.*;
-import org.eclipse.swt.events.*;
 import org.eclipse.swt.accessibility.*;
+import org.eclipse.swt.events.*;
+import org.eclipse.swt.graphics.*;
+import org.eclipse.swt.widgets.*;
 
 /**
  * A Label which supports aligned text and/or an image and different border styles.
@@ -116,28 +119,15 @@ public CLabel(Composite parent, int style) {
 	if ((style & SWT.RIGHT) != 0)  align = SWT.RIGHT;
 	if ((style & SWT.LEFT) != 0)   align = SWT.LEFT;
 
-	addPaintListener(new PaintListener() {
-		@Override
-		public void paintControl(PaintEvent event) {
-			onPaint(event);
+	addPaintListener(event -> onPaint(event));
+
+	addTraverseListener(event -> {
+		if (event.detail == SWT.TRAVERSE_MNEMONIC) {
+			onMnemonic(event);
 		}
 	});
 
-	addTraverseListener(new TraverseListener() {
-		@Override
-		public void keyTraversed(TraverseEvent event) {
-			if (event.detail == SWT.TRAVERSE_MNEMONIC) {
-				onMnemonic(event);
-			}
-		}
-	});
-
-	addListener(SWT.Dispose, new Listener() {
-		@Override
-		public void handleEvent(Event event) {
-			onDispose(event);
-		}
-	});
+	addListener(SWT.Dispose, event -> onDispose(event));
 
 	initAccessible();
 
@@ -204,6 +194,10 @@ char _findMnemonic (String string) {
  * @return SWT.LEFT, SWT.RIGHT or SWT.CENTER
  */
 public int getAlignment() {
+	/*
+	 * This call is intentionally commented out, to allow this getter method to be
+	 * called from a thread which is different from one that created the widget.
+	 */
 	//checkWidget();
 	return align;
 }
@@ -215,6 +209,10 @@ public int getAlignment() {
  * @since 3.6
  */
 public int getBottomMargin() {
+	/*
+	 * This call is intentionally commented out, to allow this getter method to be
+	 * called from a thread which is different from one that created the widget.
+	 */
 	//checkWidget();
 	return bottomMargin;
 }
@@ -224,6 +222,10 @@ public int getBottomMargin() {
  * @return the image of the label or null
  */
 public Image getImage() {
+	/*
+	 * This call is intentionally commented out, to allow this getter method to be
+	 * called from a thread which is different from one that created the widget.
+	 */
 	//checkWidget();
 	return image;
 }
@@ -235,6 +237,10 @@ public Image getImage() {
  * @since 3.6
  */
 public int getLeftMargin() {
+	/*
+	 * This call is intentionally commented out, to allow this getter method to be
+	 * called from a thread which is different from one that created the widget.
+	 */
 	//checkWidget();
 	return leftMargin;
 }
@@ -246,6 +252,10 @@ public int getLeftMargin() {
  * @since 3.6
  */
 public int getRightMargin() {
+	/*
+	 * This call is intentionally commented out, to allow this getter method to be
+	 * called from a thread which is different from one that created the widget.
+	 */
 	//checkWidget();
 	return rightMargin;
 }
@@ -291,6 +301,10 @@ public int getStyle () {
  * @return the text of the label or null
  */
 public String getText() {
+	/*
+	 * This call is intentionally commented out, to allow this getter method to be
+	 * called from a thread which is different from one that created the widget.
+	 */
 	//checkWidget();
 	return text;
 }
@@ -307,6 +321,10 @@ public String getToolTipText () {
  * @since 3.6
  */
 public int getTopMargin() {
+	/*
+	 * This call is intentionally commented out, to allow this getter method to be
+	 * called from a thread which is different from one that created the widget.
+	 */
 	//checkWidget();
 	return topMargin;
 }
