@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2016 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -99,15 +102,12 @@ public class TableTree extends Composite {
 public TableTree(Composite parent, int style) {
 	super(parent, checkStyle (style));
 	table = new Table(this, style);
-	Listener tableListener = new Listener() {
-		@Override
-		public void handleEvent(Event e) {
-			switch (e.type) {
-			case SWT.MouseDown: onMouseDown(e); break;
-			case SWT.Selection: onSelection(e); break;
-			case SWT.DefaultSelection: onSelection(e); break;
-			case SWT.KeyDown: onKeyDown(e); break;
-			}
+	Listener tableListener = e -> {
+		switch (e.type) {
+		case SWT.MouseDown: onMouseDown(e); break;
+		case SWT.Selection: onSelection(e); break;
+		case SWT.DefaultSelection: onSelection(e); break;
+		case SWT.KeyDown: onKeyDown(e); break;
 		}
 	};
 	int[] tableEvents = new int[]{SWT.MouseDown,
@@ -118,14 +118,11 @@ public TableTree(Composite parent, int style) {
 		table.addListener(tableEvents[i], tableListener);
 	}
 
-	listener = new Listener() {
-		@Override
-		public void handleEvent(Event e) {
-			switch (e.type) {
-			case SWT.Dispose: onDispose(e); break;
-			case SWT.Resize:  onResize(e); break;
-			case SWT.FocusIn: onFocusIn(e); break;
-			}
+	listener = e -> {
+		switch (e.type) {
+		case SWT.Dispose: onDispose(e); break;
+		case SWT.Resize:  onResize(e); break;
+		case SWT.FocusIn: onFocusIn(e); break;
 		}
 	};
 	int[] events = new int[]{SWT.Dispose,
@@ -392,7 +389,7 @@ void createImages () {
 	Color background = getBackground();
 
 	/* Plus image */
-	PaletteData palette = new PaletteData(new RGB[]{foreground.getRGB(), background.getRGB(), plusMinus.getRGB()});
+	PaletteData palette = new PaletteData(foreground.getRGB(), background.getRGB(), plusMinus.getRGB());
 	ImageData imageData = new ImageData(itemHeight, itemHeight, 4, palette);
 	imageData.transparentPixel = 1;
 	plusImage = new Image(getDisplay(), imageData);
@@ -407,7 +404,7 @@ void createImages () {
 	gc.dispose();
 
 	/* Minus image */
-	palette = new PaletteData(new RGB[]{foreground.getRGB(), background.getRGB(), plusMinus.getRGB()});
+	palette = new PaletteData(foreground.getRGB(), background.getRGB(), plusMinus.getRGB());
 	imageData = new ImageData(itemHeight, itemHeight, 4, palette);
 	imageData.transparentPixel = 1;
 	minusImage = new Image(getDisplay(), imageData);
