@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2011 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2016 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -11,7 +14,7 @@
 package org.eclipse.swt.custom;
 
 import org.eclipse.swt.events.*;
-import org.eclipse.swt.internal.SWTEventListener;
+import org.eclipse.swt.internal.*;
 
 /**
  * Classes which implement this interface provide a method
@@ -21,6 +24,7 @@ import org.eclipse.swt.internal.SWTEventListener;
  * @see VerifyEvent
  * @see <a href="http://www.eclipse.org/swt/">Sample code and further information</a>
  */
+@FunctionalInterface
 public interface VerifyKeyListener extends SWTEventListener {
 /**
  * The following event fields are used:<ul>
