@@ -1,12 +1,16 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2016 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2017 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
+ *     Lars Vogel <Lars.Vogel@vogella.com> - Bug 497807
  *******************************************************************************/
 package org.eclipse.swt.dnd;
 
@@ -160,30 +164,22 @@ public DragSource(Control control, int style) {
 	createCOMInterfaces();
 	this.AddRef();
 
-	controlListener = new Listener() {
-		@Override
-		public void handleEvent(Event event) {
-			if (event.type == SWT.Dispose) {
-				if (!DragSource.this.isDisposed()) {
-					DragSource.this.dispose();
-				}
+	controlListener = event -> {
+		if (event.type == SWT.Dispose) {
+			if (!DragSource.this.isDisposed()) {
+				DragSource.this.dispose();
 			}
-			if (event.type == SWT.DragDetect) {
-				if (!DragSource.this.isDisposed()) {
-					DragSource.this.drag(event);
-				}
+		}
+		if (event.type == SWT.DragDetect) {
+			if (!DragSource.this.isDisposed()) {
+				DragSource.this.drag(event);
 			}
 		}
 	};
 	control.addListener(SWT.Dispose, controlListener);
 	control.addListener(SWT.DragDetect, controlListener);
 
-	this.addListener(SWT.Dispose, new Listener() {
-		@Override
-		public void handleEvent(Event e) {
-			DragSource.this.onDispose();
-		}
-	});
+	this.addListener(SWT.Dispose, e -> DragSource.this.onDispose());
 
 	Object effect = control.getData(DEFAULT_DRAG_SOURCE_EFFECT);
 	if (effect instanceof DragSourceEffect) {
@@ -359,12 +355,8 @@ private void drag(Event dragEvent) {
         * The fix is to force all paints to be delivered before
         * calling ImageList_DragEnter().
         */
-		if (OS.IsWinCE) {
-			OS.UpdateWindow (topControl.handle);
-		} else {
-			int flags = OS.RDW_UPDATENOW | OS.RDW_ALLCHILDREN;
-			OS.RedrawWindow (topControl.handle, null, 0, flags);
-		}
+		int flags = OS.RDW_UPDATENOW | OS.RDW_ALLCHILDREN;
+		OS.RedrawWindow (topControl.handle, null, 0, flags);
 		POINT pt = new POINT ();
 		pt.x = DPIUtil.autoScaleUp(dragEvent.x);// To Pixels
 		pt.y = DPIUtil.autoScaleUp(dragEvent.y);// To Pixels
@@ -435,7 +427,7 @@ private int EnumFormatEtc(int dwDirection, long /*int*/ ppenumFormatetc) {
 	}
 	enumFORMATETC.setFormats(formats);
 
-	OS.MoveMemory(ppenumFormatetc, new long /*int*/[] {enumFORMATETC.getAddress()}, OS.PTR_SIZEOF);
+	OS.MoveMemory(ppenumFormatetc, new long /*int*/[] {enumFORMATETC.getAddress()}, C.PTR_SIZEOF);
 	return COM.S_OK;
 }
 /**
@@ -651,18 +643,18 @@ private int QueryInterface(long /*int*/ riid, long /*int*/ ppvObject) {
 	COM.MoveMemory(guid, riid, GUID.sizeof);
 
 	if (COM.IsEqualGUID(guid, COM.IIDIUnknown) || COM.IsEqualGUID(guid, COM.IIDIDropSource)) {
-		OS.MoveMemory(ppvObject, new long /*int*/[] {iDropSource.getAddress()}, OS.PTR_SIZEOF);
+		OS.MoveMemory(ppvObject, new long /*int*/[] {iDropSource.getAddress()}, C.PTR_SIZEOF);
 		AddRef();
 		return COM.S_OK;
 	}
 
 	if (COM.IsEqualGUID(guid, COM.IIDIDataObject) ) {
-		OS.MoveMemory(ppvObject, new long /*int*/[] {iDataObject.getAddress()}, OS.PTR_SIZEOF);
+		OS.MoveMemory(ppvObject, new long /*int*/[] {iDataObject.getAddress()}, C.PTR_SIZEOF);
 		AddRef();
 		return COM.S_OK;
 	}
 
-	OS.MoveMemory(ppvObject, new long /*int*/[] {0}, OS.PTR_SIZEOF);
+	OS.MoveMemory(ppvObject, new long /*int*/[] {0}, C.PTR_SIZEOF);
 	return COM.E_NOINTERFACE;
 }
 
@@ -711,7 +703,7 @@ private int SetData(long /*int*/ pFormatetc, long /*int*/ pmedium, int fRelease)
 		COM.MoveMemory(stgmedium, pmedium,STGMEDIUM.sizeof);
 		//TODO - this should be GlobalLock()
 		long /*int*/[] ptrEffect = new long /*int*/[1];
-		OS.MoveMemory(ptrEffect, stgmedium.unionField, OS.PTR_SIZEOF);
+		OS.MoveMemory(ptrEffect, stgmedium.unionField, C.PTR_SIZEOF);
 		int[] effect = new int[1];
 		OS.MoveMemory(effect, ptrEffect[0], 4);
 		dataEffect = osToOp(effect[0]);
@@ -742,7 +734,7 @@ public void setDragSourceEffect(DragSourceEffect effect) {
  * @param transferAgents a list of Transfer objects which define the types of data that can be
  * dragged from this source
  */
-public void setTransfer(Transfer[] transferAgents){
+public void setTransfer(Transfer... transferAgents){
 	this.transferAgents = transferAgents;
 }
 
