@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2016 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2017 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -143,22 +146,14 @@ public DropTarget(Control control, int style) {
 	if (COM.RegisterDragDrop( control.handle, iDropTarget.getAddress()) != COM.S_OK)
 		DND.error(DND.ERROR_CANNOT_INIT_DROP);
 
-	controlListener = new Listener () {
-		@Override
-		public void handleEvent (Event event) {
-			if (!DropTarget.this.isDisposed()){
-				DropTarget.this.dispose();
-			}
+	controlListener = event -> {
+		if (!DropTarget.this.isDisposed()){
+			DropTarget.this.dispose();
 		}
 	};
 	control.addListener (SWT.Dispose, controlListener);
 
-	this.addListener(SWT.Dispose, new Listener () {
-		@Override
-		public void handleEvent (Event event) {
-			onDispose();
-		}
-	});
+	this.addListener(SWT.Dispose, event -> onDispose());
 
 	Object effect = control.getData(DEFAULT_DROP_TARGET_EFFECT);
 	if (effect instanceof DropTargetEffect) {
@@ -631,12 +626,12 @@ int QueryInterface(long /*int*/ riid, long /*int*/ ppvObject) {
 	GUID guid = new GUID();
 	COM.MoveMemory(guid, riid, GUID.sizeof);
 	if (COM.IsEqualGUID(guid, COM.IIDIUnknown) || COM.IsEqualGUID(guid, COM.IIDIDropTarget)) {
-        COM.MoveMemory(ppvObject, new long /*int*/[] {iDropTarget.getAddress()}, OS.PTR_SIZEOF);
+        OS.MoveMemory(ppvObject, new long /*int*/[] {iDropTarget.getAddress()}, C.PTR_SIZEOF);
 		AddRef();
 		return COM.S_OK;
 	}
 
-    COM.MoveMemory(ppvObject, new long /*int*/[] {0}, OS.PTR_SIZEOF);
+    OS.MoveMemory(ppvObject, new long /*int*/[] {0}, C.PTR_SIZEOF);
 	return COM.E_NOINTERFACE;
 }
 
@@ -798,7 +793,7 @@ boolean setEventData(DNDEvent event, long /*int*/ pDataObject, int grfKeyState,
  *    <li>ERROR_NULL_ARGUMENT - if transferAgents is null</li>
  * </ul>
  */
-public void setTransfer(Transfer[] transferAgents){
+public void setTransfer(Transfer... transferAgents){
 	if (transferAgents == null) DND.error(SWT.ERROR_NULL_ARGUMENT);
 	this.transferAgents = transferAgents;
 }
