@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2012 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2017 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    IBM Corporation - initial API and implementation
@@ -29,8 +32,8 @@ public NSString badgeLabel() {
 	return result != 0 ? new NSString(result) : null;
 }
 
-public void setBadgeLabel(NSString string) {
-	OS.objc_msgSend(this.id, OS.sel_setBadgeLabel_, string != null ? string.id : 0);
+public void setBadgeLabel(NSString badgeLabel) {
+	OS.objc_msgSend(this.id, OS.sel_setBadgeLabel_, badgeLabel != null ? badgeLabel.id : 0);
 }
 
 }
