@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2012 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2017 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    IBM Corporation - initial API and implementation
@@ -36,12 +39,12 @@ public double minValue() {
 	return OS.objc_msgSend_fpret(this.id, OS.sel_minValue);
 }
 
-public void setMaxValue(double aDouble) {
-	OS.objc_msgSend(this.id, OS.sel_setMaxValue_, aDouble);
+public void setMaxValue(double maxValue) {
+	OS.objc_msgSend(this.id, OS.sel_setMaxValue_, maxValue);
 }
 
-public void setMinValue(double aDouble) {
-	OS.objc_msgSend(this.id, OS.sel_setMinValue_, aDouble);
+public void setMinValue(double minValue) {
+	OS.objc_msgSend(this.id, OS.sel_setMinValue_, minValue);
 }
 
 public static long /*int*/ cellClass() {
