@@ -1,21 +1,26 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2012 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2017 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.swt.internal.ole.win32;
 
+import org.eclipse.swt.internal.win32.*;
+
 public class IProvideClassInfo extends IUnknown
 {
 public IProvideClassInfo(long /*int*/ address) {
 	super(address);
 }
 public int GetClassInfo(long /*int*/[] ppTI) {
-	return COM.VtblCall(3, address, ppTI);
+	return OS.VtblCall(3, address, ppTI);
 }
 }
