@@ -1,16 +1,19 @@
 /*******************************************************************************
- * Copyright (c) 2011, 2016 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2011, 2017 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.swt.browser;
 
-import java.io.*;
+import java.nio.charset.*;
 import java.util.*;
 
 import org.eclipse.swt.*;
@@ -57,6 +60,7 @@ class WebKit extends WebBrowser {
 	static final String ADD_WIDGET_KEY = "org.eclipse.swt.internal.addWidget"; //$NON-NLS-1$
 	static final String WEBKIT_EVENTS_FIX_KEY = "org.eclipse.swt.internal.webKitEventsFix"; //$NON-NLS-1$
 	static final byte[] SWT_OBJECT = {'S', 'W', 'T', '_', 'O', 'B', 'J', 'E', 'C', 'T', '\0'};
+	static final long DEFAULT_DB_QUOTA = 5 * 1024 * 1024;
 
 	/* event strings */
 	static final String DOMEVENT_KEYUP = "keyup"; //$NON-NLS-1$
@@ -67,54 +71,45 @@ class WebKit extends WebBrowser {
 	static final String DOMEVENT_MOUSEWHEEL = "mousewheel"; //$NON-NLS-1$
 
 	static {
-		NativeClearSessions = new Runnable() {
-			@Override
-			public void run() {
-				NSHTTPCookieStorage storage = NSHTTPCookieStorage.sharedHTTPCookieStorage();
-				NSArray cookies = storage.cookies();
-				int count = (int)/*64*/cookies.count ();
-				for (int i = 0; i < count; i++) {
-					NSHTTPCookie cookie = new NSHTTPCookie(cookies.objectAtIndex(i));
-					if (cookie.isSessionOnly()) {
-						storage.deleteCookie(cookie);
-					}
+		NativeClearSessions = () -> {
+			NSHTTPCookieStorage storage = NSHTTPCookieStorage.sharedHTTPCookieStorage();
+			NSArray cookies = storage.cookies();
+			int count = (int)/*64*/cookies.count ();
+			for (int i = 0; i < count; i++) {
+				NSHTTPCookie cookie = new NSHTTPCookie(cookies.objectAtIndex(i));
+				if (cookie.isSessionOnly()) {
+					storage.deleteCookie(cookie);
 				}
 			}
 		};
 
-		NativeGetCookie = new Runnable () {
-			@Override
-			public void run () {
-				NSHTTPCookieStorage storage = NSHTTPCookieStorage.sharedHTTPCookieStorage ();
-				NSURL url = NSURL.URLWithString (NSString.stringWith (CookieUrl));
-				NSArray cookies = storage.cookiesForURL (url);
-				int count = (int)/*64*/cookies.count ();
-				if (count == 0) return;
+		NativeGetCookie = () -> {
+			NSHTTPCookieStorage storage = NSHTTPCookieStorage.sharedHTTPCookieStorage ();
+			NSURL url = NSURL.URLWithString (NSString.stringWith (CookieUrl));
+			NSArray cookies = storage.cookiesForURL (url);
+			int count = (int)/*64*/cookies.count ();
+			if (count == 0) return;
 
-				NSString name = NSString.stringWith (CookieName);
-				for (int i = 0; i < count; i++) {
-					NSHTTPCookie current = new NSHTTPCookie (cookies.objectAtIndex (i));
-					if (current.name ().compare (name) == OS.NSOrderedSame) {
-						CookieValue = current.value ().getString ();
-						return;
-					}
+			NSString name = NSString.stringWith (CookieName);
+			for (int i = 0; i < count; i++) {
+				NSHTTPCookie current = new NSHTTPCookie (cookies.objectAtIndex (i));
+				if (current.name ().compare (name) == OS.NSOrderedSame) {
+					CookieValue = current.value ().getString ();
+					return;
 				}
 			}
 		};
 
-		NativeSetCookie = new Runnable () {
-			@Override
-			public void run () {
-				NSURL url = NSURL.URLWithString (NSString.stringWith (CookieUrl));
-				NSMutableDictionary headers = NSMutableDictionary.dictionaryWithCapacity (1);
-				headers.setValue (NSString.stringWith (CookieValue), NSString.stringWith (HEADER_SETCOOKIE));
-				NSArray cookies = NSHTTPCookie.cookiesWithResponseHeaderFields (headers, url);
-				if (cookies.count () == 0) return;
-				NSHTTPCookieStorage storage = NSHTTPCookieStorage.sharedHTTPCookieStorage ();
-				NSHTTPCookie cookie = new NSHTTPCookie (cookies.objectAtIndex (0));
-				storage.setCookie (cookie);
-				CookieResult = true;
-			}
+		NativeSetCookie = () -> {
+			NSURL url = NSURL.URLWithString (NSString.stringWith (CookieUrl));
+			NSMutableDictionary headers = NSMutableDictionary.dictionaryWithCapacity (1);
+			headers.setValue (NSString.stringWith (CookieValue), NSString.stringWith (HEADER_SETCOOKIE));
+			NSArray cookies = NSHTTPCookie.cookiesWithResponseHeaderFields (headers, url);
+			if (cookies.count () == 0) return;
+			NSHTTPCookieStorage storage = NSHTTPCookieStorage.sharedHTTPCookieStorage ();
+			NSHTTPCookie cookie = new NSHTTPCookie (cookies.objectAtIndex (0));
+			storage.setCookie (cookie);
+			CookieResult = true;
 		};
 
 		if (NativePendingCookies != null) {
@@ -160,6 +155,7 @@ public void create (Composite parent, int style) {
 		OS.class_addMethod(delegateClass, OS.sel_webView_resource_didFinishLoadingFromDataSource_, proc5, "@:@@@"); //$NON-NLS-1$
 		OS.class_addMethod(delegateClass, OS.sel_webView_resource_didReceiveAuthenticationChallenge_fromDataSource_, proc6, "@:@@@@"); //$NON-NLS-1$
 		OS.class_addMethod(delegateClass, OS.sel_webView_resource_didFailLoadingWithError_fromDataSource_, proc6, "@:@@@@"); //$NON-NLS-1$
+		OS.class_addMethod(delegateClass, OS.sel_webView_frame_exceededDatabaseQuotaForSecurityOrigin_database_, proc6, "@:@@@@" ); //$NON-NLS-1$
 		OS.class_addMethod(delegateClass, OS.sel_webView_identifierForInitialRequest_fromDataSource_, proc5, "@:@@@"); //$NON-NLS-1$
 		OS.class_addMethod(delegateClass, OS.sel_webView_resource_willSendRequest_redirectResponse_fromDataSource_, proc7, "@:@@@@@"); //$NON-NLS-1$
 		OS.class_addMethod(delegateClass, OS.sel_webView_createWebViewWithRequest_, proc4, "@:@@"); //$NON-NLS-1$
@@ -218,56 +214,53 @@ public void create (Composite parent, int style) {
 	this.webView = webView;
 	browser.view.addSubview(webView);
 
-	Listener listener = new Listener() {
-		@Override
-		public void handleEvent(Event e) {
-			switch (e.type) {
-				case SWT.FocusIn:
-					WebKit.this.webView.window().makeFirstResponder(WebKit.this.webView);
-					break;
-				case SWT.Dispose: {
-					/* make this handler run after other dispose listeners */
-					if (ignoreDispose) {
-						ignoreDispose = false;
-						break;
-					}
-					ignoreDispose = true;
-					browser.notifyListeners (e.type, e);
-					e.type = SWT.NONE;
-
-					/* Browser could have been disposed by one of the Dispose listeners */
-					if (!browser.isDisposed()) {
-						/* invoke onbeforeunload handlers */
-						if (!browser.isClosing) {
-							close (false);
-						}
-
-						e.display.setData(ADD_WIDGET_KEY, new Object[] {delegate, null});
-					}
-
-					WebKit.this.webView.setFrameLoadDelegate(null);
-					WebKit.this.webView.setResourceLoadDelegate(null);
-					WebKit.this.webView.setUIDelegate(null);
-					WebKit.this.webView.setPolicyDelegate(null);
-					WebKit.this.webView.setDownloadDelegate(null);
-
-					WebKit.this.webView.release();
-					WebKit.this.webView = null;
-					WebKit.this.delegate.release();
-					WebKit.this.delegate = null;
-					html = null;
-					lastHoveredLinkURL = lastNavigateURL = null;
-
-					Iterator<BrowserFunction> elements = functions.values().iterator ();
-					while (elements.hasNext ()) {
-						elements.next ().dispose (false);
-					}
-					functions = null;
-
-					if (preferences != null) preferences.release ();
-					preferences = null;
+	Listener listener = e -> {
+		switch (e.type) {
+			case SWT.FocusIn:
+				WebKit.this.webView.window().makeFirstResponder(WebKit.this.webView);
+				break;
+			case SWT.Dispose: {
+				/* make this handler run after other dispose listeners */
+				if (ignoreDispose) {
+					ignoreDispose = false;
 					break;
 				}
+				ignoreDispose = true;
+				browser.notifyListeners (e.type, e);
+				e.type = SWT.NONE;
+
+				/* Browser could have been disposed by one of the Dispose listeners */
+				if (!browser.isDisposed()) {
+					/* invoke onbeforeunload handlers */
+					if (!browser.isClosing) {
+						close (false);
+					}
+
+					e.display.setData(ADD_WIDGET_KEY, new Object[] {delegate, null});
+				}
+
+				WebKit.this.webView.setFrameLoadDelegate(null);
+				WebKit.this.webView.setResourceLoadDelegate(null);
+				WebKit.this.webView.setUIDelegate(null);
+				WebKit.this.webView.setPolicyDelegate(null);
+				WebKit.this.webView.setDownloadDelegate(null);
+
+				WebKit.this.webView.release();
+				WebKit.this.webView = null;
+				WebKit.this.delegate.release();
+				WebKit.this.delegate = null;
+				html = null;
+				lastHoveredLinkURL = lastNavigateURL = null;
+
+				Iterator<BrowserFunction> elements = functions.values().iterator ();
+				while (elements.hasNext ()) {
+					elements.next ().dispose (false);
+				}
+				functions = null;
+
+				if (preferences != null) preferences.release ();
+				preferences = null;
+				break;
 			}
 		}
 	};
@@ -412,6 +405,8 @@ static long /*int*/ browserProc(long /*int*/ id, long /*int*/ sel, long /*int*/
 	} else if (sel == OS.sel_callJava) {
 		id result = webKit.callJava(arg0, arg1, arg2, arg3);
 		return result == null ? 0 : result.id;
+	} else if (sel == OS.sel_webView_frame_exceededDatabaseQuotaForSecurityOrigin_database_){
+		webView_frame_exceededDatabaseQuotaForSecurityOrigin_database(arg0, arg1, arg2, arg3);
 	}
 	return 0;
 }
@@ -448,6 +443,13 @@ static long /*int*/ webScriptNameForSelector (long /*int*/ aSelector) {
 	return 0;
 }
 
+static void webView_frame_exceededDatabaseQuotaForSecurityOrigin_database(
+		long /* int */sender, long /* int */frame, long /* int */origin,
+		long /* int */database )
+{
+	OS.objc_msgSend( origin, OS.sel_setQuota, DEFAULT_DB_QUOTA );
+}
+
 @Override
 public boolean close () {
 	return close (true);
@@ -483,19 +485,10 @@ public boolean execute (String script) {
 	WebFrame frame = webView.mainFrame();
 	long /*int*/ context = frame.globalContext();
 
-	byte[] bytes = null;
-	try {
-		bytes = (script + '\0').getBytes("UTF-8"); //$NON-NLS-1$
-	} catch (UnsupportedEncodingException e) {
-		bytes = (script + '\0').getBytes();
-	}
+	byte[] bytes = (script + '\0').getBytes(StandardCharsets.UTF_8); //$NON-NLS-1$
 	long /*int*/ scriptString = OS.JSStringCreateWithUTF8CString(bytes);
 
-	try {
-		bytes = (getUrl() + '\0').getBytes("UTF-8"); //$NON-NLS-1$
-	} catch (UnsupportedEncodingException e) {
-		bytes = (getUrl() + '\0').getBytes();
-	}
+	bytes = (getUrl() + '\0').getBytes(StandardCharsets.UTF_8); //$NON-NLS-1$
 	long /*int*/ urlString = OS.JSStringCreateWithUTF8CString(bytes);
 
 	long /*int*/ result = OS.JSEvaluateScript(context, scriptString, 0, urlString, 0, null);
@@ -1134,14 +1127,11 @@ boolean showAuthenticationDialog (final String[] user, final String[] password,
 
 	final boolean[] result = new boolean[1];
 	final Button[] buttons = new Button[2];
-	Listener listener = new Listener() {
-		@Override
-		public void handleEvent(Event event) {
-			user[0] = userText.getText();
-			password[0] = passwordText.getText();
-			result[0] = event.widget == buttons[1];
-			shell.close();
-		}
+	Listener listener = event -> {
+		user[0] = userText.getText();
+		password[0] = passwordText.getText();
+		result[0] = event.widget == buttons[1];
+		shell.close();
 	};
 
 	Composite composite = new Composite (shell, SWT.NONE);
@@ -1716,7 +1706,7 @@ Object convertToJava (long /*int*/ value) {
 		NSNumber number = new NSNumber (value);
 		long /*int*/ ptr = number.objCType ();
 		byte[] type = new byte[1];
-		OS.memmove (type, ptr, 1);
+		C.memmove (type, ptr, 1);
 		if (type[0] == 'c' || type[0] == 'B') {
 			return new Boolean (number.boolValue ());
 		}
