@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2010, 2012 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2010, 2017 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -12,6 +15,7 @@ package org.eclipse.swt.internal.webkit;
 
 
 import org.eclipse.swt.internal.ole.win32.*;
+import org.eclipse.swt.internal.win32.*;
 
 public class IWebErrorPrivate extends IUnknown {
 
@@ -20,7 +24,7 @@ public IWebErrorPrivate (long /*int*/ address) {
 }
 
 public int sslPeerCertificate (long /*int*/[] result) {
-	return COM.VtblCall (3, getAddress (), result);
+	return OS.VtblCall (3, getAddress (), result);
 }
 
 }
