@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2010, 2012 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2010, 2017 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -11,6 +14,7 @@
 package org.eclipse.swt.internal.webkit;
 
 import org.eclipse.swt.internal.ole.win32.*;
+import org.eclipse.swt.internal.win32.*;
 
 public class IWebOpenPanelResultListener extends IUnknown {
 
@@ -19,11 +23,11 @@ public IWebOpenPanelResultListener (long /*int*/ address) {
 }
 
 public int chooseFilename (long /*int*/ fileName) {
-	return COM.VtblCall (3, getAddress (), fileName);
+	return OS.VtblCall (3, getAddress (), fileName);
 }
 
 public int cancel () {
-	return COM.VtblCall (4, getAddress ());
+	return OS.VtblCall (4, getAddress ());
 }
 
 }
