@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2016 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -240,7 +243,9 @@ protected void destroy () {
  */
 public Rectangle getBounds () {
 	checkDevice ();
-	NSRect frame = getPrimaryScreen().frame();
+	NSScreen primaryScreen = getPrimaryScreen();
+	if (primaryScreen == null) return new Rectangle(0, 0, 0, 0);
+	NSRect frame = primaryScreen.frame();
 	return new Rectangle((int)frame.x, (int)frame.y, (int)frame.width, (int)frame.height);
 }
 
@@ -317,13 +322,15 @@ public Rectangle getClientArea () {
  */
 public int getDepth () {
 	checkDevice ();
-	return (int)/*64*/OS.NSBitsPerPixelFromDepth(getPrimaryScreen().depth());
+	NSScreen primaryScreen = getPrimaryScreen();
+	if (primaryScreen == null) return 0;
+	return (int)/*64*/OS.NSBitsPerPixelFromDepth(primaryScreen.depth());
 }
 
 /**
- * Returns a point whose x coordinate is the horizontal
+ * Returns a point whose x coordinate is the logical horizontal
  * dots per inch of the display, and whose y coordinate
- * is the vertical dots per inch of the display.
+ * is the logical vertical dots per inch of the display.
  *
  * @return the horizontal and vertical DPI
  *
@@ -338,7 +345,7 @@ public Point getDPI () {
 
 NSScreen getPrimaryScreen () {
 	NSArray screens = NSScreen.screens();
-	return new NSScreen(screens.objectAtIndex(0));
+	return (screens != null) ? new NSScreen(screens.objectAtIndex(0)) : null;
 }
 
 /**
@@ -416,6 +423,8 @@ public FontData[] getFontList (String faceName, boolean scalable) {
 
 Point getScreenDPI () {
 	NSScreen screen = getPrimaryScreen();
+	if (screen == null) return new Point(0, 0);
+
 	NSDictionary dictionary = screen.deviceDescription();
 	NSValue value = new NSValue(dictionary.objectForKey(new id(OS.NSDeviceResolution())).id);
 	NSSize size = value.sizeValue();
@@ -542,7 +551,7 @@ protected void init () {
 	COLOR_WHITE = new Color (this, 0xFF,0xFF,0xFF);
 
 	paragraphStyle = (NSMutableParagraphStyle)new NSMutableParagraphStyle().alloc().init();
-	paragraphStyle.setAlignment(OS.NSLeftTextAlignment);
+	paragraphStyle.setAlignment(OS.NSTextAlignmentLeft);
 	paragraphStyle.setLineBreakMode(OS.NSLineBreakByClipping);
 	NSArray tabs = new NSArray(new NSArray().alloc().init());
 	paragraphStyle.setTabStops(tabs);
@@ -789,7 +798,7 @@ public void setWarnings (boolean warnings) {
  */
 protected int getDeviceZoom () {
 	NSScreen mainScreen = NSScreen.mainScreen();
-	int scaleFactor = (int) mainScreen.backingScaleFactor();
+	int scaleFactor = mainScreen != null ? (int) mainScreen.backingScaleFactor() : 1;
 	return scaleFactor * 100;
 }
 
