@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2012 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2017 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -50,7 +53,7 @@ public class Canvas extends Composite {
 		// This array is leaked.
 		supportedPboardTypes = NSMutableArray.arrayWithCapacity(1);
 		supportedPboardTypes.retain();
-		supportedPboardTypes.addObject(OS.NSStringPboardType);
+		supportedPboardTypes.addObject(OS.NSPasteboardTypeString);
 		//supportedPboardTypes.addObject(OS.NSRTFPboardType);
 	};
 
@@ -319,8 +322,8 @@ boolean readSelectionFromPasteboard(long /*int*/ id, long /*int*/ sel, long /*in
 boolean readSelectionFromPasteboard(NSPasteboard pboard, NSString type) {
     boolean result = false;
     NSString newSelection = null;
-    if (type.isEqualToString(OS.NSStringPboardType)) {
-        NSString string = pboard.stringForType(OS.NSStringPboardType);
+    if (type.isEqualToString(OS.NSPasteboardTypeString)) {
+        NSString string = pboard.stringForType(OS.NSPasteboardTypeString);
         if (string != null && string.length() > 0) {
         	newSelection = string;
         }
@@ -425,7 +428,10 @@ public void scroll (int destX, int destY, int x, int y, int width, int height, b
 		dest.y = destY;
 
 		view.lockFocus();
-		OS.NSCopyBits(0, damage , dest);
+		NSSize delta = new NSSize();
+		delta.width = deltaX;
+		delta.height = deltaY;
+		view.scrollRect(damage, delta);
 		view.unlockFocus();
 
 		boolean disjoint = (destX + width < x) || (x + width < destX) || (destY + height < y) || (y + height < destY);
@@ -677,12 +683,12 @@ boolean writeSelectionToPasteboard(long /*int*/ id, long /*int*/ sel, long /*int
 boolean writeSelectionToPasteboard(NSPasteboard pboard, NSString type) {
 	boolean result = false;
 
-	if (type.isEqualToString(OS.NSStringPboardType)) {
+	if (type.isEqualToString(OS.NSPasteboardTypeString)) {
 		Accessible acc = getAccessible();
 		id selection = acc.internal_accessibilityAttributeValue(OS.NSAccessibilitySelectedTextAttribute, ACC.CHILDID_SELF);
 		if (selection != null) {
 			NSString selectionString = new NSString(selection);
-			if (selectionString.length() > 0) result = pboard.setString(selectionString, OS.NSStringPboardType);
+			if (selectionString.length() > 0) result = pboard.setString(selectionString, OS.NSPasteboardTypeString);
 		}
 	}
 
