@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2011 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -50,6 +53,7 @@ public class Composite extends Scrollable {
 	Layout layout;
 	Control[] tabList;
 	int layoutCount, backgroundMode;
+	private boolean isStyledText; // This field is set to true if Object is StyledText
 
 Composite () {
 	/* Do nothing */
@@ -206,35 +210,12 @@ boolean accessibilityIsIgnored(long /*int*/ id, long /*int*/ sel) {
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
  *
+ * @deprecated use {@link Composite#layout(Control[], int)} instead
  * @since 3.1
  */
+@Deprecated
 public void changed (Control[] changed) {
-	checkWidget ();
-	if (changed == null) error (SWT.ERROR_INVALID_ARGUMENT);
-	for (int i=0; i<changed.length; i++) {
-		Control control = changed [i];
-		if (control == null) error (SWT.ERROR_INVALID_ARGUMENT);
-		if (control.isDisposed ()) error (SWT.ERROR_INVALID_ARGUMENT);
-		boolean ancestor = false;
-		Composite composite = control.parent;
-		while (composite != null) {
-			ancestor = composite == this;
-			if (ancestor) break;
-			composite = composite.parent;
-		}
-		if (!ancestor) error (SWT.ERROR_INVALID_PARENT);
-	}
-	for (int i=0; i<changed.length; i++) {
-		Control child = changed [i];
-		Composite composite = child.parent;
-		while (child != this) {
-			if (composite.layout == null || !composite.layout.flushCache (child)) {
-				composite.state |= LAYOUT_CHANGED;
-			}
-			child = composite;
-			composite = child.parent;
-		}
-	}
+	layout(changed, SWT.DEFER);
 }
 
 @Override
@@ -587,12 +568,25 @@ public boolean isLayoutDeferred () {
 boolean isOpaque (long /*int*/ id, long /*int*/ sel) {
 	if ((state & CANVAS) != 0) {
 		if (id == view.id) {
-			return region == null && background != null && background[3] == 1 && !isObscured ();
+			return region == null && isOpaque();
 		}
 	}
 	return super.isOpaque (id, sel);
 }
 
+boolean isOpaque() {
+	if (isStyledText) {
+		/*
+		 * When overlay scroll bars are used, Control.isObscured() always returns true,
+		 * causing isOpaque to return false. This causes performance problems while
+		 * redraw/scrolling. To workaround this, don't check for isObscured() here if the
+		 * object is StyledText.
+		 */
+		return background == null || backgroundAlpha == 255;
+	}
+	return background != null && background[3] == 1 && !isObscured ();
+}
+
 @Override
 boolean isTabGroup () {
 	if ((state & CANVAS) != 0) return true;
@@ -1098,6 +1092,11 @@ public boolean setFocus () {
 	return super.setFocus ();
 }
 
+@Override
+void setIsStyledText() {
+	isStyledText = true;
+}
+
 /**
  * Sets the layout which is associated with the receiver to be
  * the argument which may be null.
@@ -1275,4 +1274,5 @@ void updateLayout (boolean all) {
 		}
 	}
 }
+
 }
