@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2015 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2016 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -707,6 +710,15 @@ void createHandle () {
 				OS.objc_msgSend(window.id, OS.sel_setMovable_, 0);
 			}
 		}
+		if (OS.VERSION_MMB >= OS.VERSION_MMB(10, 12, 0)) {
+			/*
+			 * In macOS 10.12, a new system preference "prefer tabs when opening documents"
+			 * has been added which causes automatic tabbing of windows in Eclipse.
+			 * Disable automatic window tabbing, but setting the NSWindow.allowsAutomaticWindowTabbing
+			 * property to false.
+			 */
+			OS.objc_msgSend(OS.class_NSWindow, OS.sel_setAllowsAutomaticWindowTabbing_, false);
+		}
 		display.cascadeWindow(window, screen);
 		NSRect screenFrame = screen.frame();
 		double /*float*/ width = screenFrame.width * 5 / 8, height = screenFrame.height * 5 / 8;;
@@ -1209,7 +1221,7 @@ float getThemeAlpha () {
  * having a tool bar.
  * <p>
  *
- * @return a ToolBar object representing the Shell's tool bar, or <ocde>null</code>.
+ * @return a ToolBar object representing the Shell's tool bar, or <code>null</code>.
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
