@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2015 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2017 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -84,6 +87,7 @@ public abstract class Widget {
 	static final int WEBKIT_EVENTS_FIX = 1<<20;
 	static final String WEBKIT_EVENTS_FIX_KEY = "org.eclipse.swt.internal.webKitEventsFix"; //$NON-NLS-1$
 	static final String GLCONTEXT_KEY = "org.eclipse.swt.internal.cocoa.glcontext"; //$NON-NLS-1$
+	static final String STYLEDTEXT_KEY = "org.eclipse.swt.internal.cocoa.styledtext"; //$NON-NLS-1$
 
 	static final String IS_ACTIVE = "org.eclipse.swt.internal.isActive"; //$NON-NLS-1$
 
@@ -202,7 +206,7 @@ void accessibilitySetValue_forAttribute(long /*int*/ id, long /*int*/ sel, long
 String getClipboardText () {
 	NSPasteboard pasteboard = NSPasteboard.generalPasteboard ();
 	if (pasteboard == null) return "";
-	NSString string = pasteboard.stringForType (OS.NSStringPboardType);
+	NSString string = pasteboard.stringForType (OS.NSPasteboardTypeString);
 	return string != null ? string.getString () : null;
 }
 
@@ -213,6 +217,10 @@ long /*int*/ attributedSubstringFromRange (long /*int*/ id, long /*int*/ sel, lo
 	return 0;
 }
 
+void cacheDisplayInRect_toBitmapImageRep (long id, long sel, NSRect rect, long imageRep) {
+	callSuper(id, sel, rect, imageRep);
+}
+
 void callSuper(long /*int*/ id, long /*int*/ sel) {
 	objc_super super_struct = new objc_super();
 	super_struct.receiver = id;
@@ -352,7 +360,7 @@ boolean canDragRowsWithIndexes_atPoint(long /*int*/ id, long /*int*/ sel, long /
 }
 
 long /*int*/ characterIndexForPoint (long /*int*/ id, long /*int*/ sel, long /*int*/ point) {
-	return OS.NSNotFound;
+	return OS.NSNotFound();
 }
 
 long /*int*/ columnAtPoint(long /*int*/ id, long /*int*/ sel, NSPoint point) {
@@ -608,8 +616,8 @@ void copyToClipboard (char [] buffer) {
 	if (buffer.length == 0) return;
 	NSPasteboard pasteboard = NSPasteboard.generalPasteboard ();
 	if (pasteboard == null) return;
-	pasteboard.declareTypes (NSArray.arrayWithObject (OS.NSStringPboardType), null);
-	pasteboard.setString (NSString.stringWithCharacters (buffer, buffer.length), OS.NSStringPboardType);
+	pasteboard.declareTypes (NSArray.arrayWithObject (OS.NSPasteboardTypeString), null);
+	pasteboard.setString (NSString.stringWithCharacters (buffer, buffer.length), OS.NSPasteboardTypeString);
 }
 
 void createHandle () {
@@ -727,6 +735,9 @@ void drawInteriorWithFrame_inView (long /*int*/ id, long /*int*/ sel, NSRect cel
 	callSuper(id, sel, cellFrame, view);
 }
 
+void drawBezelWithFrame_inView (long /*int*/ id, long /*int*/ sel, NSRect cellFrame, long /*int*/ view) {
+	callSuper(id, sel, cellFrame, view);
+}
 
 void drawLabelInRect(long /*int*/ id, long /*int*/ sel, boolean shouldTruncateLabel, NSRect rect) {
 	objc_super super_struct = new objc_super();
@@ -1043,7 +1054,7 @@ boolean isActive () {
  * and <code>false</code> otherwise.
  *
  * @return <code>true</code> when the widget has auto direction and <code>false</code> otherwise
- * 
+ *
  * @see SWT#AUTO_TEXT_DIRECTION
  *
  * @since 3.105
@@ -1605,6 +1616,10 @@ public void setData (Object data) {
 		state |= WEBKIT_EVENTS_FIX;
 		return;
 	}
+	if (STYLEDTEXT_KEY.equals(data)) {
+		setIsStyledText();
+		return;
+	}
 	if ((state & KEYED_DATA) != 0) {
 		((Object []) this.data) [0] = data;
 	} else {
@@ -1612,6 +1627,9 @@ public void setData (Object data) {
 	}
 }
 
+void setIsStyledText() {
+}
+
 /**
  * Sets the application defined property of the receiver
  * with the specified name to the given value.
