@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2016 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -253,7 +256,7 @@ Image createArrowImage (int width, int height) {
 	Color black = display.getSystemColor (SWT.COLOR_BLACK);
 	Color background = parent.getBackground ();
 
-	PaletteData palette = new PaletteData (new RGB[]{foreground.getRGB(), background.getRGB(), black.getRGB()});
+	PaletteData palette = new PaletteData (foreground.getRGB(), background.getRGB(), black.getRGB());
 	ImageData imageData = new ImageData (width, height, 4, palette);
 	imageData.transparentPixel = 1;
 	Image image = new Image (display, imageData);
@@ -621,12 +624,7 @@ void updateChevron() {
 			if (chevron == null) {
 				chevron = new ToolBar (parent, SWT.FLAT | SWT.NO_FOCUS);
 				ToolItem toolItem = new ToolItem (chevron, SWT.PUSH);
-				toolItem.addListener (SWT.Selection, new Listener() {
-					@Override
-					public void handleEvent (Event event) {
-						CoolItem.this.onSelection (event);
-					}
-				});
+				toolItem.addListener (SWT.Selection, event -> CoolItem.this.onSelection (event));
 			}
 			int controlHeight, currentImageHeight = 0;
 			if ((parent.style & SWT.VERTICAL) != 0) {
