@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2016 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2018 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -78,9 +81,9 @@ public final class TextLayout extends Resource {
 public TextLayout (Device device) {
 	super(device);
 	device = this.device;
-	context = OS.gdk_pango_context_get();
+	context = GDK.gdk_pango_context_get();
 	if (context == 0) SWT.error(SWT.ERROR_NO_HANDLES);
-	OS.pango_context_set_language(context, OS.gtk_get_default_language());
+	OS.pango_context_set_language(context, GTK.gtk_get_default_language());
 	OS.pango_context_set_base_dir(context, OS.PANGO_DIRECTION_LTR);
 	layout = OS.pango_layout_new(context);
 	if (layout == 0) SWT.error(SWT.ERROR_NO_HANDLES);
@@ -104,7 +107,7 @@ void checkLayout() {
 void computeRuns () {
 	if (attrList != 0) return;
 	String segmentsText = getSegmentsText();
-	byte[] buffer = Converter.wcsToMbcs(null, segmentsText, false);
+	byte[] buffer = Converter.wcsToMbcs(segmentsText, false);
 	OS.pango_layout_set_text (layout, buffer, buffer.length);
 	if (stylesCount == 2 && styles[0].style == null && ascentInPoints == -1 && descentInPoints == -1 && segments == null) return;
 	long /*int*/ ptr = OS.pango_layout_get_text(layout);
@@ -154,7 +157,7 @@ void computeRuns () {
 			lineIndex++;
 		}
 		segmentsText.getChars(oldPos, segementsLength, chars,  oldPos + lineIndex * 2);
-		buffer = Converter.wcsToMbcs(null, chars, false);
+		buffer = Converter.wcsToMbcs(chars, false);
 		OS.pango_layout_set_text (layout, buffer, buffer.length);
 		ptr = OS.pango_layout_get_text(layout);
 		offsetCount += 2 * lineCount;
@@ -179,7 +182,7 @@ void computeRuns () {
 			}
 		}
 	}
-	int strlen = OS.strlen(ptr);
+	int strlen = C.strlen(ptr);
 	Font defaultFont = font != null ? font : device.systemFont;
 	for (int i = 0; i < stylesCount - 1; i++) {
 		StyleItem styleItem = styles[i];
@@ -216,7 +219,18 @@ void computeRuns () {
 					break;
 				case SWT.UNDERLINE_LINK: {
 					if (style.foreground == null) {
-						long /*int*/ attr = OS.pango_attr_foreground_new((short)0, (short)0x3333, (short)0x9999);
+						// Bug 497071: use COLOR_LINK_FOREGROUND for StyledText links
+						long /*int*/ attr;
+						if (GTK.GTK3) {
+							GdkRGBA linkRGBA = device.getSystemColor(SWT.COLOR_LINK_FOREGROUND).handleRGBA;
+							// Manual conversion since PangoAttrColor is a special case.
+							// It uses GdkColor style colors but is supported on GTK3.
+							attr = OS.pango_attr_foreground_new((short)(linkRGBA.red * 0xFFFF),
+									(short)(linkRGBA.green * 0xFFFF), (short)(linkRGBA.blue * 0xFFFF));
+						} else {
+							GdkColor linkColor = device.getSystemColor(SWT.COLOR_LINK_FOREGROUND).handle;
+							attr = OS.pango_attr_foreground_new(linkColor.red, linkColor.green, linkColor.blue);
+						}
 						OS.memmove (attribute, attr, PangoAttribute.sizeof);
 						attribute.start_index = byteStart;
 						attribute.end_index = byteEnd;
@@ -235,8 +249,14 @@ void computeRuns () {
 			OS.pango_attr_list_insert(attrList, attr);
 			OS.pango_attr_list_insert(selAttrList, OS.pango_attribute_copy(attr));
 			if (style.underlineColor != null) {
-				GdkColor fg = style.underlineColor.handle;
-				attr = OS.pango_attr_underline_color_new(fg.red, fg.green, fg.blue);
+				if (GTK.GTK3) {
+					GdkRGBA rgba = style.underlineColor.handleRGBA;
+					attr = OS.pango_attr_underline_color_new((short)(rgba.red * 0xFFFF),
+							(short)(rgba.green * 0xFFFF), (short)(rgba.blue * 0xFFFF));
+				} else {
+					GdkColor fg = style.underlineColor.handle;
+					attr = OS.pango_attr_underline_color_new(fg.red, fg.green, fg.blue);
+				}
 				if (attr != 0) {
 					OS.memmove(attribute, attr, PangoAttribute.sizeof);
 					attribute.start_index = byteStart;
@@ -256,8 +276,14 @@ void computeRuns () {
 			OS.pango_attr_list_insert(attrList, attr);
 			OS.pango_attr_list_insert(selAttrList, OS.pango_attribute_copy(attr));
 			if (style.strikeoutColor != null) {
-				GdkColor fg = style.strikeoutColor.handle;
-				attr = OS.pango_attr_strikethrough_color_new(fg.red, fg.green, fg.blue);
+				if (GTK.GTK3) {
+					GdkRGBA rgba = style.strikeoutColor.handleRGBA;
+					attr = OS.pango_attr_strikethrough_color_new((short)(rgba.red * 0xFFFF),
+							(short)(rgba.green * 0xFFFF), (short)(rgba.blue * 0xFFFF));
+				} else {
+					GdkColor fg = style.strikeoutColor.handle;
+					attr = OS.pango_attr_strikethrough_color_new(fg.red, fg.green, fg.blue);
+				}
 				if (attr != 0) {
 					OS.memmove(attribute, attr, PangoAttribute.sizeof);
 					attribute.start_index = byteStart;
@@ -270,8 +296,15 @@ void computeRuns () {
 		}
 		Color foreground = style.foreground;
 		if (foreground != null && !foreground.isDisposed()) {
-			GdkColor fg = foreground.handle;
-			long /*int*/ attr = OS.pango_attr_foreground_new(fg.red, fg.green, fg.blue);
+			long /*int*/ attr;
+			if (GTK.GTK3) {
+				GdkRGBA rgba = foreground.handleRGBA;
+				attr = OS.pango_attr_foreground_new((short)(rgba.red * 0xFFFF),
+						(short)(rgba.green * 0xFFFF), (short)(rgba.blue * 0xFFFF));
+			} else {
+				GdkColor fg = foreground.handle;
+				attr = OS.pango_attr_foreground_new(fg.red, fg.green, fg.blue);
+			}
 			OS.memmove (attribute, attr, PangoAttribute.sizeof);
 			attribute.start_index = byteStart;
 			attribute.end_index = byteEnd;
@@ -280,8 +313,15 @@ void computeRuns () {
 		}
 		Color background = style.background;
 		if (background != null && !background.isDisposed()) {
-			GdkColor bg = background.handle;
-			long /*int*/ attr = OS.pango_attr_background_new(bg.red, bg.green, bg.blue);
+			long /*int*/ attr;
+			if (GTK.GTK3) {
+				GdkRGBA rgba = background.handleRGBA;
+				attr = OS.pango_attr_background_new((short)(rgba.red * 0xFFFF),
+						(short)(rgba.green * 0xFFFF), (short)(rgba.blue * 0xFFFF));
+			} else {
+				GdkColor bg = background.handle;
+				attr = OS.pango_attr_background_new(bg.red, bg.green, bg.blue);
+			}
 			OS.memmove (attribute, attr, PangoAttribute.sizeof);
 			attribute.start_index = byteStart;
 			attribute.end_index = byteEnd;
@@ -462,12 +502,13 @@ void drawInPixels(GC gc, int x, int y, int selectionStart, int selectionEnd, Col
 		long /*int*/ ptr = OS.pango_layout_get_text(layout);
 		long /*int*/ iter = OS.pango_layout_get_iter(layout);
 		if (selectionBackground == null) selectionBackground = device.getSystemColor(SWT.COLOR_LIST_SELECTION);
-		if (cairo != 0) {
-			Cairo.cairo_save(cairo);
+		Cairo.cairo_save(cairo);
+		if (GTK.GTK3) {
+			GdkRGBA rgba = selectionBackground.handleRGBA;
+			Cairo.cairo_set_source_rgba(cairo, rgba.red, rgba.green, rgba.blue, rgba.alpha);
+		} else {
 			GdkColor color = selectionBackground.handle;
 			Cairo.cairo_set_source_rgba(cairo, (color.red & 0xFFFF) / (float)0xFFFF, (color.green & 0xFFFF) / (float)0xFFFF, (color.blue & 0xFFFF) / (float)0xFFFF, data.alpha / (float)0xFF);
-		} else {
-			OS.gdk_gc_set_foreground(gc.handle, selectionBackground.handle);
 		}
 		int lineIndex = 0;
 		do {
@@ -501,40 +542,27 @@ void drawInPixels(GC gc, int x, int y, int selectionStart, int selectionEnd, Col
 					height = Math.max (height, DPIUtil.autoScaleUp(getDevice(), ascentInPoints + descentInPoints));
 				}
 				int width = (flags & SWT.FULL_SELECTION) != 0 ? 0x7fff : height / 3;
-				if (cairo != 0) {
-					Cairo.cairo_rectangle(cairo, lineX, lineY, width, height);
-					Cairo.cairo_fill(cairo);
-				} else {
-					OS.gdk_draw_rectangle(data.drawable, gc.handle, 1, lineX, lineY, width, height);
-				}
+				Cairo.cairo_rectangle(cairo, lineX, lineY, width, height);
+				Cairo.cairo_fill(cairo);
 			}
 			lineIndex++;
 		} while (lineIndex < lineCount);
 		OS.pango_layout_iter_free(iter);
 		if (attrs[0] != 0) OS.g_free(attrs[0]);
-		if (cairo != 0) {
-			Cairo.cairo_restore(cairo);
-		} else {
-			OS.gdk_gc_set_foreground(gc.handle, data.foreground);
-		}
+		Cairo.cairo_restore(cairo);
 	}
 	if (length == 0) return;
 	if (!hasSelection) {
-		if (cairo != 0) {
-			if ((data.style & SWT.MIRRORED) != 0) {
-				Cairo.cairo_save(cairo);
-				Cairo.cairo_scale(cairo, -1,  1);
-				Cairo.cairo_translate(cairo, -2 * x - width(), 0);
-			}
-			Cairo.cairo_move_to(cairo, x, y);
-			OS.pango_cairo_show_layout(cairo, layout);
-			drawBorder(gc, x, y, null);
-			if ((data.style & SWT.MIRRORED) != 0) {
-			    Cairo.cairo_restore(cairo);
-			}
-		} else {
-			OS.gdk_draw_layout(data.drawable, gc.handle, x, y, layout);
-			drawBorder(gc, x, y, null);
+		if ((data.style & SWT.MIRRORED) != 0) {
+			Cairo.cairo_save(cairo);
+			Cairo.cairo_scale(cairo, -1,  1);
+			Cairo.cairo_translate(cairo, -2 * x - width(), 0);
+		}
+		Cairo.cairo_move_to(cairo, x, y);
+		OS.pango_cairo_show_layout(cairo, layout);
+		drawBorder(gc, x, y, null);
+		if ((data.style & SWT.MIRRORED) != 0) {
+		    Cairo.cairo_restore(cairo);
 		}
 	} else {
 		selectionStart = Math.min(Math.max(0, selectionStart), length - 1);
@@ -546,62 +574,40 @@ void drawInPixels(GC gc, int x, int y, int selectionStart, int selectionEnd, Col
 		if (selectionBackground == null) selectionBackground = device.getSystemColor(SWT.COLOR_LIST_SELECTION);
 		boolean fullSelection = selectionStart == 0 && selectionEnd == length - 1;
 		if (fullSelection) {
-			if (cairo != 0) {
-				long /*int*/ ptr = OS.pango_layout_get_text(layout);
-				if ((data.style & SWT.MIRRORED) != 0) {
-					Cairo.cairo_save(cairo);
-					Cairo.cairo_scale(cairo, -1,  1);
-					Cairo.cairo_translate(cairo, -2 * x - width(), 0);
-				}
-				drawWithCairo(gc, x, y, 0, OS.strlen(ptr), fullSelection, selectionForeground.handle, selectionBackground.handle);
-				if ((data.style & SWT.MIRRORED) != 0) {
-					Cairo.cairo_restore(cairo);
-				}
-			} else {
-				OS.gdk_draw_layout_with_colors(data.drawable, gc.handle, x, y, layout, selectionForeground.handle, selectionBackground.handle);
-				drawBorder(gc, x, y, selectionForeground.handle);
+			long /*int*/ ptr = OS.pango_layout_get_text(layout);
+			if ((data.style & SWT.MIRRORED) != 0) {
+				Cairo.cairo_save(cairo);
+				Cairo.cairo_scale(cairo, -1,  1);
+				Cairo.cairo_translate(cairo, -2 * x - width(), 0);
+			}
+			drawWithCairo(gc, x, y, 0, C.strlen(ptr), fullSelection, selectionForeground, selectionBackground);
+			if ((data.style & SWT.MIRRORED) != 0) {
+				Cairo.cairo_restore(cairo);
 			}
 		} else {
 			long /*int*/ ptr = OS.pango_layout_get_text(layout);
 			int byteSelStart = (int)/*64*/(OS.g_utf16_offset_to_pointer(ptr, selectionStart) - ptr);
 			int byteSelEnd = (int)/*64*/(OS.g_utf16_offset_to_pointer(ptr, selectionEnd + 1) - ptr);
-			int strlen = OS.strlen(ptr);
+			int strlen = C.strlen(ptr);
 			byteSelStart = Math.min(byteSelStart, strlen);
 			byteSelEnd = Math.min(byteSelEnd, strlen);
-			if (cairo != 0) {
-				if ((data.style & SWT.MIRRORED) != 0) {
-					Cairo.cairo_save(cairo);
-					Cairo.cairo_scale(cairo, -1,  1);
-					Cairo.cairo_translate(cairo, -2 * x - width(), 0);
-				}
-				drawWithCairo(gc, x, y, byteSelStart, byteSelEnd, fullSelection, selectionForeground.handle, selectionBackground.handle);
-				if ((data.style & SWT.MIRRORED) != 0) {
-					Cairo.cairo_restore(cairo);
-				}
-			} else {
-				Region clipping = new Region();
-				gc.getClipping(clipping);
-				OS.gdk_draw_layout(data.drawable, gc.handle, x, y, layout);
-				drawBorder(gc, x, y, null);
-				int[] ranges = new int[]{byteSelStart, byteSelEnd};
-				long /*int*/ rgn = OS.gdk_pango_layout_get_clip_region(layout, x, y, ranges, ranges.length / 2);
-				if (rgn != 0) {
-					OS.gdk_gc_set_clip_region(gc.handle, rgn);
-					OS.gdk_region_destroy(rgn);
-				}
-				OS.gdk_draw_layout_with_colors(data.drawable, gc.handle, x, y, layout, selectionForeground.handle, selectionBackground.handle);
-				drawBorder(gc, x, y, selectionForeground.handle);
-				gc.setClipping(clipping);
-				clipping.dispose();
+			if ((data.style & SWT.MIRRORED) != 0) {
+				Cairo.cairo_save(cairo);
+				Cairo.cairo_scale(cairo, -1,  1);
+				Cairo.cairo_translate(cairo, -2 * x - width(), 0);
+			}
+			drawWithCairo(gc, x, y, byteSelStart, byteSelEnd, fullSelection, selectionForeground, selectionBackground);
+			if ((data.style & SWT.MIRRORED) != 0) {
+				Cairo.cairo_restore(cairo);
 			}
 		}
 	}
-	if (cairo != 0) {
-		Cairo.cairo_new_path(cairo);
-	}
+	Cairo.cairo_new_path(cairo);
 }
 
-void drawWithCairo(GC gc, int x, int y, int start, int end, boolean fullSelection, GdkColor fg, GdkColor bg) {
+// Bug 477950: In order to support GTK2 and GTK3 colors simultaneously, this method's parameters
+// were modified to accept SWT Color objects instead of GdkColor structs.
+void drawWithCairo(GC gc, int x, int y, int start, int end, boolean fullSelection, Color fg, Color bg) {
 	GCData data = gc.data;
 	long /*int*/ cairo = data.cairo;
 	Cairo.cairo_save(cairo);
@@ -611,15 +617,23 @@ void drawWithCairo(GC gc, int x, int y, int start, int end, boolean fullSelectio
 		drawBorder(gc, x, y, null);
 	}
 	int[] ranges = new int[]{start, end};
-	long /*int*/ rgn = OS.gdk_pango_layout_get_clip_region(layout, x, y, ranges, ranges.length / 2);
+	long /*int*/ rgn = GDK.gdk_pango_layout_get_clip_region(layout, x, y, ranges, ranges.length / 2);
 	if (rgn != 0) {
-		OS.gdk_cairo_region(cairo, rgn);
+		GDK.gdk_cairo_region(cairo, rgn);
 		Cairo.cairo_clip(cairo);
-		Cairo.cairo_set_source_rgba(cairo, (bg.red & 0xFFFF) / (float)0xFFFF, (bg.green & 0xFFFF) / (float)0xFFFF, (bg.blue & 0xFFFF) / (float)0xFFFF, data.alpha / (float)0xFF);
+		if (GTK.GTK3) {
+			Cairo.cairo_set_source_rgba(cairo, bg.handleRGBA.red, bg.handleRGBA.green, bg.handleRGBA.blue, bg.handleRGBA.alpha);
+		} else {
+			Cairo.cairo_set_source_rgba(cairo, (bg.handle.red & 0xFFFF) / (float)0xFFFF, (bg.handle.green & 0xFFFF) / (float)0xFFFF, (bg.handle.blue & 0xFFFF) / (float)0xFFFF, data.alpha / (float)0xFF);
+		}
 		Cairo.cairo_paint(cairo);
-		OS.gdk_region_destroy(rgn);
+		GDK.gdk_region_destroy(rgn);
+	}
+	if (GTK.GTK3) {
+		Cairo.cairo_set_source_rgba(cairo, fg.handleRGBA.red, fg.handleRGBA.green, fg.handleRGBA.blue, fg.handleRGBA.alpha);
+	} else {
+		Cairo.cairo_set_source_rgba(cairo, (fg.handle.red & 0xFFFF) / (float)0xFFFF, (fg.handle.green & 0xFFFF) / (float)0xFFFF, (fg.handle.blue & 0xFFFF) / (float)0xFFFF, data.alpha / (float)0xFF);
 	}
-	Cairo.cairo_set_source_rgba(cairo, (fg.red & 0xFFFF) / (float)0xFFFF, (fg.green & 0xFFFF) / (float)0xFFFF, (fg.blue & 0xFFFF) / (float)0xFFFF, data.alpha / (float)0xFF);
 	Cairo.cairo_move_to(cairo, x, y);
 	OS.pango_layout_set_attributes(layout, selAttrList);
 	OS.pango_cairo_show_layout(cairo, layout);
@@ -628,15 +642,13 @@ void drawWithCairo(GC gc, int x, int y, int start, int end, boolean fullSelectio
 	Cairo.cairo_restore(cairo);
 }
 
-void drawBorder(GC gc, int x, int y, GdkColor selectionColor) {
+//Bug 477950: In order to support GTK2 and GTK3 colors simultaneously, this method's parameters
+//were modified to accept SWT Color objects instead of GdkColor structs.
+void drawBorder(GC gc, int x, int y, Color selectionColor) {
 	GCData data = gc.data;
 	long /*int*/ cairo = data.cairo;
-	long /*int*/ gdkGC = gc.handle;
 	long /*int*/ ptr = OS.pango_layout_get_text(layout);
-	GdkGCValues gcValues = null;
-	if (cairo != 0) {
-		Cairo.cairo_save(cairo);
-	}
+	Cairo.cairo_save(cairo);
 	for (int i = 0; i < stylesCount - 1; i++) {
 		TextStyle style = styles[i].style;
 		if (style == null) continue;
@@ -652,17 +664,25 @@ void drawBorder(GC gc, int x, int y, GdkColor selectionColor) {
 			int byteStart = (int)/*64*/(OS.g_utf16_offset_to_pointer(ptr, start) - ptr);
 			int byteEnd = (int)/*64*/(OS.g_utf16_offset_to_pointer(ptr, end + 1) - ptr);
 			int[] ranges = new int[]{byteStart, byteEnd};
-			long /*int*/ rgn = OS.gdk_pango_layout_get_clip_region(layout, x, y, ranges, ranges.length / 2);
+			long /*int*/ rgn = GDK.gdk_pango_layout_get_clip_region(layout, x, y, ranges, ranges.length / 2);
 			if (rgn != 0) {
 				int[] nRects = new int[1];
 				long /*int*/[] rects = new long /*int*/[1];
 				Region.gdk_region_get_rectangles(rgn, rects, nRects);
 				GdkRectangle rect = new GdkRectangle();
+				GdkRGBA colorRGBA = null;
 				GdkColor color = null;
-				if (color == null && style.borderColor != null) color = style.borderColor.handle;
-				if (color == null && selectionColor != null) color = selectionColor;
-				if (color == null && style.foreground != null) color = style.foreground.handle;
-				if (color == null) color = data.foreground;
+				if (GTK.GTK3) {
+					if (colorRGBA == null && style.borderColor != null) colorRGBA = style.borderColor.handleRGBA;
+					if (colorRGBA == null && selectionColor != null) colorRGBA = selectionColor.handleRGBA;
+					if (colorRGBA == null && style.foreground != null) colorRGBA = style.foreground.handleRGBA;
+					if (colorRGBA == null) colorRGBA = data.foregroundRGBA;
+				} else {
+					if (color == null && style.borderColor != null) color = style.borderColor.handle;
+					if (color == null && selectionColor != null) color = selectionColor.handle;
+					if (color == null && style.foreground != null) color = style.foreground.handle;
+					if (color == null) color = data.foreground;
+				}
 				int width = 1;
 				float[] dashes = null;
 				switch (style.borderStyle) {
@@ -670,61 +690,32 @@ void drawBorder(GC gc, int x, int y, GdkColor selectionColor) {
 					case SWT.BORDER_DASH: dashes = width != 0 ? GC.LINE_DASH : GC.LINE_DASH_ZERO; break;
 					case SWT.BORDER_DOT: dashes = width != 0 ? GC.LINE_DOT : GC.LINE_DOT_ZERO; break;
 				}
-				if (cairo != 0) {
-					Cairo.cairo_set_source_rgba(cairo, (color.red & 0xFFFF) / (float)0xFFFF, (color.green & 0xFFFF) / (float)0xFFFF, (color.blue & 0xFFFF) / (float)0xFFFF, data.alpha / (float)0xFF);
-					Cairo.cairo_set_line_width(cairo, width);
-					if (dashes != null) {
-						double[] cairoDashes = new double[dashes.length];
-						for (int j = 0; j < cairoDashes.length; j++) {
-							cairoDashes[j] = width == 0 || data.lineStyle == SWT.LINE_CUSTOM ? dashes[j] : dashes[j] * width;
-						}
-						Cairo.cairo_set_dash(cairo, cairoDashes, cairoDashes.length, 0);
-					} else {
-						Cairo.cairo_set_dash(cairo, null, 0, 0);
-					}
-					for (int j=0; j<nRects[0]; j++) {
-						OS.memmove(rect, rects[0] + (j * GdkRectangle.sizeof), GdkRectangle.sizeof);
-						Cairo.cairo_rectangle(cairo, rect.x + 0.5, rect.y + 0.5, rect.width - 1, rect.height - 1);
-					}
-					Cairo.cairo_stroke(cairo);
+				if (GTK.GTK3) {
+					Cairo.cairo_set_source_rgba(cairo, colorRGBA.red, colorRGBA.green, colorRGBA.blue, colorRGBA.alpha);
 				} else {
-					if (gcValues == null) {
-						gcValues = new GdkGCValues();
-						OS.gdk_gc_get_values(gdkGC, gcValues);
-					}
-					OS.gdk_gc_set_foreground(gdkGC, color);
-					int cap_style = OS.GDK_CAP_BUTT;
-					int join_style = OS.GDK_JOIN_MITER;
-					int line_style = 0;
-					if (dashes != null) {
-						byte[] dash_list = new byte[dashes.length];
-						for (int j = 0; j < dash_list.length; j++) {
-							dash_list[j] = (byte)(width == 0 || data.lineStyle == SWT.LINE_CUSTOM ? dashes[j] : dashes[j] * width);
-						}
-						OS.gdk_gc_set_dashes(gdkGC, 0, dash_list, dash_list.length);
-						line_style = OS.GDK_LINE_ON_OFF_DASH;
-					} else {
-						line_style = OS.GDK_LINE_SOLID;
-					}
-					OS.gdk_gc_set_line_attributes(gdkGC, width, line_style, cap_style, join_style);
-					for (int j=0; j<nRects[0]; j++) {
-						OS.memmove(rect, rects[0] + (j * GdkRectangle.sizeof), GdkRectangle.sizeof);
-						OS.gdk_draw_rectangle(data.drawable, gdkGC, 0, rect.x, rect.y, rect.width - 1, rect.height - 1);
-					}
+					Cairo.cairo_set_source_rgba(cairo, (color.red & 0xFFFF) / (float)0xFFFF, (color.green & 0xFFFF) / (float)0xFFFF, (color.blue & 0xFFFF) / (float)0xFFFF, data.alpha / (float)0xFF);
 				}
+				Cairo.cairo_set_line_width(cairo, width);
+				if (dashes != null) {
+					double[] cairoDashes = new double[dashes.length];
+					for (int j = 0; j < cairoDashes.length; j++) {
+						cairoDashes[j] = width == 0 || data.lineStyle == SWT.LINE_CUSTOM ? dashes[j] : dashes[j] * width;
+					}
+					Cairo.cairo_set_dash(cairo, cairoDashes, cairoDashes.length, 0);
+				} else {
+					Cairo.cairo_set_dash(cairo, null, 0, 0);
+				}
+				for (int j=0; j<nRects[0]; j++) {
+					OS.memmove(rect, rects[0] + (j * GdkRectangle.sizeof), GdkRectangle.sizeof);
+					Cairo.cairo_rectangle(cairo, rect.x + 0.5, rect.y + 0.5, rect.width - 1, rect.height - 1);
+				}
+				Cairo.cairo_stroke(cairo);
 				if (rects[0] != 0) OS.g_free(rects[0]);
-				OS.gdk_region_destroy(rgn);
+				GDK.gdk_region_destroy(rgn);
 			}
 		}
 	}
-	if (gcValues != null) {
-		int mask = OS.GDK_GC_FOREGROUND | OS.GDK_GC_LINE_WIDTH | OS.GDK_GC_LINE_STYLE | OS.GDK_GC_CAP_STYLE | OS.GDK_GC_JOIN_STYLE;
-		OS.gdk_gc_set_values(gdkGC, gcValues, mask);
-		data.state &= ~GC.LINE_STYLE;
-	}
-	if (cairo != 0) {
-		Cairo.cairo_restore(cairo);
-	}
+	Cairo.cairo_restore(cairo);
 }
 
 void freeRuns() {
@@ -800,7 +791,7 @@ public Rectangle getBounds() {
 	int lineCount = OS.pango_layout_get_line_count(layout);
 	int totalLineheight = 0;
 	for (int i = 0; i < lineCount; i++) {
-		totalLineheight += this.getLineBounds(i).height;
+		totalLineheight += this.getLineBounds(i).height + OS.PANGO_PIXELS(OS.pango_layout_get_spacing(layout));
 	}
 	bounds.height = totalLineheight;
 	return bounds;
@@ -853,11 +844,11 @@ Rectangle getBoundsInPixels(int start, int end) {
 	long /*int*/ ptr = OS.pango_layout_get_text(layout);
 	int byteStart = (int)/*64*/(OS.g_utf16_offset_to_pointer (ptr, start) - ptr);
 	int byteEnd = (int)/*64*/(OS.g_utf16_offset_to_pointer (ptr, end + 1) - ptr);
-	int strlen = OS.strlen(ptr);
+	int strlen = C.strlen(ptr);
 	byteStart = Math.min(byteStart, strlen);
 	byteEnd = Math.min(byteEnd, strlen);
 	int[] ranges = new int[]{byteStart, byteEnd};
-	long /*int*/ clipRegion = OS.gdk_pango_layout_get_clip_region(layout, 0, 0, ranges, 1);
+	long /*int*/ clipRegion = GDK.gdk_pango_layout_get_clip_region(layout, 0, 0, ranges, 1);
 	if (clipRegion == 0) return new Rectangle(0, 0, 0, 0);
 	GdkRectangle rect = new GdkRectangle();
 
@@ -869,7 +860,7 @@ Rectangle getBoundsInPixels(int start, int end) {
 	PangoRectangle pangoRect = new PangoRectangle();
 	long /*int*/ iter = OS.pango_layout_get_iter(layout);
 	if (iter == 0) SWT.error(SWT.ERROR_NO_HANDLES);
-	long /*int*/ linesRegion = OS.gdk_region_new();
+	long /*int*/ linesRegion = GDK.gdk_region_new();
 	if (linesRegion == 0) SWT.error(SWT.ERROR_NO_HANDLES);
 	int lineEnd = 0;
 	do {
@@ -884,17 +875,14 @@ Rectangle getBoundsInPixels(int start, int end) {
 		rect.y = OS.PANGO_PIXELS(pangoRect.y);
 		rect.width = OS.PANGO_PIXELS(pangoRect.width);
 		rect.height = OS.PANGO_PIXELS(pangoRect.height);
-		OS.gdk_region_union_with_rect(linesRegion, rect);
+		GDK.gdk_region_union_with_rect(linesRegion, rect);
 	} while (lineEnd + 1 <= byteEnd);
-	OS.gdk_region_intersect(clipRegion, linesRegion);
-	OS.gdk_region_destroy(linesRegion);
+	GDK.gdk_region_intersect(clipRegion, linesRegion);
+	GDK.gdk_region_destroy(linesRegion);
 	OS.pango_layout_iter_free(iter);
 
-	OS.gdk_region_get_clipbox(clipRegion, rect);
-	OS.gdk_region_destroy(clipRegion);
-	if (OS.pango_context_get_base_dir(context) == OS.PANGO_DIRECTION_RTL) {
-		rect.x = width() - rect.x - rect.width;
-	}
+	GDK.gdk_region_get_clipbox(clipRegion, rect);
+	GDK.gdk_region_destroy(clipRegion);
 	rect.x += Math.min (indent, wrapIndent);
 	return new Rectangle(rect.x, rect.y, rect.width, rect.height);
 }
@@ -996,7 +984,7 @@ public int getLevel(int offset) {
 	PangoLayoutRun run = new PangoLayoutRun();
 	long /*int*/ ptr = OS.pango_layout_get_text(layout);
 	long /*int*/ byteOffset = OS.g_utf16_offset_to_pointer(ptr, offset) - ptr;
-	int strlen = OS.strlen(ptr);
+	int strlen = C.strlen(ptr);
 	byteOffset = Math.min(byteOffset, strlen);
 	do {
 		long /*int*/ runPtr = OS.pango_layout_iter_get_run(iter);
@@ -1048,9 +1036,6 @@ Rectangle getLineBoundsInPixels(int lineIndex) {
 	if (ascentInPoints != -1 && descentInPoints != -1) {
 		height = Math.max (height, DPIUtil.autoScaleUp(getDevice(), ascentInPoints + descentInPoints));
 	}
-	if (OS.pango_context_get_base_dir(context) == OS.PANGO_DIRECTION_RTL) {
-		x = width() - x - width;
-	}
 	x += Math.min (indent, wrapIndent);
 	return new Rectangle(x, y, width, height);
 }
@@ -1094,7 +1079,7 @@ public int getLineIndex(int offset) {
 	int line = 0;
 	long /*int*/ ptr = OS.pango_layout_get_text(layout);
 	long /*int*/ byteOffset = OS.g_utf16_offset_to_pointer(ptr,offset) - ptr;
-	int strlen = OS.strlen(ptr);
+	int strlen = C.strlen(ptr);
 	byteOffset = Math.min(byteOffset, strlen);
 	long /*int*/ iter = OS.pango_layout_get_iter(layout);
 	if (iter == 0) SWT.error(SWT.ERROR_NO_HANDLES);
@@ -1205,7 +1190,7 @@ Point getLocationInPixels(int offset, boolean trailing) {
 	offset = translateOffset(offset);
 	long /*int*/ ptr = OS.pango_layout_get_text(layout);
 	int byteOffset = (int)/*64*/(OS.g_utf16_offset_to_pointer(ptr, offset) - ptr);
-	int strlen = OS.strlen(ptr);
+	int strlen = C.strlen(ptr);
 	byteOffset = Math.min(byteOffset, strlen);
 	PangoRectangle pos = new PangoRectangle();
 	OS.pango_layout_index_to_pos(layout, byteOffset, pos);
@@ -2344,4 +2329,25 @@ int width () {
 	return w[0];
 }
 
+/**
+ * Sets Default Tab Width in terms if number of space characters.
+ *
+ * @param tabLength in number of characters
+ *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_INVALID_ARGUMENT - if the tabLength is less than <code>0</code></li>
+ * </ul>
+ * @exception SWTException <ul>
+ *    <li>ERROR_GRAPHIC_DISPOSED - if the receiver has been disposed</li>
+ * </ul>
+ *
+ * @noreference This method is not intended to be referenced by clients.
+ *
+ * DO NOT USE This might be removed in 4.8
+ * @since 3.107
+ */
+public void setDefaultTabWidth(int tabLength) {
+	//unused in GTK
+}
+
 }
