@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2016 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2017 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -42,6 +45,7 @@ import org.eclipse.swt.internal.gtk.*;
 public class Canvas extends Composite {
 	Caret caret;
 	IME ime;
+	boolean blink, drawFlag;
 
 Canvas () {}
 
@@ -167,13 +171,70 @@ long /*int*/ gtk_commit (long /*int*/ imcontext, long /*int*/ text) {
 @Override
 long /*int*/ gtk_draw (long /*int*/ widget, long /*int*/ cairo) {
 	if ((state & OBSCURED) != 0) return 0;
-	boolean isFocus = caret != null && caret.isFocusCaret ();
-	if (isFocus) caret.killFocus ();
-	long /*int*/ result = super.gtk_draw (widget, cairo);
-	if (isFocus) caret.setFocus ();
+	long /*int*/ result;
+	if ( GTK.GTK_VERSION < OS.VERSION(3, 22, 0)) {
+		boolean isFocus = caret != null && caret.isFocusCaret ();
+		if (isFocus) caret.killFocus ();
+		 result = super.gtk_draw (widget, cairo);
+		if (isFocus) caret.setFocus ();
+	} else {
+		result = super.gtk_draw (widget, cairo);
+		/*
+		 *  blink is needed to be checked as gtk_draw() signals sent from other parts of the canvas
+		 *  can interfere with the blinking state. This will ensure that we are only draw/redrawing the
+		 *  caret when it is intended to. See Bug 517487.
+		 *
+		 *  Additionally, only draw the caret if it has focus. See bug 528819.
+		 */
+		if (caret != null && blink == true && caret.isFocusCaret()) {
+			drawCaret(widget,cairo);
+			blink = false;
+		}
+	}
 	return result;
 }
 
+private void drawCaret (long /*int*/ widget, long /*int*/ cairo) {
+	if(this.isDisposed()) return;
+	if (cairo == 0) error(SWT.ERROR_NO_HANDLES);
+	if (drawFlag) {
+		Cairo.cairo_save(cairo);
+		if (caret.image != null && !caret.image.isDisposed() && caret.image.mask == 0) {
+			Cairo.cairo_set_source_rgb(cairo, 1, 1, 1);
+			Cairo.cairo_set_operator(cairo, Cairo.CAIRO_OPERATOR_DIFFERENCE);
+			long /*int*/ surface = Cairo.cairo_get_target(cairo);
+			int nWidth = 0;
+			switch (Cairo.cairo_surface_get_type(surface)) {
+				case Cairo.CAIRO_SURFACE_TYPE_IMAGE:
+					nWidth = Cairo.cairo_image_surface_get_width(surface);
+					break;
+				case Cairo.CAIRO_SURFACE_TYPE_XLIB:
+					nWidth = Cairo.cairo_xlib_surface_get_width(surface);
+					break;
+			}
+			int nX = caret.x;
+			if ((style & SWT.MIRRORED) != 0) nX = getClientWidth () - nWidth - nX;
+			Cairo.cairo_translate(cairo, nX, caret.y);
+			Cairo.cairo_set_source_surface(cairo, caret.image.surface, 0, 0);
+			Cairo.cairo_paint(cairo);
+		} else {
+			Cairo.cairo_set_source_rgb(cairo, 1, 1, 1);
+			Cairo.cairo_set_operator(cairo, Cairo.CAIRO_OPERATOR_DIFFERENCE);
+			int nWidth = caret.width, nHeight = caret.height;
+			if (nWidth <= 0) nWidth = Caret.DEFAULT_WIDTH;
+			int nX = caret.x;
+			if ((style & SWT.MIRRORED) != 0) nX = getClientWidth () - nWidth - nX;
+			Cairo.cairo_rectangle(cairo, nX, caret.y, nWidth, nHeight);
+			}
+		Cairo.cairo_fill(cairo);
+		Cairo.cairo_restore(cairo);
+		drawFlag = false;
+	} else {
+		drawFlag = true;
+		}
+	return;
+}
+
 @Override
 long /*int*/ gtk_expose_event (long /*int*/ widget, long /*int*/ event) {
 	if ((state & OBSCURED) != 0) return 0;
@@ -278,10 +339,10 @@ void scrollInPixels (int destX, int destY, int x, int y, int width, int height,
 	if (isFocus) caret.killFocus ();
 	long /*int*/ window = paintWindow ();
 	long /*int*/ visibleRegion;
-	if (OS.GTK3) {
-		visibleRegion = OS.gdk_window_get_visible_region (window);
+	if (GTK.GTK3) {
+		visibleRegion = GDK.gdk_window_get_visible_region (window);
 	} else {
-		visibleRegion = OS.gdk_drawable_get_visible_region (window);
+		visibleRegion = GDK.gdk_drawable_get_visible_region (window);
 	}
 	GdkRectangle srcRect = new GdkRectangle ();
 	srcRect.x = x;
@@ -292,12 +353,12 @@ void scrollInPixels (int destX, int destY, int x, int y, int width, int height,
 	 * location the scrollbars are re-painted when scrolling, causing the
 	 * hopping effect. See bug 480458.
 	 */
-	if (OS.GTK_VERSION >= OS.VERSION(3, 16, 0)) {
+	if (GTK.GTK_VERSION >= OS.VERSION(3, 16, 0)) {
 		long /*int*/ hBarHandle = 0;
 		long /*int*/ vBarHandle = 0;
-		if (OS.GTK_IS_SCROLLED_WINDOW(scrolledHandle)) {
-			hBarHandle = OS.gtk_scrolled_window_get_hscrollbar (scrolledHandle);
-			vBarHandle = OS.gtk_scrolled_window_get_vscrollbar (scrolledHandle);
+		if (GTK.GTK_IS_SCROLLED_WINDOW(scrolledHandle)) {
+			hBarHandle = GTK.gtk_scrolled_window_get_hscrollbar (scrolledHandle);
+			vBarHandle = GTK.gtk_scrolled_window_get_vscrollbar (scrolledHandle);
 		}
 		GtkRequisition requisition = new GtkRequisition();
 		if (hBarHandle != 0) {
@@ -315,13 +376,13 @@ void scrollInPixels (int destX, int destY, int x, int y, int width, int height,
 	}
 	srcRect.width = width;
 	srcRect.height = height;
-	long /*int*/ copyRegion = OS.gdk_region_rectangle (srcRect);
-	OS.gdk_region_intersect(copyRegion, visibleRegion);
-	long /*int*/ invalidateRegion = OS.gdk_region_rectangle (srcRect);
-	OS.gdk_region_subtract (invalidateRegion, visibleRegion);
-	OS.gdk_region_offset (invalidateRegion, deltaX, deltaY);
+	long /*int*/ copyRegion = GDK.gdk_region_rectangle (srcRect);
+	GDK.gdk_region_intersect(copyRegion, visibleRegion);
+	long /*int*/ invalidateRegion = GDK.gdk_region_rectangle (srcRect);
+	GDK.gdk_region_subtract (invalidateRegion, visibleRegion);
+	GDK.gdk_region_offset (invalidateRegion, deltaX, deltaY);
 	GdkRectangle copyRect = new GdkRectangle();
-	OS.gdk_region_get_clipbox (copyRegion, copyRect);
+	GDK.gdk_region_get_clipbox (copyRegion, copyRect);
 	if (copyRect.width != 0 && copyRect.height != 0) {
 		update ();
 	}
@@ -331,28 +392,21 @@ void scrollInPixels (int destX, int destY, int x, int y, int width, int height,
 		redrawWidget (x, y, width, height, false, false, false);
 		redrawWidget (destX, destY, width, height, false, false, false);
 	} else {
-		if (OS.GTK3) {
-			long /*int*/ cairo = OS.gdk_cairo_create(window);
-			if (Cairo.cairo_version() < Cairo.CAIRO_VERSION_ENCODE(1, 12, 0)) {
-				OS.gdk_cairo_set_source_window(cairo, window, 0, 0);
-			} else {
-				Cairo.cairo_push_group(cairo);
-				OS.gdk_cairo_set_source_window(cairo, window, 0, 0);
-				Cairo.cairo_paint(cairo);
-				Cairo.cairo_pop_group_to_source(cairo);
-			}
-			double[] matrix = {1, 0, 0, 1, -deltaX, -deltaY};
-			Cairo.cairo_pattern_set_matrix(Cairo.cairo_get_source(cairo), matrix);
-			Cairo.cairo_rectangle(cairo, copyRect.x + deltaX, copyRect.y + deltaY, copyRect.width, copyRect.height);
-			Cairo.cairo_clip(cairo);
-			Cairo.cairo_paint(cairo);
-			Cairo.cairo_destroy(cairo);
+		long /*int*/ cairo = GDK.gdk_cairo_create(window);
+		if (Cairo.cairo_version() < Cairo.CAIRO_VERSION_ENCODE(1, 12, 0)) {
+			GDK.gdk_cairo_set_source_window(cairo, window, 0, 0);
 		} else {
-			long /*int*/ gdkGC = OS.gdk_gc_new (window);
-			OS.gdk_gc_set_exposures (gdkGC, true);
-			OS.gdk_draw_drawable (window, gdkGC, window, copyRect.x, copyRect.y, copyRect.x + deltaX, copyRect.y + deltaY, copyRect.width, copyRect.height);
-			OS.g_object_unref (gdkGC);
+			Cairo.cairo_push_group(cairo);
+			GDK.gdk_cairo_set_source_window(cairo, window, 0, 0);
+			Cairo.cairo_paint(cairo);
+			Cairo.cairo_pop_group_to_source(cairo);
 		}
+		double[] matrix = {1, 0, 0, 1, -deltaX, -deltaY};
+		Cairo.cairo_pattern_set_matrix(Cairo.cairo_get_source(cairo), matrix);
+		Cairo.cairo_rectangle(cairo, copyRect.x + deltaX, copyRect.y + deltaY, copyRect.width, copyRect.height);
+		Cairo.cairo_clip(cairo);
+		Cairo.cairo_paint(cairo);
+		Cairo.cairo_destroy(cairo);
 		boolean disjoint = (destX + width < x) || (x + width < destX) || (destY + height < y) || (y + height < destY);
 		if (disjoint) {
 			GdkRectangle rect = new GdkRectangle ();
@@ -360,7 +414,7 @@ void scrollInPixels (int destX, int destY, int x, int y, int width, int height,
 			rect.y = y;
 			rect.width = width;
 			rect.height = height;
-			OS.gdk_region_union_with_rect (invalidateRegion, rect);
+			GDK.gdk_region_union_with_rect (invalidateRegion, rect);
 		} else {
 			GdkRectangle rect = new GdkRectangle ();
 			if (deltaX != 0) {
@@ -370,7 +424,7 @@ void scrollInPixels (int destX, int destY, int x, int y, int width, int height,
 				rect.y = y;
 				rect.width = Math.abs(deltaX);
 				rect.height = height;
-				OS.gdk_region_union_with_rect (invalidateRegion, rect);
+				GDK.gdk_region_union_with_rect (invalidateRegion, rect);
 			}
 			if (deltaY != 0) {
 				int newY = destY - deltaY;
@@ -379,14 +433,14 @@ void scrollInPixels (int destX, int destY, int x, int y, int width, int height,
 				rect.y = newY;
 				rect.width = width;
 				rect.height = Math.abs(deltaY);
-				OS.gdk_region_union_with_rect (invalidateRegion, rect);
+				GDK.gdk_region_union_with_rect (invalidateRegion, rect);
 			}
 		}
-		OS.gdk_window_invalidate_region(window, invalidateRegion, all);
+		GDK.gdk_window_invalidate_region(window, invalidateRegion, all);
 	}
-	OS.gdk_region_destroy (visibleRegion);
-	OS.gdk_region_destroy (copyRegion);
-	OS.gdk_region_destroy (invalidateRegion);
+	GDK.gdk_region_destroy (visibleRegion);
+	GDK.gdk_region_destroy (copyRegion);
+	GDK.gdk_region_destroy (invalidateRegion);
 	if (all) {
 		Control [] children = _getChildren ();
 		for (int i=0; i<children.length; i++) {
@@ -403,7 +457,7 @@ void scrollInPixels (int destX, int destY, int x, int y, int width, int height,
 	 * Due to overlay drawing of scrollbars current method of scrolling leaves scrollbar and notifiers for them inside the canvas
 	 * after scroll. Fix is to redraw once done.
 	 */
-	if (OS.GTK_VERSION >= OS.VERSION(3, 16, 0)) {
+	if (GTK.GTK_VERSION >= OS.VERSION(3, 16, 0)) {
 		redraw(false);
 	}
 }
@@ -417,20 +471,6 @@ int setBounds (int x, int y, int width, int height, boolean move, boolean resize
 	return result;
 }
 
-@Override
-void setForegroundColor (long /*int*/ handle, GdkRGBA rgba) {
-	/* For CSS foreground colors (GTK3.16+) we need to make sure the default
-	 * background color stays set if no background color is specified. If no
-	 * background color has been set, set cssBackground to be COLOR_WIDGET_BACKGROUND.
-	 */
-	if (cssBackground == null) {
-		GdkRGBA defaultBackground = display.toGdkRGBA (display.COLOR_WIDGET_BACKGROUND);
-		cssBackground = "SwtFixed {background-color: " +
-				display.gtk_rgba_to_css_string(defaultBackground) + ";}";
-	}
-	super.setForegroundColor (handle, rgba);
-}
-
 /**
  * Sets the receiver's caret.
  * <p>
@@ -501,7 +541,7 @@ void updateCaret () {
 	rect.y = caret.y;
 	rect.width = caret.width;
 	rect.height = caret.height;
-	OS.gtk_im_context_set_cursor_location (imHandle, rect);
+	GTK.gtk_im_context_set_cursor_location (imHandle, rect);
 }
 
 }
