@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2016 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -124,21 +127,21 @@ protected void checkSubclass () {
 @Override
 void createHandle (int index) {
 	state |= HANDLE;
-	handle = OS.gtk_expander_new (null);
+	handle = GTK.gtk_expander_new (null);
 	if (handle == 0) error (SWT.ERROR_NO_HANDLES);
 	clientHandle = OS.g_object_new (display.gtk_fixed_get_type (), 0);
 	if (clientHandle == 0) error (SWT.ERROR_NO_HANDLES);
-	OS.gtk_container_add (handle, clientHandle);
-	boxHandle = gtk_box_new (OS.GTK_ORIENTATION_HORIZONTAL, false, 4);
+	GTK.gtk_container_add (handle, clientHandle);
+	boxHandle = gtk_box_new (GTK.GTK_ORIENTATION_HORIZONTAL, false, 4);
 	if (boxHandle == 0) error (SWT.ERROR_NO_HANDLES);
-	labelHandle = OS.gtk_label_new (null);
+	labelHandle = GTK.gtk_label_new (null);
 	if (labelHandle == 0) error (SWT.ERROR_NO_HANDLES);
-	imageHandle = OS.gtk_image_new ();
+	imageHandle = GTK.gtk_image_new ();
 	if (imageHandle == 0) error (SWT.ERROR_NO_HANDLES);
-	OS.gtk_container_add (boxHandle, imageHandle);
-	OS.gtk_container_add (boxHandle, labelHandle);
-	OS.gtk_expander_set_label_widget (handle, boxHandle);
-	OS.gtk_widget_set_can_focus (handle, true);
+	GTK.gtk_container_add (boxHandle, imageHandle);
+	GTK.gtk_container_add (boxHandle, labelHandle);
+	GTK.gtk_expander_set_label_widget (handle, boxHandle);
+	GTK.gtk_widget_set_can_focus (handle, true);
 }
 
 @Override
@@ -159,7 +162,7 @@ void deregister() {
 
 @Override
 void release (boolean destroy) {
-	if (OS.GTK3) {
+	if (GTK.GTK3) {
 		//454940 ExpandBar DND fix.
 		//Since controls are now nested under the Item,
 		//Item is responsible for it's release.
@@ -289,8 +292,15 @@ public int getHeaderHeight () {
 int getHeaderHeightInPixels () {
 	checkWidget ();
 	GtkAllocation allocation = new GtkAllocation ();
-	OS.gtk_widget_get_allocation (handle, allocation);
-	return allocation.height - (expanded ? height : 0);
+	GTK.gtk_widget_get_allocation (handle, allocation);
+	// allocation.height normally returns the header height instead of the whole
+	// widget itself. This is to prevent situations where allocation.height actually
+	// returns the correct header height.
+	int headerHeight = allocation.height - (expanded ? height : 0);
+	if (expanded && headerHeight < 0) {
+		return allocation.height;
+	}
+	return headerHeight;
 }
 /**
  * Gets the height of the receiver.
@@ -342,7 +352,7 @@ int getPreferredWidth (GC gc) {
 long /*int*/ gtk_activate (long /*int*/ widget) {
 	Event event = new Event ();
 	event.item = this;
-	int type = OS.gtk_expander_get_expanded (handle) ? SWT.Collapse : SWT.Expand;
+	int type = GTK.gtk_expander_get_expanded (handle) ? SWT.Collapse : SWT.Expand;
 	parent.sendEvent (type, event);
 	return 0;
 }
@@ -355,7 +365,7 @@ long /*int*/ gtk_button_press_event (long /*int*/ widget, long /*int*/ event) {
 
 @Override
 long /*int*/ gtk_focus_out_event (long /*int*/ widget, long /*int*/ event) {
-	OS.gtk_widget_set_can_focus (handle, false);
+	GTK.gtk_widget_set_can_focus (handle, false);
 	parent.lastFocus = this;
 	return 0;
 }
@@ -373,7 +383,7 @@ long /*int*/ gtk_enter_notify_event (long /*int*/ widget, long /*int*/ event) {
 }
 
 boolean hasFocus () {
-	return OS.gtk_widget_has_focus (handle);
+	return GTK.gtk_widget_has_focus (handle);
 }
 
 @Override
@@ -417,46 +427,57 @@ void releaseWidget () {
 
 void resizeControl (int yScroll) {
 	if (control != null && !control.isDisposed ()) {
-		boolean visible = OS.gtk_expander_get_expanded (handle);
-		if (visible) {
-			GtkAllocation allocation = new GtkAllocation ();
-			OS.gtk_widget_get_allocation (clientHandle, allocation);
+		boolean visible = GTK.gtk_expander_get_expanded (handle);
+		GtkAllocation allocation = new GtkAllocation ();
+		GTK.gtk_widget_get_allocation (clientHandle, allocation);
 
-			//454940 change in hierarchy
-			/* SWT's calls to gtk_widget_size_allocate and gtk_widget_set_allocation
-			* causes GTK+ to move the clientHandle's SwtFixed down by the size of the label.
-			* These calls can come up from 'shell' and ExpandItem has no control over these calls.
-			* This is an undesired side-effect. Client handle's x & y positions should never
-			* be incremented as this is an internal sub-container.
-			* As of GTK3, the hierarchy is changed, this affected child-size allocation and a fix
-			* is now neccessary.
-			* See also other 454940 notes and similar fix in: 453827 */
-			int x = (OS.GTK3) ? 0 : allocation.x;
-			int y = (OS.GTK3) ? 0 : allocation.y;
+		//454940 change in hierarchy
+		/* SWT's calls to gtk_widget_size_allocate and gtk_widget_set_allocation
+		* causes GTK+ to move the clientHandle's SwtFixed down by the size of the label.
+		* These calls can come up from 'shell' and ExpandItem has no control over these calls.
+		* This is an undesired side-effect. Client handle's x & y positions should never
+		* be incremented as this is an internal sub-container.
+		* As of GTK3, the hierarchy is changed, this affected child-size allocation and a fix
+		* is now neccessary.
+		* See also other 454940 notes and similar fix in: 453827 */
+		int x = (GTK.GTK3) ? 0 : allocation.x;
+		int y = (GTK.GTK3) ? 0 : allocation.y;
 
-			if (x != -1 && y != -1) {
-				int width = allocation.width;
-				int height = allocation.height;
-				int [] property = new int [1];
-				OS.gtk_widget_style_get (handle, OS.focus_line_width, property, 0);
-				y += property [0] * 2;
-				height -= property [0] * 2;
+		if (x != -1 && y != -1) {
+			int width = allocation.width;
+			int height = allocation.height;
+			int [] property = new int [1];
+			GTK.gtk_widget_style_get (handle, OS.focus_line_width, property, 0);
+			y += property [0] * 2;
+			height -= property [0] * 2;
 
-				/*
-				* Feature in GTK. When the ExpandBar is resize too small the control
-				* shows up on top of the vertical scrollbar. This happen because the
-				* GtkExpander does not set the size of child smaller than the request
-				* size of its parent and because the control is not parented in the
-				* hierarchy of the GtkScrolledWindow.
-				* The fix is calculate the width ourselves when the scrollbar is visible.
-				*/
-				ScrollBar vBar = parent.verticalBar;
-				if (vBar != null) {
-					if (OS.gtk_widget_get_visible (vBar.handle)) {
-						OS.gtk_widget_get_allocation (parent.scrolledHandle, allocation);
-						width = allocation.width - parent.vScrollBarWidth () - 2 * parent.spacing;
-					}
+			/*
+			* Feature in GTK. When the ExpandBar is resize too small the control
+			* shows up on top of the vertical scrollbar. This happen because the
+			* GtkExpander does not set the size of child smaller than the request
+			* size of its parent and because the control is not parented in the
+			* hierarchy of the GtkScrolledWindow.
+			* The fix is calculate the width ourselves when the scrollbar is visible.
+			*/
+			ScrollBar vBar = parent.verticalBar;
+			if (vBar != null) {
+				if (GTK.gtk_widget_get_visible (vBar.handle)) {
+					GTK.gtk_widget_get_allocation (parent.scrolledHandle, allocation);
+					width = allocation.width - parent.vScrollBarWidth () - 2 * parent.spacing;
 				}
+			}
+			// Bug 479242: Bound calculation is correct without needing to use yScroll in GTK3
+			if (GTK.GTK3) {
+				/*
+				 * Bug 538114: ExpandBar has no content until resized or collapsed/expanded.
+				 * When widget is first created inside ExpandItem's control, the size is allocated
+				 * to be zero, and the widget is never shown during a layout operation, similar to
+				 * Bug 487757. The fix is to show the control before setting any bounds.
+				 */
+				if (visible) GTK.gtk_widget_show(control.topHandle ());
+				control.setBounds (x, y, width, Math.max (0, height), true, true);
+			}
+			else {
 				control.setBounds (x, y - yScroll, width, Math.max (0, height), true, true);
 			}
 		}
@@ -512,14 +533,14 @@ public void setControl (Control control) {
 	this.control = control;
 	if (control != null) {
 		control.setVisible (expanded);
-		if (OS.GTK3) {
+		if (GTK.GTK3) {
 			//454940 ExpandBar DND fix.
 			//Reparenting on the GTK side.
 			//Proper hierachy on gtk side is required for DND to function properly.
 			//As ExpandItem's child can be created before the ExpandItem, our only
 			//option is to reparent the child upon the setControl(..) call.
 			//This is simmilar to TabFolder.
-			gtk_widget_reparent (control.topHandle (), clientHandle ());
+			Control.gtk_widget_reparent (control, clientHandle ());
 		}
 	}
 	parent.layoutItems (0, true);
@@ -543,18 +564,18 @@ long /*int*/ clientHandle () {
 public void setExpanded (boolean expanded) {
 	checkWidget ();
 	this.expanded = expanded;
-	OS.gtk_expander_set_expanded (handle, expanded);
+	GTK.gtk_expander_set_expanded (handle, expanded);
 	parent.layoutItems (0, true);
 }
 
 boolean setFocus () {
-	if (!OS.gtk_widget_get_child_visible (handle)) return false;
-	OS.gtk_widget_set_can_focus (handle, true);
-	OS.gtk_widget_grab_focus (handle);
+	if (!GTK.gtk_widget_get_child_visible (handle)) return false;
+	GTK.gtk_widget_set_can_focus (handle, true);
+	GTK.gtk_widget_grab_focus (handle);
 	// widget could be disposed at this point
 	if (isDisposed ()) return false;
-	boolean result = OS.gtk_widget_is_focus (handle);
-	if (!result) OS.gtk_widget_set_can_focus (handle, false);
+	boolean result = GTK.gtk_widget_is_focus (handle);
+	if (!result) GTK.gtk_widget_set_can_focus (handle, false);
 	return result;
 }
 
@@ -565,19 +586,17 @@ void setFontDescription (long /*int*/ font) {
 }
 
 void setForegroundColor (GdkColor color) {
-	if (OS.GTK_VERSION >= OS.VERSION(3, 16, 0)) {
-		GdkRGBA rgba = null;
-		if (color != null) {
-			rgba = display.toGdkRGBA (color);
-		}
-		parent.setForegroundColor (handle, rgba);
-		if (labelHandle != 0) parent.setForegroundColor (labelHandle, rgba);
-		if (imageHandle != 0) parent.setForegroundColor (imageHandle, rgba);
-	} else {
-		setForegroundColor (handle, color);
-		if (labelHandle != 0) setForegroundColor (labelHandle, color);
-		if (imageHandle != 0) setForegroundColor (imageHandle, color);
-	}
+	assert !GTK.GTK3 : "GTK2 code was run by GTK3";
+	setForegroundColor (handle, color);
+	if (labelHandle != 0) setForegroundColor (labelHandle, color);
+	if (imageHandle != 0) setForegroundColor (imageHandle, color);
+}
+
+void setForegroundRGBA (GdkRGBA rgba) {
+	assert GTK.GTK3 : "GTK3 code was run by GTK2";
+	parent.setForegroundGdkRGBA (handle, rgba);
+	if (labelHandle != 0) parent.setForegroundGdkRGBA (labelHandle, rgba);
+	if (imageHandle != 0) parent.setForegroundGdkRGBA (imageHandle, rgba);
 }
 
 /**
@@ -600,7 +619,7 @@ void setHeightInPixels (int height) {
 	checkWidget ();
 	if (height < 0) return;
 	this.height = height;
-	OS.gtk_widget_set_size_request (clientHandle, -1, height);
+	GTK.gtk_widget_set_size_request (clientHandle, -1, height);
 	parent.layoutItems (0, false);
 }
 
@@ -615,12 +634,12 @@ public void setImage (Image image) {
 		int imageIndex = imageList.add (image);
 		long /*int*/ pixbuf = imageList.getPixbuf (imageIndex);
 		gtk_image_set_from_pixbuf (imageHandle, pixbuf);
-		if (text.length () == 0) OS.gtk_widget_hide (labelHandle);
-		OS.gtk_widget_show (imageHandle);
+		if (text.length () == 0) GTK.gtk_widget_hide (labelHandle);
+		GTK.gtk_widget_show (imageHandle);
 	} else {
 		gtk_image_set_from_pixbuf (imageHandle, 0);
-		OS.gtk_widget_show (labelHandle);
-		OS.gtk_widget_hide (imageHandle);
+		GTK.gtk_widget_show (labelHandle);
+		GTK.gtk_widget_hide (imageHandle);
 	}
 }
 
@@ -628,35 +647,35 @@ public void setImage (Image image) {
 void setOrientation (boolean create) {
 	super.setOrientation (create);
 	if ((parent.style & SWT.RIGHT_TO_LEFT) != 0 || !create) {
-		int dir = (parent.style & SWT.RIGHT_TO_LEFT) != 0 ? OS.GTK_TEXT_DIR_RTL : OS.GTK_TEXT_DIR_LTR;
-		OS.gtk_widget_set_direction (handle, dir);
-		OS.gtk_container_forall (handle, display.setDirectionProc, dir);
+		int dir = (parent.style & SWT.RIGHT_TO_LEFT) != 0 ? GTK.GTK_TEXT_DIR_RTL : GTK.GTK_TEXT_DIR_LTR;
+		GTK.gtk_widget_set_direction (handle, dir);
+		GTK.gtk_container_forall (handle, display.setDirectionProc, dir);
 	}
 }
 
 @Override
 public void setText (String string) {
 	super.setText (string);
-	byte [] buffer = Converter.wcsToMbcs (null, string, true);
-	OS.gtk_label_set_text (labelHandle, buffer);
+	byte [] buffer = Converter.wcsToMbcs (string, true);
+	GTK.gtk_label_set_text (labelHandle, buffer);
 }
 
 void showWidget (int index) {
-		OS.gtk_widget_show (handle);
-		OS.gtk_widget_show (clientHandle);
+		GTK.gtk_widget_show (handle);
+		GTK.gtk_widget_show (clientHandle);
 		if (labelHandle != 0)
-			OS.gtk_widget_show (labelHandle);
+			GTK.gtk_widget_show (labelHandle);
 		if (boxHandle != 0)
-			OS.gtk_widget_show (boxHandle);
-		OS.gtk_container_add (parent.handle, handle);
-		OS.gtk_box_set_child_packing (parent.handle, handle, false, false, 0, OS.GTK_PACK_START);
+			GTK.gtk_widget_show (boxHandle);
+		GTK.gtk_container_add (parent.handle, handle);
+		GTK.gtk_box_set_child_packing (parent.handle, handle, false, false, 0, GTK.GTK_PACK_START);
 }
 
 @Override
 long /*int*/ windowProc (long /*int*/ handle, long /*int*/ user_data) {
 	switch ((int)/*64*/user_data) {
 		case ACTIVATE_INVERSE: {
-			expanded = OS.gtk_expander_get_expanded (handle);
+			expanded = GTK.gtk_expander_get_expanded (handle);
 			parent.layoutItems (0, false);
 			return 0;
 		}
