@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2016 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2018 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -63,7 +66,7 @@ public class Spinner extends Composite {
 	* to stop the compiler from inlining.
 	*/
 	static {
-		LIMIT = 0x7FFFFFFF;
+		LIMIT = 0xFFFF;
 	}
 	/* Spinner uses non-standard CSS to set its background color, so we need
 	 * a global variable to keep track of its background color.
@@ -209,15 +212,15 @@ Point computeSizeInPixels (int wHint, int hHint, boolean changed) {
 	if (wHint != SWT.DEFAULT && wHint < 0) wHint = 0;
 	if (hHint != SWT.DEFAULT && hHint < 0) hHint = 0;
 	int[] w = new int [1], h = new int [1];
-	OS.gtk_widget_realize (handle);
-	long /*int*/ layout = OS.gtk_entry_get_layout (handle);
-	long /*int*/ hAdjustment = OS.gtk_spin_button_get_adjustment (handle);
-	double upper = OS.gtk_adjustment_get_upper (hAdjustment);
-	int digits = OS.gtk_spin_button_get_digits (handle);
+	GTK.gtk_widget_realize (handle);
+	long /*int*/ layout = GTK.gtk_entry_get_layout (handle);
+	long /*int*/ hAdjustment = GTK.gtk_spin_button_get_adjustment (handle);
+	double upper = GTK.gtk_adjustment_get_upper (hAdjustment);
+	int digits = GTK.gtk_spin_button_get_digits (handle);
 	for (int i = 0; i < digits; i++) upper *= 10;
 	String string = String.valueOf ((int) upper);
 	if (digits > 0) {
-		StringBuffer buffer = new StringBuffer ();
+		StringBuilder buffer = new StringBuilder ();
 		buffer.append (string);
 		buffer.append (getDecimalSeparator ());
 		int count = digits - string.length ();
@@ -227,19 +230,19 @@ Point computeSizeInPixels (int wHint, int hHint, boolean changed) {
 		}
 		string = buffer.toString ();
 	}
-	byte [] buffer1 = Converter.wcsToMbcs (null, string, false);
+	byte [] buffer1 = Converter.wcsToMbcs (string, false);
 	long /*int*/ ptr = OS.pango_layout_get_text (layout);
-	int length = OS.strlen (ptr);
+	int length = C.strlen (ptr);
 	byte [] buffer2 = new byte [length];
-	OS.memmove (buffer2, ptr, length);
+	C.memmove (buffer2, ptr, length);
 	OS.pango_layout_set_text (layout, buffer1, buffer1.length);
 	int width, height = 0 ;
-	OS.gtk_widget_realize (handle);
-	if (OS.GTK3) {
-		OS.gtk_widget_set_size_request (handle, wHint, hHint);
+	GTK.gtk_widget_realize (handle);
+	if (GTK.GTK3) {
+		GTK.gtk_widget_set_size_request (handle, wHint, hHint);
 		GtkRequisition requisition = new GtkRequisition ();
-		OS.gtk_widget_get_preferred_size (handle, requisition, null);
-		width = wHint == SWT.DEFAULT ? requisition.width : wHint;
+		GTK.gtk_widget_get_preferred_size (handle, requisition, null);
+		width = wHint == SWT.DEFAULT ? requisition.width : Math.max(wHint, requisition.width);
 		height = hHint == SWT.DEFAULT ? requisition.height : hHint;
 	} else {
 		OS.pango_layout_get_pixel_size (layout, w, h);
@@ -256,19 +259,19 @@ Rectangle computeTrimInPixels (int x, int y, int width, int height) {
 	checkWidget ();
 	int xborder = 0, yborder = 0;
 	Rectangle trim = super.computeTrimInPixels (x, y, width, height);
-	if (OS.GTK3) {
+	if (GTK.GTK3) {
 		GtkBorder tmp = new GtkBorder();
-		long /*int*/ context = OS.gtk_widget_get_style_context (handle);
-		if (OS.GTK_VERSION < OS.VERSION(3, 18, 0)) {
-			OS.gtk_style_context_get_padding (context, OS.GTK_STATE_FLAG_NORMAL, tmp);
+		long /*int*/ context = GTK.gtk_widget_get_style_context (handle);
+		if (GTK.GTK_VERSION < OS.VERSION(3, 18, 0)) {
+			GTK.gtk_style_context_get_padding (context, GTK.GTK_STATE_FLAG_NORMAL, tmp);
 		} else {
-			OS.gtk_style_context_get_padding (context, OS.gtk_widget_get_state_flags(handle), tmp);
+			GTK.gtk_style_context_get_padding (context, GTK.gtk_widget_get_state_flags(handle), tmp);
 		}
 		if ((style & SWT.BORDER) != 0) {
-			if (OS.GTK_VERSION < OS.VERSION(3, 18, 0)) {
-				OS.gtk_style_context_get_border (context, OS.GTK_STATE_FLAG_NORMAL, tmp);
+			if (GTK.GTK_VERSION < OS.VERSION(3, 18, 0)) {
+				GTK.gtk_style_context_get_border (context, GTK.GTK_STATE_FLAG_NORMAL, tmp);
 			} else {
-				OS.gtk_style_context_get_border (context, OS.gtk_widget_get_state_flags(handle), tmp);
+				GTK.gtk_style_context_get_border (context, GTK.gtk_widget_get_state_flags(handle), tmp);
 			}
 			trim.x -= tmp.left;
 			trim.y -= tmp.top;
@@ -288,9 +291,9 @@ Rectangle computeTrimInPixels (int x, int y, int width, int height) {
 		trim.width += arrowSize + (2 * thickness.x);
 	}
 	int [] property = new int [1];
-	OS.gtk_widget_style_get (handle, OS.interior_focus, property, 0);
+	GTK.gtk_widget_style_get (handle, OS.interior_focus, property, 0);
 	if (property [0] == 0) {
-		OS.gtk_widget_style_get (handle, OS.focus_line_width, property, 0);
+		GTK.gtk_widget_style_get (handle, OS.focus_line_width, property, 0);
 		xborder += property [0];
 		yborder += property [0];
 	}
@@ -319,7 +322,7 @@ Rectangle computeTrimInPixels (int x, int y, int width, int height) {
  */
 public void copy () {
 	checkWidget ();
-	OS.gtk_editable_copy_clipboard (handle);
+	GTK.gtk_editable_copy_clipboard (handle);
 }
 
 @Override
@@ -327,23 +330,23 @@ void createHandle (int index) {
 	state |= HANDLE | MENU;
 	fixedHandle = OS.g_object_new (display.gtk_fixed_get_type (), 0);
 	if (fixedHandle == 0) error (SWT.ERROR_NO_HANDLES);
-	OS.gtk_widget_set_has_window (fixedHandle, true);
-	long /*int*/ adjustment = OS.gtk_adjustment_new (0, 0, 100, 1, 10, 0);
+	GTK.gtk_widget_set_has_window (fixedHandle, true);
+	long /*int*/ adjustment = GTK.gtk_adjustment_new (0, 0, 100, 1, 10, 0);
 	if (adjustment == 0) error (SWT.ERROR_NO_HANDLES);
-	handle = OS.gtk_spin_button_new (adjustment, climbRate, 0);
+	handle = GTK.gtk_spin_button_new (adjustment, climbRate, 0);
 	if (handle == 0) error (SWT.ERROR_NO_HANDLES);
-	OS.gtk_container_add (fixedHandle, handle);
-	OS.gtk_editable_set_editable (handle, (style & SWT.READ_ONLY) == 0);
-	if (OS.GTK_VERSION <= OS.VERSION(3, 20, 0)) {
-		OS.gtk_entry_set_has_frame (handle, (style & SWT.BORDER) != 0);
+	GTK.gtk_container_add (fixedHandle, handle);
+	GTK.gtk_editable_set_editable (handle, (style & SWT.READ_ONLY) == 0);
+	if (GTK.GTK_VERSION <= OS.VERSION(3, 20, 0)) {
+		GTK.gtk_entry_set_has_frame (handle, (style & SWT.BORDER) != 0);
 	}
-	OS.gtk_spin_button_set_wrap (handle, (style & SWT.WRAP) != 0);
-	if (OS.GTK3) {
+	GTK.gtk_spin_button_set_wrap (handle, (style & SWT.WRAP) != 0);
+	if (GTK.GTK3) {
 		imContext = OS.imContextLast();
 	}
 	// In GTK 3 font description is inherited from parent widget which is not how SWT has always worked,
 	// reset to default font to get the usual behavior
-	if (OS.GTK3) {
+	if (GTK.GTK3) {
 		setFontDescription(defaultFont().handle);
 	}
 }
@@ -362,7 +365,12 @@ void createHandle (int index) {
  */
 public void cut () {
 	checkWidget ();
-	OS.gtk_editable_cut_clipboard (handle);
+	GTK.gtk_editable_cut_clipboard (handle);
+}
+
+@Override
+GdkRGBA defaultBackground () {
+	return display.getSystemColor(SWT.COLOR_LIST_BACKGROUND).handleRGBA;
 }
 
 @Override
@@ -384,12 +392,12 @@ long /*int*/ enterExitHandle () {
 
 @Override
 boolean filterKey (int keyval, long /*int*/ event) {
-	int time = OS.gdk_event_get_time (event);
+	int time = GDK.gdk_event_get_time (event);
 	if (time != lastEventTime) {
 		lastEventTime = time;
 		long /*int*/ imContext = imContext ();
 		if (imContext != 0) {
-			return OS.gtk_im_context_filter_keypress (imContext, event);
+			return GTK.gtk_im_context_filter_keypress (imContext, event);
 		}
 	}
 	gdkEventKey = event;
@@ -408,7 +416,7 @@ void fixIM () {
 	if (gdkEventKey != 0 && gdkEventKey != -1) {
 		long /*int*/ imContext = imContext ();
 		if (imContext != 0) {
-			OS.gtk_im_context_filter_keypress (imContext, gdkEventKey);
+			GTK.gtk_im_context_filter_keypress (imContext, gdkEventKey);
 			gdkEventKey = -1;
 			return;
 		}
@@ -417,8 +425,9 @@ void fixIM () {
 }
 
 @Override
-GdkColor getBackgroundColor () {
-	return getBaseColor ();
+GdkColor getBackgroundGdkColor () {
+	assert !GTK.GTK3 : "GTK2 code was run by GTK3";
+	return getBaseGdkColor ();
 }
 
 @Override
@@ -431,7 +440,8 @@ int getBorderWidthInPixels () {
 }
 
 @Override
-GdkColor getForegroundColor () {
+GdkColor getForegroundGdkColor () {
+	assert !GTK.GTK3 : "GTK2 code was run by GTK3";
 	return getTextColor ();
 }
 
@@ -448,9 +458,9 @@ GdkColor getForegroundColor () {
  */
 public int getIncrement () {
 	checkWidget ();
-	long /*int*/ hAdjustment = OS.gtk_spin_button_get_adjustment (handle);
-	int digits = OS.gtk_spin_button_get_digits (handle);
-	double value = OS.gtk_adjustment_get_step_increment (hAdjustment);
+	long /*int*/ hAdjustment = GTK.gtk_spin_button_get_adjustment (handle);
+	int digits = GTK.gtk_spin_button_get_digits (handle);
+	double value = GTK.gtk_adjustment_get_step_increment (hAdjustment);
 	for (int i = 0; i < digits; i++) value *= 10;
 	return (int) (value > 0 ? value + 0.5 : value - 0.5);
 }
@@ -467,9 +477,9 @@ public int getIncrement () {
  */
 public int getMaximum () {
 	checkWidget ();
-	long /*int*/ hAdjustment = OS.gtk_spin_button_get_adjustment (handle);
-	int digits = OS.gtk_spin_button_get_digits (handle);
-	double value = OS.gtk_adjustment_get_upper (hAdjustment);
+	long /*int*/ hAdjustment = GTK.gtk_spin_button_get_adjustment (handle);
+	int digits = GTK.gtk_spin_button_get_digits (handle);
+	double value = GTK.gtk_adjustment_get_upper (hAdjustment);
 	for (int i = 0; i < digits; i++) value *= 10;
 	return (int) (value > 0 ? value + 0.5 : value - 0.5);
 }
@@ -486,9 +496,9 @@ public int getMaximum () {
  */
 public int getMinimum () {
 	checkWidget ();
-	long /*int*/ hAdjustment = OS.gtk_spin_button_get_adjustment (handle);
-	int digits = OS.gtk_spin_button_get_digits (handle);
-	double value = OS.gtk_adjustment_get_lower (hAdjustment);
+	long /*int*/ hAdjustment = GTK.gtk_spin_button_get_adjustment (handle);
+	int digits = GTK.gtk_spin_button_get_digits (handle);
+	double value = GTK.gtk_adjustment_get_lower (hAdjustment);
 	for (int i = 0; i < digits; i++) value *= 10;
 	return (int) (value > 0 ? value + 0.5 : value - 0.5);
 }
@@ -506,9 +516,9 @@ public int getMinimum () {
  */
 public int getPageIncrement () {
 	checkWidget ();
-	long /*int*/ hAdjustment = OS.gtk_spin_button_get_adjustment (handle);
-	int digits = OS.gtk_spin_button_get_digits (handle);
-	double value = OS.gtk_adjustment_get_page_increment (hAdjustment);
+	long /*int*/ hAdjustment = GTK.gtk_spin_button_get_adjustment (handle);
+	int digits = GTK.gtk_spin_button_get_digits (handle);
+	double value = GTK.gtk_adjustment_get_page_increment (hAdjustment);
 	for (int i = 0; i < digits; i++) value *= 10;
 	return (int) (value > 0 ? value + 0.5 : value - 0.5);
 }
@@ -525,9 +535,9 @@ public int getPageIncrement () {
  */
 public int getSelection () {
 	checkWidget ();
-	long /*int*/ hAdjustment = OS.gtk_spin_button_get_adjustment (handle);
-	int digits = OS.gtk_spin_button_get_digits (handle);
-	double value = OS.gtk_adjustment_get_value (hAdjustment);
+	long /*int*/ hAdjustment = GTK.gtk_spin_button_get_adjustment (handle);
+	int digits = GTK.gtk_spin_button_get_digits (handle);
+	double value = GTK.gtk_adjustment_get_value (hAdjustment);
 	for (int i = 0; i < digits; i++) value *= 10;
 	return (int) (value > 0 ? value + 0.5 : value - 0.5);
 }
@@ -548,12 +558,12 @@ public int getSelection () {
  */
 public String getText () {
 	checkWidget ();
-	long /*int*/ str = OS.gtk_entry_get_text (handle);
+	long /*int*/ str = GTK.gtk_entry_get_text (handle);
 	if (str == 0) return "";
-	int length = OS.strlen (str);
+	int length = C.strlen (str);
 	byte [] buffer = new byte [length];
-	OS.memmove (buffer, str, length);
-	return new String (Converter.mbcsToWcs (null, buffer));
+	C.memmove (buffer, str, length);
+	return new String (Converter.mbcsToWcs (buffer));
 }
 
 /**
@@ -575,8 +585,8 @@ public String getText () {
  */
 public int getTextLimit () {
 	checkWidget ();
-	int limit = OS.gtk_entry_get_max_length (handle);
-	return limit == 0 ? 0xFFFF : limit;
+	int limit = GTK.gtk_entry_get_max_length (handle);
+	return limit == 0 ? LIMIT : limit;
 }
 
 /**
@@ -591,15 +601,15 @@ public int getTextLimit () {
  */
 public int getDigits () {
 	checkWidget ();
-	return OS.gtk_spin_button_get_digits (handle);
+	return GTK.gtk_spin_button_get_digits (handle);
 }
 
 String getDecimalSeparator () {
 	long /*int*/ ptr = OS.localeconv_decimal_point ();
-	int length = OS.strlen (ptr);
+	int length = C.strlen (ptr);
 	byte [] buffer = new byte [length];
-	OS.memmove (buffer, ptr, length);
-	return new String (Converter.mbcsToWcs (null, buffer));
+	C.memmove (buffer, ptr, length);
+	return new String (Converter.mbcsToWcs (buffer));
 }
 
 @Override
@@ -610,17 +620,18 @@ long /*int*/ gtk_activate (long /*int*/ widget) {
 
 @Override
 long /*int*/ gtk_changed (long /*int*/ widget) {
-	long /*int*/ str = OS.gtk_entry_get_text (handle);
-	int length = OS.strlen (str);
+	long /*int*/ str = GTK.gtk_entry_get_text (handle);
+	int length = C.strlen (str);
 	if (length > 0) {
 		long /*int*/ [] endptr = new long /*int*/ [1];
 		double value = OS.g_strtod (str, endptr);
-		if (endptr [0] == str + length) {
-			long /*int*/ hAdjustment = OS.gtk_spin_button_get_adjustment (handle);
+		int valueLength = (getDigits() == 0) ? String.valueOf((int)value).length() : String.valueOf(value).length();
+		if ((endptr [0] == str + length) && valueLength == length) {
+			long /*int*/ hAdjustment = GTK.gtk_spin_button_get_adjustment (handle);
 			GtkAdjustment adjustment = new GtkAdjustment ();
 			gtk_adjustment_get (hAdjustment, adjustment);
 			if (value != adjustment.value && adjustment.lower <= value && value <= adjustment.upper) {
-				OS.gtk_spin_button_update (handle);
+				GTK.gtk_spin_button_update (handle);
 			}
 		}
 	}
@@ -633,16 +644,16 @@ long /*int*/ gtk_changed (long /*int*/ widget) {
 	* is to post the modify event when the user is typing.
 	*/
 	boolean keyPress = false;
-	long /*int*/ eventPtr = OS.gtk_get_current_event ();
+	long /*int*/ eventPtr = GTK.gtk_get_current_event ();
 	if (eventPtr != 0) {
 		GdkEventKey gdkEvent = new GdkEventKey ();
 		OS.memmove (gdkEvent, eventPtr, GdkEventKey.sizeof);
 		switch (gdkEvent.type) {
-			case OS.GDK_KEY_PRESS:
+			case GDK.GDK_KEY_PRESS:
 				keyPress = true;
 				break;
 		}
-		OS.gdk_event_free (eventPtr);
+		GDK.gdk_event_free (eventPtr);
 	}
 	if (keyPress) {
 		postEvent (SWT.Modify);
@@ -655,12 +666,12 @@ long /*int*/ gtk_changed (long /*int*/ widget) {
 @Override
 long /*int*/ gtk_commit (long /*int*/ imContext, long /*int*/ text) {
 	if (text == 0) return 0;
-	if (!OS.gtk_editable_get_editable (handle)) return 0;
-	int length = OS.strlen (text);
+	if (!GTK.gtk_editable_get_editable (handle)) return 0;
+	int length = C.strlen (text);
 	if (length == 0) return 0;
 	byte [] buffer = new byte [length];
-	OS.memmove (buffer, text, length);
-	char [] chars = Converter.mbcsToWcs (null, buffer);
+	C.memmove (buffer, text, length);
+	char [] chars = Converter.mbcsToWcs (buffer);
 	char [] newChars = sendIMKeyEvent (SWT.KeyDown, null, chars);
 	if (newChars == null) return 0;
 	/*
@@ -672,20 +683,20 @@ long /*int*/ gtk_commit (long /*int*/ imContext, long /*int*/ text) {
 	*/
 	fixStart = fixEnd = -1;
 	OS.g_signal_handlers_block_matched (imContext, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, COMMIT);
-	int id = OS.g_signal_lookup (OS.commit, OS.gtk_im_context_get_type ());
+	int id = OS.g_signal_lookup (OS.commit, GTK.gtk_im_context_get_type ());
 	int mask =  OS.G_SIGNAL_MATCH_DATA | OS.G_SIGNAL_MATCH_ID;
 	OS.g_signal_handlers_unblock_matched (imContext, mask, id, 0, 0, 0, handle);
 	if (newChars == chars) {
 		OS.g_signal_emit_by_name (imContext, OS.commit, text);
 	} else {
-		buffer = Converter.wcsToMbcs (null, newChars, true);
+		buffer = Converter.wcsToMbcs (newChars, true);
 		OS.g_signal_emit_by_name (imContext, OS.commit, buffer);
 	}
 	OS.g_signal_handlers_unblock_matched (imContext, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, COMMIT);
 	OS.g_signal_handlers_block_matched (imContext, mask, id, 0, 0, 0, handle);
 	if (fixStart != -1 && fixEnd != -1) {
-		OS.gtk_editable_set_position (handle, fixStart);
-		OS.gtk_editable_select_region (handle, fixStart, fixEnd);
+		GTK.gtk_editable_set_position (handle, fixStart);
+		GTK.gtk_editable_select_region (handle, fixStart, fixEnd);
 	}
 	fixStart = fixEnd = -1;
 	return 0;
@@ -694,7 +705,7 @@ long /*int*/ gtk_commit (long /*int*/ imContext, long /*int*/ text) {
 @Override
 long /*int*/ gtk_delete_text (long /*int*/ widget, long /*int*/ start_pos, long /*int*/ end_pos) {
 	if (!hooks (SWT.Verify) && !filters (SWT.Verify)) return 0;
-	long /*int*/ ptr = OS.gtk_entry_get_text (handle);
+	long /*int*/ ptr = GTK.gtk_entry_get_text (handle);
 	if (end_pos == -1) end_pos = OS.g_utf8_strlen (ptr, -1);
 	int start = (int)/*64*/OS.g_utf8_offset_to_utf16_offset (ptr, start_pos);
 	int end = (int)/*64*/OS.g_utf8_offset_to_utf16_offset (ptr, end_pos);
@@ -705,13 +716,13 @@ long /*int*/ gtk_delete_text (long /*int*/ widget, long /*int*/ start_pos, long
 		if (newText.length () > 0) {
 			int [] pos = new int [1];
 			pos [0] = (int)/*64*/end_pos;
-			byte [] buffer = Converter.wcsToMbcs (null, newText, false);
+			byte [] buffer = Converter.wcsToMbcs (newText, false);
 			OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, CHANGED);
 			OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, INSERT_TEXT);
-			OS.gtk_editable_insert_text (handle, buffer, buffer.length, pos);
+			GTK.gtk_editable_insert_text (handle, buffer, buffer.length, pos);
 			OS.g_signal_handlers_unblock_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, INSERT_TEXT);
 			OS.g_signal_handlers_unblock_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, CHANGED);
-			OS.gtk_editable_set_position (handle, pos [0]);
+			GTK.gtk_editable_set_position (handle, pos [0]);
 		}
 	}
 	return 0;
@@ -734,28 +745,28 @@ long /*int*/ gtk_insert_text (long /*int*/ widget, long /*int*/ new_text, long /
 //	if (!hooks (SWT.Verify) && !filters (SWT.Verify)) return 0;
 	if (new_text == 0 || new_text_length == 0) return 0;
 	byte [] buffer = new byte [(int)/*64*/new_text_length];
-	OS.memmove (buffer, new_text, buffer.length);
-	String oldText = new String (Converter.mbcsToWcs (null, buffer));
+	C.memmove (buffer, new_text, buffer.length);
+	String oldText = new String (Converter.mbcsToWcs (buffer));
 	int [] pos = new int [1];
-	OS.memmove (pos, position, 4);
-	long /*int*/ ptr = OS.gtk_entry_get_text (handle);
+	C.memmove (pos, position, 4);
+	long /*int*/ ptr = GTK.gtk_entry_get_text (handle);
 	if (pos [0] == -1) pos [0] = (int)/*64*/OS.g_utf8_strlen (ptr, -1);
 	int start = (int)/*64*/OS.g_utf16_pointer_to_offset (ptr, pos [0]);
 	String newText = verifyText (oldText, start, start);
 	if (newText != oldText) {
 		int [] newStart = new int [1], newEnd = new int [1];
-		OS.gtk_editable_get_selection_bounds (handle, newStart, newEnd);
+		GTK.gtk_editable_get_selection_bounds (handle, newStart, newEnd);
 		if (newText != null) {
 			if (newStart [0] != newEnd [0]) {
 				OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, DELETE_TEXT);
 				OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, CHANGED);
-				OS.gtk_editable_delete_selection (handle);
+				GTK.gtk_editable_delete_selection (handle);
 				OS.g_signal_handlers_unblock_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, DELETE_TEXT);
 				OS.g_signal_handlers_unblock_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, CHANGED);
 			}
-			byte [] buffer3 = Converter.wcsToMbcs (null, newText, false);
+			byte [] buffer3 = Converter.wcsToMbcs (newText, false);
 			OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, INSERT_TEXT);
-			OS.gtk_editable_insert_text (handle, buffer3, buffer3.length, pos);
+			GTK.gtk_editable_insert_text (handle, buffer3, buffer3.length, pos);
 			OS.g_signal_handlers_unblock_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, INSERT_TEXT);
 			newStart [0] = newEnd [0] = pos [0];
 		}
@@ -764,7 +775,7 @@ long /*int*/ gtk_insert_text (long /*int*/ widget, long /*int*/ new_text, long /
 			fixStart = newStart [0];
 			fixEnd = newEnd [0];
 		}
-		OS.memmove (position, pos, 4);
+		C.memmove (position, pos, 4);
 		OS.g_signal_stop_emission_by_name (handle, OS.insert_text);
 	}
 	return 0;
@@ -782,8 +793,8 @@ long /*int*/ gtk_key_press_event (long /*int*/ widget, long /*int*/ event) {
 @Override
 long /*int*/ gtk_populate_popup (long /*int*/ widget, long /*int*/ menu) {
 	if ((style & SWT.RIGHT_TO_LEFT) != 0) {
-		OS.gtk_widget_set_direction (menu, OS.GTK_TEXT_DIR_RTL);
-		OS.gtk_container_forall (menu, display.setDirectionProc, OS.GTK_TEXT_DIR_RTL);
+		GTK.gtk_widget_set_direction (menu, GTK.GTK_TEXT_DIR_RTL);
+		GTK.gtk_container_forall (menu, display.setDirectionProc, GTK.GTK_TEXT_DIR_RTL);
 	}
 	return 0;
 }
@@ -806,7 +817,7 @@ void hookEvents () {
 	long /*int*/ imContext = imContext ();
 	if (imContext != 0) {
 		OS.g_signal_connect_closure (imContext, OS.commit, display.getClosure (COMMIT), false);
-		int id = OS.g_signal_lookup (OS.commit, OS.gtk_im_context_get_type ());
+		int id = OS.g_signal_lookup (OS.commit, GTK.gtk_im_context_get_type ());
 		int mask =  OS.G_SIGNAL_MATCH_DATA | OS.G_SIGNAL_MATCH_ID;
 		OS.g_signal_handlers_block_matched (imContext, mask, id, 0, 0, 0, handle);
 	}
@@ -814,13 +825,13 @@ void hookEvents () {
 
 long /*int*/ imContext () {
 	if (imContext != 0) return imContext;
-	return OS.GTK_ENTRY_IM_CONTEXT (handle);
+	return GTK.GTK_ENTRY_IM_CONTEXT (handle);
 }
 
 @Override
 long /*int*/ paintWindow () {
 	long /*int*/ window = super.paintWindow ();
-	long /*int*/ children = OS.gdk_window_get_children (window);
+	long /*int*/ children = GDK.gdk_window_get_children (window);
 	if (children != 0) window = OS.g_list_data (children);
 	OS.g_list_free (children);
 	return window;
@@ -840,7 +851,7 @@ long /*int*/ paintWindow () {
  */
 public void paste () {
 	checkWidget ();
-	OS.gtk_editable_paste_clipboard (handle);
+	GTK.gtk_editable_paste_clipboard (handle);
 }
 
 @Override
@@ -930,51 +941,40 @@ void removeVerifyListener (VerifyListener listener) {
 }
 
 @Override
-GdkColor getContextBackground () {
-	if (OS.GTK_VERSION >= OS.VERSION(3, 16, 0)) {
-		if (background != null) {
-			return display.toGdkColor (background);
-		} else {
-			return display.COLOR_WIDGET_BACKGROUND;
-		}
-	} else {
-		return super.getContextBackground ();
+GdkRGBA getContextBackgroundGdkRGBA () {
+	assert GTK.GTK3 : "GTK3 code was run by GTK2";
+	if (background != null && (state & BACKGROUND) != 0) {
+		return background;
 	}
+	return defaultBackground();
 }
 
 @Override
-void setBackgroundColor (long /*int*/ context, long /*int*/ handle, GdkRGBA rgba) {
+void setBackgroundGdkRGBA (long /*int*/ context, long /*int*/ handle, GdkRGBA rgba) {
+	assert GTK.GTK3 : "GTK3 code was run by GTK2";
 	background = rgba;
-	setBackgroundColorGradient (context, handle, rgba);
+	setBackgroundGradientGdkRGBA (context, handle, rgba);
 }
 
 @Override
-void setBackgroundColor (GdkColor color) {
-	super.setBackgroundColor (color);
-	if (!OS.GTK3) {
-		OS.gtk_widget_modify_base (handle, 0, color);
-	}
+void setBackgroundGdkColor (GdkColor color) {
+	assert !GTK.GTK3 : "GTK2 code was run by GTK3";
+	super.setBackgroundGdkColor (color);
+	GTK.gtk_widget_modify_base (handle, 0, color);
 }
 
 @Override
 void setCursor (long /*int*/ cursor) {
 	long /*int*/ defaultCursor = 0;
-	if (cursor == 0) defaultCursor = OS.gdk_cursor_new_for_display (OS.gdk_display_get_default(), OS.GDK_XTERM);
+	if (cursor == 0) defaultCursor = GDK.gdk_cursor_new_for_display (GDK.gdk_display_get_default(), GDK.GDK_XTERM);
 	super.setCursor (cursor != 0 ? cursor : defaultCursor);
 	if (cursor == 0) gdk_cursor_unref (defaultCursor);
 }
 
 @Override
-void setForegroundColor (GdkColor color) {
-	if (OS.GTK_VERSION >= OS.VERSION (3, 16, 0)) {
-		GdkRGBA rgba = null;
-		if (color != null) {
-			rgba = display.toGdkRGBA (color);
-		}
-		setForegroundColor (handle, rgba);
-	} else {
-		setForegroundColor (handle, color, false);
-	}
+void setForegroundGdkColor (GdkColor color) {
+	assert !GTK.GTK3 : "GTK2 code was run by GTK3";
+	setForegroundColor (handle, color, false);
 }
 
 /**
@@ -992,13 +992,13 @@ void setForegroundColor (GdkColor color) {
 public void setIncrement (int value) {
 	checkWidget ();
 	if (value < 1) return;
-	long /*int*/ hAdjustment = OS.gtk_spin_button_get_adjustment (handle);
-	double page_increment = OS.gtk_adjustment_get_page_increment (hAdjustment);
+	long /*int*/ hAdjustment = GTK.gtk_spin_button_get_adjustment (handle);
+	double page_increment = GTK.gtk_adjustment_get_page_increment (hAdjustment);
 	double newValue = value;
-	int digits = OS.gtk_spin_button_get_digits (handle);
+	int digits = GTK.gtk_spin_button_get_digits (handle);
 	for (int i = 0; i < digits; i++) newValue /= 10;
 	OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
-	OS.gtk_spin_button_set_increments (handle, newValue, page_increment);
+	GTK.gtk_spin_button_set_increments (handle, newValue, page_increment);
 	OS.g_signal_handlers_unblock_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
 }
 
@@ -1017,14 +1017,14 @@ public void setIncrement (int value) {
  */
 public void setMaximum (int value) {
 	checkWidget ();
-	long /*int*/ hAdjustment = OS.gtk_spin_button_get_adjustment (handle);
-	double lower = OS.gtk_adjustment_get_lower (hAdjustment);
+	long /*int*/ hAdjustment = GTK.gtk_spin_button_get_adjustment (handle);
+	double lower = GTK.gtk_adjustment_get_lower (hAdjustment);
 	double newValue = value;
-	int digits = OS.gtk_spin_button_get_digits (handle);
+	int digits = GTK.gtk_spin_button_get_digits (handle);
 	for (int i = 0; i < digits; i++) newValue /= 10;
 	if (newValue < lower) return;
 	OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
-	OS.gtk_spin_button_set_range (handle, lower, newValue);
+	GTK.gtk_spin_button_set_range (handle, lower, newValue);
 	OS.g_signal_handlers_unblock_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
 }
 
@@ -1043,14 +1043,14 @@ public void setMaximum (int value) {
  */
 public void setMinimum (int value) {
 	checkWidget ();
-	long /*int*/ hAdjustment = OS.gtk_spin_button_get_adjustment (handle);
-	double upper = OS.gtk_adjustment_get_upper (hAdjustment);
+	long /*int*/ hAdjustment = GTK.gtk_spin_button_get_adjustment (handle);
+	double upper = GTK.gtk_adjustment_get_upper (hAdjustment);
 	double newValue = value;
-	int digits = OS.gtk_spin_button_get_digits (handle);
+	int digits = GTK.gtk_spin_button_get_digits (handle);
 	for (int i = 0; i < digits; i++) newValue /= 10;
 	if (newValue > upper) return;
 	OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
-	OS.gtk_spin_button_set_range (handle, newValue, upper);
+	GTK.gtk_spin_button_set_range (handle, newValue, upper);
 	OS.g_signal_handlers_unblock_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
 }
 
@@ -1069,13 +1069,13 @@ public void setMinimum (int value) {
 public void setPageIncrement (int value) {
 	checkWidget ();
 	if (value < 1) return;
-	long /*int*/ hAdjustment = OS.gtk_spin_button_get_adjustment (handle);
-	double step_increment = OS.gtk_adjustment_get_step_increment(hAdjustment);
+	long /*int*/ hAdjustment = GTK.gtk_spin_button_get_adjustment (handle);
+	double step_increment = GTK.gtk_adjustment_get_step_increment(hAdjustment);
 	double newValue = value;
-	int digits = OS.gtk_spin_button_get_digits (handle);
+	int digits = GTK.gtk_spin_button_get_digits (handle);
 	for (int i = 0; i < digits; i++) newValue /= 10;
 	OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
-	OS.gtk_spin_button_set_increments (handle, step_increment, newValue);
+	GTK.gtk_spin_button_set_increments (handle, step_increment, newValue);
 	OS.g_signal_handlers_unblock_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
 }
 
@@ -1095,10 +1095,10 @@ public void setPageIncrement (int value) {
 public void setSelection (int value) {
 	checkWidget ();
 	double newValue = value;
-	int digits = OS.gtk_spin_button_get_digits (handle);
+	int digits = GTK.gtk_spin_button_get_digits (handle);
 	for (int i = 0; i < digits; i++) newValue /= 10;
 	OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
-	OS.gtk_spin_button_set_value (handle, newValue);
+	GTK.gtk_spin_button_set_value (handle, newValue);
 	OS.g_signal_handlers_unblock_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
 }
 
@@ -1127,7 +1127,7 @@ public void setSelection (int value) {
 public void setTextLimit (int limit) {
 	checkWidget ();
 	if (limit == 0) error (SWT.ERROR_CANNOT_BE_ZERO);
-	OS.gtk_entry_set_max_length (handle, limit);
+	GTK.gtk_entry_set_max_length (handle, limit);
 }
 
 /**
@@ -1153,9 +1153,9 @@ public void setTextLimit (int limit) {
 public void setDigits (int value) {
 	checkWidget ();
 	if (value < 0) error (SWT.ERROR_INVALID_ARGUMENT);
-	int digits = OS.gtk_spin_button_get_digits (handle);
+	int digits = GTK.gtk_spin_button_get_digits (handle);
 	if (value == digits) return;
-	long /*int*/ hAdjustment = OS.gtk_spin_button_get_adjustment (handle);
+	long /*int*/ hAdjustment = GTK.gtk_spin_button_get_adjustment (handle);
 	GtkAdjustment adjustment = new GtkAdjustment ();
 	gtk_adjustment_get (hAdjustment, adjustment);
 	int diff = Math.abs (value - digits);
@@ -1176,10 +1176,10 @@ public void setDigits (int value) {
 		adjustment.page_increment /= factor;
 		climbRate /= factor;
 	}
-	OS.gtk_adjustment_configure(hAdjustment, adjustment.value, adjustment.lower, adjustment.upper,
+	GTK.gtk_adjustment_configure(hAdjustment, adjustment.value, adjustment.lower, adjustment.upper,
 		adjustment.step_increment, adjustment.page_increment, adjustment.page_size);
 	OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
-	OS.gtk_spin_button_configure (handle, hAdjustment, climbRate, value);
+	GTK.gtk_spin_button_configure (handle, hAdjustment, climbRate, value);
 	OS.g_signal_handlers_unblock_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
 }
 
@@ -1215,19 +1215,20 @@ public void setValues (int selection, int minimum, int maximum, int digits, int
 	selection = Math.min (Math.max (minimum, selection), maximum);
 	double factor = 1;
 	for (int i = 0; i < digits; i++) factor *= 10;
-	OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
-	OS.gtk_spin_button_set_range (handle, minimum / factor, maximum / factor);
-	OS.gtk_spin_button_set_increments (handle, increment / factor, pageIncrement / factor);
-	OS.gtk_spin_button_set_value (handle, selection / factor);
 	/*
 	* The value of climb-rate indicates the acceleration rate
 	* to spin the value when the button is pressed and hold
 	* on the arrow button. This value should be varied
 	* depending upon the value of digits.
 	*/
+	OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
 	climbRate = 1.0 / factor;
-	long /*int*/ adjustment = OS.gtk_spin_button_get_adjustment(handle);
-	OS.gtk_spin_button_configure (handle, adjustment, climbRate, digits);
+	long /*int*/ adjustment = GTK.gtk_spin_button_get_adjustment(handle);
+	GTK.gtk_spin_button_configure (handle, adjustment, climbRate, digits);
+
+	GTK.gtk_spin_button_set_range (handle, minimum / factor, maximum / factor);
+	GTK.gtk_spin_button_set_increments (handle, increment / factor, pageIncrement / factor);
+	GTK.gtk_spin_button_set_value (handle, selection / factor);
 	OS.g_signal_handlers_unblock_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
 }
 
@@ -1240,14 +1241,14 @@ boolean checkSubwindow () {
 boolean translateTraversal (GdkEventKey keyEvent) {
 	int key = keyEvent.keyval;
 	switch (key) {
-		case OS.GDK_KP_Enter:
-		case OS.GDK_Return: {
+		case GDK.GDK_KP_Enter:
+		case GDK.GDK_Return: {
 			long /*int*/ imContext =  imContext ();
 			if (imContext != 0) {
 				long /*int*/ [] preeditString = new long /*int*/ [1];
-				OS.gtk_im_context_get_preedit_string (imContext, preeditString, null, null);
+				GTK.gtk_im_context_get_preedit_string (imContext, preeditString, null, null);
 				if (preeditString [0] != 0) {
-					int length = OS.strlen (preeditString [0]);
+					int length = C.strlen (preeditString [0]);
 					OS.g_free (preeditString [0]);
 					if (length != 0) return false;
 				}
@@ -1263,19 +1264,19 @@ String verifyText (String string, int start, int end) {
 	event.text = string;
 	event.start = start;
 	event.end = end;
-	long /*int*/ eventPtr = OS.gtk_get_current_event ();
+	long /*int*/ eventPtr = GTK.gtk_get_current_event ();
 	if (eventPtr != 0) {
 		GdkEventKey gdkEvent = new GdkEventKey ();
 		OS.memmove (gdkEvent, eventPtr, GdkEventKey.sizeof);
 		switch (gdkEvent.type) {
-			case OS.GDK_KEY_PRESS:
+			case GDK.GDK_KEY_PRESS:
 				setKeyState (event, gdkEvent);
 				break;
 		}
-		OS.gdk_event_free (eventPtr);
+		GDK.gdk_event_free (eventPtr);
 	}
 	int index = 0;
-	if (OS.gtk_spin_button_get_digits (handle) > 0) {
+	if (GTK.gtk_spin_button_get_digits (handle) > 0) {
 		String decimalSeparator = getDecimalSeparator ();
 		index = string.indexOf (decimalSeparator);
 		if (index != -1) {
@@ -1284,8 +1285,8 @@ String verifyText (String string, int start, int end) {
 		index = 0;
 	}
 	if (string.length () > 0) {
-		long /*int*/ hAdjustment = OS.gtk_spin_button_get_adjustment (handle);
-		double lower = OS.gtk_adjustment_get_lower (hAdjustment);
+		long /*int*/ hAdjustment = GTK.gtk_spin_button_get_adjustment (handle);
+		double lower = GTK.gtk_adjustment_get_lower (hAdjustment);
 		if (lower < 0 && string.charAt (0) == '-') index++;
 	}
 	while (index < string.length ()) {
