@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2017 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -441,7 +444,7 @@ public void pack () {
 	int width = 0;
 	if (buttonHandle != 0) {
 		GtkRequisition requisition = new GtkRequisition ();
-		if (parent.getHeaderVisible() && GTK.GTK_VERSION >= OS.VERSION (3, 8, 0) && !GTK.gtk_widget_get_visible(buttonHandle)) {
+		if (GTK.GTK3) {
 			GTK.gtk_widget_show(buttonHandle);
 			gtk_widget_get_preferred_size (buttonHandle, requisition);
 			GTK.gtk_widget_hide(buttonHandle);
@@ -451,9 +454,25 @@ public void pack () {
 		width = requisition.width;
 	}
 	if ((parent.style & SWT.VIRTUAL) != 0) {
+		boolean calcWidth = false;
+		Rectangle itemBounds = null;
+		int tableHeight = 0;
+		if (parent.isVisible()) {
+			Rectangle tableBounds = parent.getBounds();
+			tableHeight = tableBounds.height - parent.getHeaderHeight();
+		} else {
+			calcWidth = true;
+		}
 		for (int i=0; i<parent.items.length; i++) {
 			TableItem item = parent.items [i];
-			if (item != null && item.cached) {
+			if (itemBounds == null && item != null) itemBounds = item.getBounds();
+			boolean isVisible = false;
+			if (!calcWidth && itemBounds != null) {
+				int itemTopBound = itemBounds.y + itemBounds.height * i + i;
+				int itemBottomBound = itemTopBound + itemBounds.height;
+				isVisible = (itemTopBound > 0 && itemBottomBound < tableHeight);
+			}
+			if (item != null && item.cached && (isVisible || calcWidth)) {
 				width = Math.max (width, parent.calculateWidth (handle, item.handle));
 			}
 		}
