@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2016 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -49,6 +52,12 @@ public class Tracker extends Widget {
 	Rectangle bounds;
 	int cursorOrientation = SWT.NONE;
 	int oldX, oldY;
+	long /*int*/ provider; // Gtk3.14
+
+	// Re-use/cache some items for performance reasons as draw-events must be efficient to prevent jitter.
+	Rectangle cachedCombinedDisplayResolution = Display.getDefault().getBounds(); // Cached for performance reasons.
+	Rectangle cachedUnion = new Rectangle(0, 0, 0, 0);
+	Boolean cachedBackgroundIsOpaque; // Note purposely lazy init.
 
 	final static int STEPSIZE_SMALL = 1;
 	final static int STEPSIZE_LARGE = 9;
@@ -306,57 +315,110 @@ Rectangle [] computeProportions (Rectangle [] rects) {
 	return result;
 }
 
+/**
+ * Developer note:
+ * - Rectangles can have absolute coords [Tracker(Display)] or relative to parent [Tracker(Composite)]
+ * - This method is called a lot, optimize your code.
+ * - Note, region != rectangle. A region can have a non-squared form, e.g an 'L' shape.
+ *
+ * @param rects
+ */
 void drawRectangles (Rectangle [] rects) {
-	long /*int*/ window = OS.gdk_get_default_root_window();
+	long /*int*/ window = GDK.gdk_get_default_root_window();
 	if (parent != null) {
 		window = gtk_widget_get_window (parent.paintHandle());
 	}
 	if (window == 0) return;
-	if (OS.GTK3) {
+	if (GTK.GTK3) {
 		if (overlay == 0) return;
-		OS.gtk_widget_shape_combine_region (overlay, 0);
-		long /*int*/ region = OS.gdk_region_new ();
-		GdkRectangle rect = new GdkRectangle();
-		for (int i = 0; i < rects.length; i++) {
-			Rectangle r = parent != null ? display.mapInPixels(parent, null, rects[i]) : rects[i];
-			rect.x = r.x;
-			rect.y = r.y;
-			rect.width = r.width + 1;
-			rect.height = 1;
-			OS.gdk_region_union_with_rect(region, rect);
-			rect.width = 1;
-			rect.height = r.height + 1;
-			OS.gdk_region_union_with_rect(region, rect);
-			rect.x = r.x + r.width;
-			OS.gdk_region_union_with_rect(region, rect);
-			rect.x = r.x;
-			rect.y = r.y + r.height;
-			rect.width = r.width + 1;
-			rect.height = 1;
-			OS.gdk_region_union_with_rect(region, rect);
+		GTK.gtk_widget_shape_combine_region (overlay, 0);
+
+		// Bug 498217.
+		// As of Gtk 3.9.1, Commit a60ccd3672467efb454b121993febc36f33cbc79, off-screen GDK windows are not processed.
+		// Because of this gtk doesn't send move events to SWT. Platform.UI uses an off-screen tracker to draw
+		// custom rectangles for it's part-drag-preview. Drawing/updates for these broke because tracker is off screen.
+		// Solution: If a tracker is to move off-screen, then instead draw it 1x1 and transparent.
+		boolean isOnScreen = true;
+		{ // Figure out if the combined rectangles are on or off screen.
+
+			// Produce a single rectangle big enough to contain all rects.
+			// Note, this is different from loop below that creates a Region. (See region != rectangle note in javadoc).
+			cachedUnion.x = rects[0].x;
+			cachedUnion.y = rects[0].y;
+			cachedUnion.width = rects[0].width;
+			cachedUnion.height = rects[0].height;
+			if (rects.length > 1) {
+				for (int i = 1; i < rects.length; i++) {
+					cachedUnion.add(rects[i]);
+				}
+			}
+
+			// Ensure we have absolute screen coordinates. (btw, there are no absolute coordinates on Wayland, so Tracker(Display) is probably broken).
+			if (parent != null) {  // if Tracker(Display) has absolute coords.  Tracker(Composite) has relative. For relative, we need to find absolute.
+				cachedUnion = display.mapInPixels(parent, null, cachedUnion) ;
+			}
+
+			if (!cachedCombinedDisplayResolution.intersects(cachedUnion)) {
+				isOnScreen = false;
+			}
 		}
-		OS.gtk_widget_shape_combine_region (overlay, region);
-		OS.gdk_region_destroy (region);
-		long /*int*/ overlayWindow = OS.gtk_widget_get_window (overlay);
-		OS.gdk_window_hide (overlayWindow);
-		OS.gdk_window_show (overlayWindow);
-		return;
+
+		long /*int*/ region = GDK.gdk_region_new ();
+		GdkRectangle rect = new GdkRectangle();
+		if (isOnScreen) {
+			// Combine Rects into a region. (region is not necessarily a rectangle, E.g it can be 'L' shaped etc..).
+			for (int i = 0; i < rects.length; i++) {
+				// Turn filled rectangles into just the outer lines by drawing one line at a time.
+				Rectangle r = parent != null ? display.mapInPixels(parent, null, rects[i]) : rects[i];
+				rect.x = r.x;
+				rect.y = r.y;
+				rect.width = r.width + 1;
+				rect.height = 1;
+				GDK.gdk_region_union_with_rect(region, rect); // Top line
+				rect.width = 1;
+				rect.height = r.height + 1;
+				GDK.gdk_region_union_with_rect(region, rect); // Left line.
+				rect.x = r.x + r.width;
+				GDK.gdk_region_union_with_rect(region, rect); // Right line.
+				rect.x = r.x;
+				rect.y = r.y + r.height;
+				rect.width = r.width + 1;
+				rect.height = 1;
+				GDK.gdk_region_union_with_rect(region, rect); // Bottom line
+			}
+			setTrackerBackground(true);
+		} else { // Offscreen
+			// part of Bug 498217 fix. Tracker must be at least 1x1 for move events to work.
+			rect.x = 0;
+			rect.y = 0;
+			rect.height = 1;
+			rect.width = 1;
+			GDK.gdk_region_union_with_rect(region, rect);
+			setTrackerBackground(false);
+		}
+
+		GTK.gtk_widget_shape_combine_region (overlay, region);
+		GDK.gdk_region_destroy (region);
+		long /*int*/ overlayWindow = GTK.gtk_widget_get_window (overlay);
+		GDK.gdk_window_hide (overlayWindow);
+		GDK.gdk_window_show (overlayWindow);
+	} else { // GTK2
+		long /*int*/ gc = GDK.gdk_gc_new (window);
+		if (gc == 0) return;
+		long /*int*/ colormap = GDK.gdk_colormap_get_system ();
+		GdkColor color = new GdkColor ();
+		GDK.gdk_color_white (colormap, color);
+		GDK.gdk_gc_set_foreground (gc, color);
+		GDK.gdk_gc_set_subwindow (gc, GDK.GDK_INCLUDE_INFERIORS);
+		GDK.gdk_gc_set_function (gc, GDK.GDK_XOR);
+		for (int i=0; i<rects.length; i++) {
+			Rectangle rect = rects [i];
+			int x = rect.x;
+			if (parent != null && (parent.style & SWT.MIRRORED) != 0) x = parent.getClientWidth () - rect.width - x;
+			GDK.gdk_draw_rectangle (window, gc, 0, x, rect.y, rect.width, rect.height);
+		}
+		OS.g_object_unref (gc);
 	}
-	long /*int*/ gc = OS.gdk_gc_new (window);
-	if (gc == 0) return;
-	long /*int*/ colormap = OS.gdk_colormap_get_system ();
-	GdkColor color = new GdkColor ();
-	OS.gdk_color_white (colormap, color);
-	OS.gdk_gc_set_foreground (gc, color);
-	OS.gdk_gc_set_subwindow (gc, OS.GDK_INCLUDE_INFERIORS);
-	OS.gdk_gc_set_function (gc, OS.GDK_XOR);
-	for (int i=0; i<rects.length; i++) {
-		Rectangle rect = rects [i];
-		int x = rect.x;
-		if (parent != null && (parent.style & SWT.MIRRORED) != 0) x = parent.getClientWidth () - rect.width - x;
-		OS.gdk_draw_rectangle (window, gc, 0, x, rect.y, rect.width, rect.height);
-	}
-	OS.g_object_unref (gc);
 }
 
 /**
@@ -408,13 +470,13 @@ public boolean getStippled () {
 
 boolean grab () {
 	long /*int*/ cursor = this.cursor != null ? this.cursor.handle : 0;
-	int result = gdk_pointer_grab (window, OS.GDK_OWNERSHIP_NONE, false, OS.GDK_POINTER_MOTION_MASK | OS.GDK_BUTTON_RELEASE_MASK, 0, cursor, OS.GDK_CURRENT_TIME);
-	return result == OS.GDK_GRAB_SUCCESS;
+	int result = gdk_pointer_grab (window, GDK.GDK_OWNERSHIP_NONE, false, GDK.GDK_POINTER_MOTION_MASK | GDK.GDK_BUTTON_RELEASE_MASK, 0, cursor, GDK.GDK_CURRENT_TIME);
+	return result == GDK.GDK_GRAB_SUCCESS;
 }
 
 @Override
 long /*int*/ gtk_button_release_event (long /*int*/ widget, long /*int*/ event) {
-	return gtk_mouse (OS.GDK_BUTTON_RELEASE, widget, event);
+	return gtk_mouse (GDK.GDK_BUTTON_RELEASE, widget, event);
 }
 
 @Override
@@ -423,25 +485,25 @@ long /*int*/ gtk_key_press_event (long /*int*/ widget, long /*int*/ eventPtr) {
 	if (result != 0) return result;
 	GdkEventKey keyEvent = new GdkEventKey ();
 	OS.memmove (keyEvent, eventPtr, GdkEventKey.sizeof);
-	int stepSize = ((keyEvent.state & OS.GDK_CONTROL_MASK) != 0) ? STEPSIZE_SMALL : STEPSIZE_LARGE;
+	int stepSize = ((keyEvent.state & GDK.GDK_CONTROL_MASK) != 0) ? STEPSIZE_SMALL : STEPSIZE_LARGE;
 	int xChange = 0, yChange = 0;
 	switch (keyEvent.keyval) {
-		case OS.GDK_Escape:
+		case GDK.GDK_Escape:
 			cancelled = true;
 			// fallthrough
-		case OS.GDK_Return:
+		case GDK.GDK_Return:
 			tracking = false;
 			break;
-		case OS.GDK_Left:
+		case GDK.GDK_Left:
 			xChange = -stepSize;
 			break;
-		case OS.GDK_Right:
+		case GDK.GDK_Right:
 			xChange = stepSize;
 			break;
-		case OS.GDK_Up:
+		case GDK.GDK_Up:
 			yChange = -stepSize;
 			break;
-		case OS.GDK_Down:
+		case GDK.GDK_Down:
 			yChange = stepSize;
 			break;
 	}
@@ -494,7 +556,7 @@ long /*int*/ gtk_key_press_event (long /*int*/ widget, long /*int*/ eventPtr) {
 				draw = true;
 			}
 			if (draw) {
-				if (!OS.GTK3) {
+				if (!GTK.GTK3) {
 					drawRectangles (rectsToErase);
 					update ();
 				}
@@ -563,7 +625,7 @@ long /*int*/ gtk_motion_notify_event (long /*int*/ widget, long /*int*/ eventPtr
 		grabbed = grab ();
 		lastCursor = cursor;
 	}
-	return gtk_mouse (OS.GDK_MOTION_NOTIFY, widget, eventPtr);
+	return gtk_mouse (GDK.GDK_MOTION_NOTIFY, widget, eventPtr);
 }
 
 long /*int*/ gtk_mouse (int eventType, long /*int*/ widget, long /*int*/ eventPtr) {
@@ -621,7 +683,7 @@ long /*int*/ gtk_mouse (int eventType, long /*int*/ widget, long /*int*/ eventPt
 				draw = true;
 			}
 			if (draw) {
-				if (!OS.GTK3) {
+				if (!GTK.GTK3) {
 					drawRectangles (rectsToErase);
 					update ();
 				}
@@ -668,7 +730,7 @@ long /*int*/ gtk_mouse (int eventType, long /*int*/ widget, long /*int*/ eventPt
 				draw = true;
 			}
 			if (draw) {
-				if (!OS.GTK3) {
+				if (!GTK.GTK3) {
 					drawRectangles (rectsToErase);
 					update ();
 				}
@@ -678,7 +740,7 @@ long /*int*/ gtk_mouse (int eventType, long /*int*/ widget, long /*int*/ eventPt
 		oldX = newX [0];
 		oldY = newY [0];
 	}
-	tracking = eventType != OS.GDK_BUTTON_RELEASE;
+	tracking = eventType != GDK.GDK_BUTTON_RELEASE;
 	return 0;
 }
 
@@ -711,14 +773,14 @@ void moveRectangles (int xChange, int yChange) {
  */
 public boolean open () {
 	checkWidget();
-	window = OS.gdk_get_default_root_window();
+	window = GDK.gdk_get_default_root_window();
 	if (parent != null) {
 		window = gtk_widget_get_window (parent.paintHandle());
 	}
 	if (window == 0) return false;
 	cancelled = false;
 	tracking = true;
-	if (!OS.GTK3) {
+	if (!GTK.GTK3) {
 		update ();
 		drawRectangles (rectangles);
 	}
@@ -738,7 +800,7 @@ public boolean open () {
 		cursorOrientation |= hStyle;
 	}
 
-	int mask = OS.GDK_BUTTON1_MASK | OS.GDK_BUTTON2_MASK | OS.GDK_BUTTON3_MASK;
+	int mask = GDK.GDK_BUTTON1_MASK | GDK.GDK_BUTTON2_MASK | GDK.GDK_BUTTON3_MASK;
 	boolean mouseDown = (state [0] & mask) != 0;
 	if (!mouseDown) {
 		Point cursorPos = null;
@@ -758,35 +820,19 @@ public boolean open () {
 	grabbed = grab ();
 	lastCursor = this.cursor != null ? this.cursor.handle : 0;
 
-	if (OS.GTK3) {
-		overlay = OS.gtk_window_new (OS.GTK_WINDOW_POPUP);
-		OS.gtk_window_set_skip_taskbar_hint (overlay, true);
-		OS.gtk_window_set_title (overlay, new byte [1]);
-		OS.gtk_widget_realize (overlay);
-		long /*int*/ overlayWindow = OS.gtk_widget_get_window (overlay);
-		OS.gdk_window_set_override_redirect (overlayWindow, true);
-		if (OS.GTK_VERSION < OS.VERSION (3, 16, 0)) {
-			OS.gtk_widget_override_background_color (overlay, OS.GTK_STATE_FLAG_NORMAL, new GdkRGBA());
-		} else {
-			String name = OS.GTK_VERSION >= OS.VERSION(3, 20, 0) ? "window" : "GtkWindow";
-			String css = name + " {background-color: rgb(0,0,0);}";
-			long /*int*/ provider = 0;
-			long /*int*/ context = OS.gtk_widget_get_style_context (overlay);
-			if (provider == 0) {
-				provider = OS.gtk_css_provider_new ();
-				OS.gtk_style_context_add_provider (context, provider, OS.GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
-				OS.g_object_unref (provider);
-			}
-			OS.gtk_css_provider_load_from_data (provider, Converter.wcsToMbcs (null, css, true), -1, null);
-		}
-		long /*int*/ region = OS.gdk_region_new ();
-		OS.gtk_widget_shape_combine_region (overlay, region);
-		OS.gtk_widget_input_shape_combine_region (overlay, region);
-		OS.gdk_region_destroy (region);
+	if (GTK.GTK3) {
+		cachedCombinedDisplayResolution = Display.getDefault().getBounds(); // In case resolution was changed during run time.
+		overlay = GTK.gtk_window_new (GTK.GTK_WINDOW_POPUP);
+		GTK.gtk_window_set_skip_taskbar_hint (overlay, true);
+		GTK.gtk_window_set_title (overlay, new byte [1]);
+		GTK.gtk_widget_realize (overlay);
+		long /*int*/ overlayWindow = GTK.gtk_widget_get_window (overlay);
+		GDK.gdk_window_set_override_redirect (overlayWindow, true);
+		setTrackerBackground(true);
 		Rectangle bounds = display.getBoundsInPixels();
-		OS.gtk_window_move (overlay, bounds.x, bounds.y);
-		OS.gtk_window_resize (overlay, bounds.width, bounds.height);
-		OS.gtk_widget_show (overlay);
+		GTK.gtk_window_move (overlay, bounds.x, bounds.y);
+		GTK.gtk_window_resize (overlay, bounds.width, bounds.height);
+		GTK.gtk_widget_show (overlay);
 	}
 
 	/* Tracker behaves like a Dialog with its own OS event loop. */
@@ -805,7 +851,7 @@ public boolean open () {
 			* code outside of SWT (i.e AWT, etc). It ensures that the current
 			* thread leaves the GTK lock before calling the function below.
 			*/
-			OS.gdk_threads_leave();
+			GDK.gdk_threads_leave();
 			OS.g_main_context_iteration (0, true);
 			display.sendPostExternalEventDispatchEvent ();
 			display.runAsyncMessages (false);
@@ -814,41 +860,78 @@ public boolean open () {
 		display.tracker = oldTracker;
 	}
 	if (!isDisposed ()) {
-		if (!OS.GTK3) {
+		if (!GTK.GTK3) {
 			update ();
 			drawRectangles (rectangles);
 		}
 	}
 	ungrab ();
 	if (overlay != 0) {
-		OS.gtk_widget_destroy (overlay);
+		GTK.gtk_widget_destroy (overlay);
 		overlay = 0;
 	}
 	window = 0;
 	return !cancelled;
 }
 
+private void setTrackerBackground(boolean opaque) {
+	if (cachedBackgroundIsOpaque == null || cachedBackgroundIsOpaque.booleanValue() != opaque) {
+		cachedBackgroundIsOpaque = Boolean.valueOf(opaque);
+	} else if (opaque == cachedBackgroundIsOpaque.booleanValue()) {
+		return;
+	}
+	if (GTK.GTK_VERSION < OS.VERSION (3, 14, 0)) {
+		GTK.gtk_widget_override_background_color (overlay, GTK.GTK_STATE_FLAG_NORMAL, new GdkRGBA());
+	} else {
+		String name = GTK.GTK_VERSION >= OS.VERSION(3, 20, 0) ? "window" : "GtkWindow";
+		String css;
+		if (opaque) {
+			GTK.gtk_widget_set_opacity (overlay, 1.0);
+			css = name + " {background-color: rgb(0,0,0);}";
+		} else {
+			GTK.gtk_widget_set_opacity (overlay, 0.0);
+			css = name +  " {  "
+					+ "border-top-color: transparent;"
+					+ "border-left-color: transparent;"
+					+ "border-right-color: transparent;"
+					+ "border-bottom-color: transparent;}";
+		}
+		long /*int*/ context = GTK.gtk_widget_get_style_context (overlay);
+		if (provider == 0) {
+			provider = GTK.gtk_css_provider_new ();
+			GTK.gtk_style_context_add_provider (context, provider, GTK.GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+			OS.g_object_unref (provider);
+		}
+		GTK.gtk_css_provider_load_from_data (provider, Converter.wcsToMbcs (css, true), -1, null);
+		GTK.gtk_style_context_invalidate (context);
+	}
+	long /*int*/ region = GDK.gdk_region_new ();
+	GTK.gtk_widget_shape_combine_region (overlay, region);
+	GTK.gtk_widget_input_shape_combine_region (overlay, region);
+	GDK.gdk_region_destroy (region);
+}
+
 boolean processEvent (long /*int*/ eventPtr) {
 	GdkEvent gdkEvent = new GdkEvent();
 	OS.memmove (gdkEvent, eventPtr, GdkEvent.sizeof);
-	long /*int*/ widget = OS.gtk_get_event_widget (eventPtr);
+	long /*int*/ widget = GTK.gtk_get_event_widget (eventPtr);
 	switch (gdkEvent.type) {
-		case OS.GDK_MOTION_NOTIFY: gtk_motion_notify_event (widget, eventPtr); break;
-		case OS.GDK_BUTTON_RELEASE: gtk_button_release_event (widget, eventPtr); break;
-		case OS.GDK_KEY_PRESS: gtk_key_press_event (widget, eventPtr); break;
-		case OS.GDK_KEY_RELEASE: gtk_key_release_event (widget, eventPtr); break;
-		case OS.GDK_BUTTON_PRESS:
-		case OS.GDK_2BUTTON_PRESS:
-		case OS.GDK_3BUTTON_PRESS:
-		case OS.GDK_ENTER_NOTIFY:
-		case OS.GDK_LEAVE_NOTIFY:
+		case GDK.GDK_MOTION_NOTIFY: gtk_motion_notify_event (widget, eventPtr); break;
+		case GDK.GDK_BUTTON_RELEASE: gtk_button_release_event (widget, eventPtr); break;
+		case GDK.GDK_KEY_PRESS: gtk_key_press_event (widget, eventPtr); break;
+		case GDK.GDK_KEY_RELEASE: gtk_key_release_event (widget, eventPtr); break;
+		case GDK.GDK_BUTTON_PRESS:
+		case GDK.GDK_2BUTTON_PRESS:
+		case GDK.GDK_3BUTTON_PRESS:
+		case GDK.GDK_ENTER_NOTIFY:
+		case GDK.GDK_LEAVE_NOTIFY:
 			/* Do not dispatch these */
 			break;
-		case OS.GDK_EXPOSE:
+		case GDK.GDK_EXPOSE:
 			update ();
-			if (!OS.GTK3) drawRectangles (rectangles);
-			OS.gtk_main_do_event (eventPtr);
-			if (!OS.GTK3) drawRectangles (rectangles);
+			if (!GTK.GTK3) drawRectangles (rectangles);
+			GTK.gtk_main_do_event (eventPtr);
+			if (!GTK.GTK3) drawRectangles (rectangles);
 			break;
 		default:
 			return true;
@@ -1096,7 +1179,7 @@ public void setStippled (boolean stippled) {
 }
 
 void ungrab () {
-	if (grabbed) gdk_pointer_ungrab (window, OS.GDK_CURRENT_TIME);
+	if (grabbed) gdk_pointer_ungrab (window, GDK.GDK_CURRENT_TIME);
 }
 
 void update () {
