@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2014 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -59,84 +62,6 @@ public final class Cursor extends Resource {
 
 	boolean isIcon;
 
-	/**
-	 * data used to create a HAND cursor.
-	 */
-	static final byte[] HAND_SOURCE = {
-		(byte)0xf9,(byte)0xff,(byte)0xff,(byte)0xff,
-		(byte)0xf0,(byte)0xff,(byte)0xff,(byte)0xff,
-		(byte)0xf0,(byte)0xff,(byte)0xff,(byte)0xff,
-		(byte)0xf0,(byte)0xff,(byte)0xff,(byte)0xff,
-		(byte)0xf0,(byte)0x3f,(byte)0xff,(byte)0xff,
-		(byte)0xf0,(byte)0x07,(byte)0xff,(byte)0xff,
-		(byte)0xf0,(byte)0x03,(byte)0xff,(byte)0xff,
-		(byte)0xf0,(byte)0x00,(byte)0xff,(byte)0xff,
-
-		(byte)0x10,(byte)0x00,(byte)0x7f,(byte)0xff,
-		(byte)0x00,(byte)0x00,(byte)0x7f,(byte)0xff,
-		(byte)0x80,(byte)0x00,(byte)0x7f,(byte)0xff,
-		(byte)0xc0,(byte)0x00,(byte)0x7f,(byte)0xff,
-		(byte)0xe0,(byte)0x00,(byte)0x7f,(byte)0xff,
-		(byte)0xf0,(byte)0x00,(byte)0x7f,(byte)0xff,
-		(byte)0xf8,(byte)0x00,(byte)0xff,(byte)0xff,
-		(byte)0xfc,(byte)0x01,(byte)0xff,(byte)0xff,
-
-		(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,
-		(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,
-		(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,
-		(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,
-		(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,
-		(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,
-		(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,
-		(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,
-
-		(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,
-		(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,
-		(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,
-		(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,
-		(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,
-		(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,
-		(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,
-		(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff
-	};
-	static final byte[] HAND_MASK = {
-		(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
-		(byte)0x06,(byte)0x00,(byte)0x00,(byte)0x00,
-		(byte)0x06,(byte)0x00,(byte)0x00,(byte)0x00,
-		(byte)0x06,(byte)0x00,(byte)0x00,(byte)0x00,
-		(byte)0x06,(byte)0x00,(byte)0x00,(byte)0x00,
-		(byte)0x06,(byte)0xc0,(byte)0x00,(byte)0x00,
-		(byte)0x06,(byte)0xd8,(byte)0x00,(byte)0x00,
-		(byte)0x06,(byte)0xd8,(byte)0x00,(byte)0x00,
-
-		(byte)0x07,(byte)0xdb,(byte)0x00,(byte)0x00,
-		(byte)0x67,(byte)0xfb,(byte)0x00,(byte)0x00,
-		(byte)0x3f,(byte)0xff,(byte)0x00,(byte)0x00,
-		(byte)0x1f,(byte)0xff,(byte)0x00,(byte)0x00,
-		(byte)0x0f,(byte)0xff,(byte)0x00,(byte)0x00,
-		(byte)0x07,(byte)0xff,(byte)0x00,(byte)0x00,
-		(byte)0x03,(byte)0xfe,(byte)0x00,(byte)0x00,
-		(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
-
-		(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
-		(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
-		(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
-		(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
-		(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
-		(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
-		(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
-		(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
-
-		(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
-		(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
-		(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
-		(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
-		(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
-		(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
-		(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
-		(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00
-	};
-
 /**
  * Prevents uninitialized instances from being created outside the package.
  */
@@ -220,20 +145,6 @@ public Cursor(Device device, int style) {
 			SWT.error(SWT.ERROR_INVALID_ARGUMENT);
 	}
 	handle = OS.LoadCursor(0, lpCursorName);
-	/*
-	* IDC_HAND is supported only on Windows 2000 and Windows 98.
-	* Create a hand cursor if running in other Windows platforms.
-	*/
-	if (handle == 0 && style == SWT.CURSOR_HAND) {
-		int width = OS.GetSystemMetrics(OS.SM_CXCURSOR);
-		int height = OS.GetSystemMetrics(OS.SM_CYCURSOR);
-		if (width == 32 && height == 32) {
-			long /*int*/ hInst = OS.GetModuleHandle(null);
-			if (OS.IsWinCE) SWT.error(SWT.ERROR_NOT_IMPLEMENTED);
-			handle = OS.CreateCursor(hInst, 5, 0, 32, 32, HAND_SOURCE, HAND_MASK);
-
-		}
-	}
 	if (handle == 0) SWT.error(SWT.ERROR_NO_HANDLES);
 	init();
 }
@@ -299,7 +210,6 @@ public Cursor(Device device, ImageData source, ImageData mask, int hotspotX, int
 
 	/* Create the cursor */
 	long /*int*/ hInst = OS.GetModuleHandle(null);
-	if (OS.IsWinCE) SWT.error (SWT.ERROR_NOT_IMPLEMENTED);
 	handle = OS.CreateCursor(hInst, hotspotX, hotspotY, source.width, source.height, sourceData, maskData);
 	if (handle == 0) SWT.error(SWT.ERROR_NO_HANDLES);
 	init();
@@ -437,7 +347,7 @@ void destroy () {
 	 	* destroy them all. If this causes problems in the future,
 	 	* put the flag back in.
 		*/
-		if (!OS.IsWinCE) OS.DestroyCursor(handle);
+		OS.DestroyCursor(handle);
 	}
 	handle = 0;
 }
