@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2012 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -118,12 +121,11 @@ Rectangle getBoundsInPixels () {
 	if (image != null) {
 		Rectangle rect = image.getBoundsInPixels ();
 		return new Rectangle (x, y, rect.width, rect.height);
-	} else {
-		if (!OS.IsWinCE && width == 0) {
-			int [] buffer = new int [1];
-			if (OS.SystemParametersInfo (OS.SPI_GETCARETWIDTH, 0, buffer, 0)) {
-				return new Rectangle (x, y, buffer [0], height);
-			}
+	}
+	if (width == 0) {
+		int [] buffer = new int [1];
+		if (OS.SystemParametersInfo (OS.SPI_GETCARETWIDTH, 0, buffer, 0)) {
+			return new Rectangle (x, y, buffer [0], height);
 		}
 	}
 	return new Rectangle (x, y, width, height);
@@ -217,12 +219,11 @@ Point getSizeInPixels () {
 	if (image != null) {
 		Rectangle rect = image.getBoundsInPixels ();
 		return new Point (rect.width, rect.height);
-	} else {
-		if (!OS.IsWinCE && width == 0) {
-			int [] buffer = new int [1];
-			if (OS.SystemParametersInfo (OS.SPI_GETCARETWIDTH, 0, buffer, 0)) {
-				return new Point (buffer [0], height);
-			}
+	}
+	if (width == 0) {
+		int [] buffer = new int [1];
+		if (OS.SystemParametersInfo (OS.SPI_GETCARETWIDTH, 0, buffer, 0)) {
+			return new Point (buffer [0], height);
 		}
 	}
 	return new Point (width, height);
@@ -340,7 +341,7 @@ void resize () {
 	OS.DestroyCaret ();
 	long /*int*/ hBitmap = image != null ? image.handle : 0;
 	int width = this.width;
-	if (!OS.IsWinCE && image == null && width == 0) {
+	if (image == null && width == 0) {
 		int [] buffer = new int [1];
 		if (OS.SystemParametersInfo (OS.SPI_GETCARETWIDTH, 0, buffer, 0)) {
 			width = buffer [0];
@@ -427,7 +428,7 @@ void setFocus () {
 	long /*int*/ hBitmap = 0;
 	if (image != null) hBitmap = image.handle;
 	int width = this.width;
-	if (!OS.IsWinCE && image == null && width == 0) {
+	if (image == null && width == 0) {
 		int [] buffer = new int [1];
 		if (OS.SystemParametersInfo (OS.SPI_GETCARETWIDTH, 0, buffer, 0)) {
 			width = buffer [0];
