@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2012 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -232,12 +235,10 @@ Rectangle getBoundsInPixels () {
 	long /*int*/ hwnd = parent.handle;
 	RECT rect = new RECT ();
 	OS.SendMessage (hwnd, OS.RB_GETRECT, index, rect);
-	if (OS.COMCTL32_MAJOR >= 6) {
-		MARGINS margins = new MARGINS ();
-		OS.SendMessage (hwnd, OS.RB_GETBANDMARGINS, 0, margins);
-		rect.left -= margins.cxLeftWidth;
-		rect.right += margins.cxRightWidth;
-	}
+	MARGINS margins = new MARGINS ();
+	OS.SendMessage (hwnd, OS.RB_GETBANDMARGINS, 0, margins);
+	rect.left -= margins.cxLeftWidth;
+	rect.right += margins.cxRightWidth;
 	if (!parent.isLastItemOfRow (index)) {
 		rect.right += (parent.style & SWT.FLAT) == 0 ? CoolBar.SEPARATOR_WIDTH : 0;
 	}
@@ -367,7 +368,7 @@ public void setControl (Control control) {
 	if (showOld) oldControl.setVisible (true);
 	if (hwndAbove != 0 && hwndAbove != hwndChild) {
 		int flags = OS.SWP_NOSIZE | OS.SWP_NOMOVE | OS.SWP_NOACTIVATE;
-		SetWindowPos (hwndChild, hwndAbove, 0, 0, 0, 0, flags);
+		OS.SetWindowPos (hwndChild, hwndAbove, 0, 0, 0, 0, flags);
 	}
 }
 
@@ -493,12 +494,10 @@ Point getSizeInPixels() {
 	long /*int*/ hwnd = parent.handle;
 	RECT rect = new RECT ();
 	OS.SendMessage (hwnd, OS.RB_GETRECT, index, rect);
-	if (OS.COMCTL32_MAJOR >= 6) {
-		MARGINS margins = new MARGINS ();
-		OS.SendMessage (hwnd, OS.RB_GETBANDMARGINS, 0, margins);
-		rect.left -= margins.cxLeftWidth;
-		rect.right += margins.cxRightWidth;
-	}
+	MARGINS margins = new MARGINS ();
+	OS.SendMessage (hwnd, OS.RB_GETBANDMARGINS, 0, margins);
+	rect.left -= margins.cxLeftWidth;
+	rect.right += margins.cxRightWidth;
 	if (!parent.isLastItemOfRow (index)) {
 		rect.right += (parent.style & SWT.FLAT) == 0 ? CoolBar.SEPARATOR_WIDTH : 0;
 	}
@@ -563,11 +562,9 @@ void setSizeInPixels (int width, int height) {
 	* Do not set the size for the last item on the row.
 	*/
 	if (!parent.isLastItemOfRow (index)) {
-		if (OS.COMCTL32_MAJOR >= 6) {
-			MARGINS margins = new MARGINS ();
-			OS.SendMessage (hwnd, OS.RB_GETBANDMARGINS, 0, margins);
-			cx -= margins.cxLeftWidth + margins.cxRightWidth;
-		}
+		MARGINS margins = new MARGINS ();
+		OS.SendMessage (hwnd, OS.RB_GETBANDMARGINS, 0, margins);
+		cx -= margins.cxLeftWidth + margins.cxRightWidth;
 		int separator = (parent.style & SWT.FLAT) == 0 ? CoolBar.SEPARATOR_WIDTH : 0;
 		rbBand.cx = cx - separator;
 		rbBand.fMask |= OS.RBBIM_SIZE;
