@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2012 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -64,17 +67,10 @@ public class Scale extends Control {
 		* code, other than SWT, could create a control with
 		* this class name, and fail unexpectedly.
 		*/
-		long /*int*/ hInstance = OS.GetModuleHandle (null);
-		long /*int*/ hHeap = OS.GetProcessHeap ();
-		lpWndClass.hInstance = hInstance;
+		lpWndClass.hInstance = OS.GetModuleHandle (null);
 		lpWndClass.style &= ~OS.CS_GLOBALCLASS;
 		lpWndClass.style |= OS.CS_DBLCLKS;
-		int byteCount = TrackBarClass.length () * TCHAR.sizeof;
-		long /*int*/ lpszClassName = OS.HeapAlloc (hHeap, OS.HEAP_ZERO_MEMORY, byteCount);
-		OS.MoveMemory (lpszClassName, TrackBarClass, byteCount);
-		lpWndClass.lpszClassName = lpszClassName;
 		OS.RegisterClass (lpWndClass);
-		OS.HeapFree (hHeap, 0, lpszClassName);
 	}
 
 /**
@@ -535,7 +531,7 @@ LRESULT WM_PAINT (long /*int*/ wParam, long /*int*/ lParam) {
 	*/
 	boolean fixPaint = findBackgroundControl () != null;
 	if (!fixPaint) {
-		if (OS.COMCTL32_MAJOR >= 6 && OS.IsAppThemed ()) {
+		if (OS.IsAppThemed ()) {
 			Control control = findThemeControl ();
 			fixPaint = control != null;
 		}
