@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2012 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -223,10 +226,8 @@ int defaultForeground () {
 @Override
 void enableWidget (boolean enabled) {
 	super.enableWidget (enabled);
-	if (!OS.IsWinCE) {
-		int flags = enabled ? OS.ESB_ENABLE_BOTH : OS.ESB_DISABLE_BOTH;
-		OS.EnableScrollBar (handle, OS.SB_CTL, flags);
-	}
+	int flags = enabled ? OS.ESB_ENABLE_BOTH : OS.ESB_DISABLE_BOTH;
+	OS.EnableScrollBar (handle, OS.SB_CTL, flags);
 	if (enabled) {
 		state &= ~DISABLED;
 	} else {
@@ -493,9 +494,7 @@ boolean SetScrollInfo (long /*int*/ hwnd, int flags, SCROLLINFO info, boolean fR
 	boolean result = OS.SetScrollInfo (hwnd, flags, info, fRedraw);
 	if ((state & DISABLED) != 0) {
 		OS.EnableWindow (handle, false);
-		if (!OS.IsWinCE) {
-			OS.EnableScrollBar (handle, OS.SB_CTL, OS.ESB_DISABLE_BOTH);
-		}
+		OS.EnableScrollBar (handle, OS.SB_CTL, OS.ESB_DISABLE_BOTH);
 	}
 
 	/*
@@ -692,11 +691,9 @@ LRESULT WM_LBUTTONDBLCLK (long /*int*/ wParam, long /*int*/ lParam) {
 	* is consumed.  The fix is to send a fake mouse up and
 	* release the automatic capture.
 	*/
-	if (!OS.IsWinCE) {
-		if (OS.GetCapture () == handle) OS.ReleaseCapture ();
-		if (!sendMouseEvent (SWT.MouseUp, 1, handle, OS.WM_LBUTTONUP, wParam, lParam)) {
-			return LRESULT.ZERO;
-		}
+	if (OS.GetCapture () == handle) OS.ReleaseCapture ();
+	if (!sendMouseEvent (SWT.MouseUp, 1, handle, OS.WM_LBUTTONUP, wParam, lParam)) {
+		return LRESULT.ZERO;
 	}
 	return result;
 }
@@ -726,11 +723,9 @@ LRESULT WM_LBUTTONDOWN (long /*int*/ wParam, long /*int*/ lParam) {
 	* is consumed.  The fix is to send a fake mouse up and
 	* release the automatic capture.
 	*/
-	if (!OS.IsWinCE) {
-		if (OS.GetCapture () == handle) OS.ReleaseCapture ();
-		if (!sendMouseEvent (SWT.MouseUp, 1, handle, OS.WM_LBUTTONUP, wParam, lParam)) {
-			return LRESULT.ONE;
-		}
+	if (OS.GetCapture () == handle) OS.ReleaseCapture ();
+	if (!sendMouseEvent (SWT.MouseUp, 1, handle, OS.WM_LBUTTONUP, wParam, lParam)) {
+		return LRESULT.ONE;
 	}
 	return result;
 }
