@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2008, 2016 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -225,10 +228,10 @@ public class ControlsWithAccessibleNamesExample {
 
 //		List list, overrideList;
 		list = new List(shell, SWT.SINGLE | SWT.BORDER);
-		list.setItems(new String[] {"Item0", "Item1", "Item2"});
+		list.setItems("Item0", "Item1", "Item2");
 
 		overrideList = new List(shell, SWT.SINGLE | SWT.BORDER);
-		overrideList.setItems(new String[] {"Item0", "Item1", "Item2"});
+		overrideList.setItems("Item0", "Item1", "Item2");
 		overrideList.setData("name", "List");
 		overrideList.setData("child", "List Item");
 		overrideList.getAccessible().addAccessibleListener(overrideAccessibleAdapter);
