@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2013 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -79,7 +82,6 @@ class GroupTab extends Tab {
 		if (shadowInButton.getSelection ()) style |= SWT.SHADOW_IN;
 		if (shadowOutButton.getSelection ()) style |= SWT.SHADOW_OUT;
 		if (shadowNoneButton.getSelection ()) style |= SWT.SHADOW_NONE;
-		if (borderButton.getSelection ()) style |= SWT.BORDER;
 
 		/* Create the example widgets */
 		group1 = new Group (groupGroup, style);
@@ -156,7 +158,7 @@ class GroupTab extends Tab {
 		shadowInButton.setSelection ((group1.getStyle () & SWT.SHADOW_IN) != 0);
 		shadowOutButton.setSelection ((group1.getStyle () & SWT.SHADOW_OUT) != 0);
 		shadowNoneButton.setSelection ((group1.getStyle () & SWT.SHADOW_NONE) != 0);
-		borderButton.setSelection ((group1.getStyle () & SWT.BORDER) != 0);
+		borderButton.setEnabled(false);
 		if (!instance.startup) setTitleText ();
 	}
 }
