@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2013 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -13,9 +16,13 @@ package org.eclipse.swt.examples.controlexample;
 
 import static org.eclipse.swt.events.SelectionListener.widgetSelectedAdapter;
 
-import org.eclipse.swt.*;
-import org.eclipse.swt.widgets.*;
-import org.eclipse.swt.layout.*;
+import org.eclipse.swt.SWT;
+import org.eclipse.swt.layout.GridData;
+import org.eclipse.swt.layout.GridLayout;
+import org.eclipse.swt.widgets.Button;
+import org.eclipse.swt.widgets.Group;
+import org.eclipse.swt.widgets.Spinner;
+import org.eclipse.swt.widgets.Widget;
 
 abstract class RangeTab extends Tab {
 	/* Style widgets added to the "Style" group */
@@ -153,7 +160,7 @@ abstract class RangeTab extends Tab {
 				horizontalButton.setSelection ((widgets [0].getStyle () & SWT.HORIZONTAL) != 0);
 				verticalButton.setSelection ((widgets [0].getStyle () & SWT.VERTICAL) != 0);
 			}
-			borderButton.setSelection ((widgets [0].getStyle () & SWT.BORDER) != 0);
+			borderButton.setEnabled(false);
 		}
 	}
 
