@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2016 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -14,7 +17,6 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.Comparator;
 import java.util.HashSet;
 import java.util.List;
 import java.util.MissingResourceException;
@@ -382,12 +384,7 @@ static Image createImage(Device device, Color color) {
 
 void createTabList(Composite parent) {
 	tabList = new Tree(parent, SWT.SINGLE | SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER);
-	Arrays.sort(tabs, new Comparator<GraphicsTab>() {
-		@Override
-		public int compare(GraphicsTab tab0, GraphicsTab tab1) {
-			return tab0.getText().compareTo(tab1.getText());
-		}
-	});
+	Arrays.sort(tabs, (tab0, tab1) -> tab0.getText().compareTo(tab1.getText()));
 	HashSet<String> set = new HashSet<>();
 	for (GraphicsTab tab : tabs) {
 		set.add(tab.getCategory());
