@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2016 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -31,10 +34,10 @@ public static void main (String [] args) {
 	Shell shell = new Shell (display);
 	shell.setLayout (new RowLayout ());
 
-	DateTime calendar = new DateTime (shell, SWT.CALENDAR);
+	DateTime calendar = new DateTime (shell, SWT.CALENDAR | SWT.CALENDAR_WEEKNUMBERS);
 	calendar.addSelectionListener (widgetSelectedAdapter(e -> System.out.println ("calendar date changed")));
 
-	DateTime calendarDropDown = new DateTime (shell, SWT.DROP_DOWN);
+	DateTime calendarDropDown = new DateTime (shell, SWT.DROP_DOWN | SWT.CALENDAR_WEEKNUMBERS);
 	calendarDropDown.addSelectionListener (
 		widgetSelectedAdapter(e -> System.out.println ("calendar date changed via drop-down")));
 
