@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2016 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2018 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -12,6 +15,7 @@ package org.eclipse.swt.tests.junit;
 
 import static org.junit.Assert.assertArrayEquals;
 import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 
@@ -76,7 +80,7 @@ public void test_addLjava_lang_String() {
 	assertArrayEquals(":b:", new String[] {"", "some \n text"}, list.getItems());
 	list.add("some text");
 	assertArrayEquals(":c:", new String[] {"", "some \n text", "some text"}, list.getItems());
-	
+
 	// test single-selection list
 
 	setSingleList();
@@ -86,7 +90,7 @@ public void test_addLjava_lang_String() {
 		fail("No exception thrown");
 	} catch (IllegalArgumentException e) {
 	}
-	
+
 	list.add("");
 	assertArrayEquals(":a:", new String[] {""}, list.getItems());
 	list.add("some \n text");
@@ -110,7 +114,7 @@ public void test_addLjava_lang_StringI() {
 	assertArrayEquals(":b:", new String[] {"", "some \n text"}, list.getItems());
 	list.add("some text", 0);
 	assertArrayEquals(":c:", new String[] {"some text", "", "some \n text" }, list.getItems());
-	
+
 	try {
 		list.add(null, 0);
 		fail("No exception thrown string == null");
@@ -122,7 +126,7 @@ public void test_addLjava_lang_StringI() {
 		fail("No exception thrown index < 0");
 	} catch (IllegalArgumentException e) {
 	}
-	
+
 	// test single-selection list
 
 	setSingleList();
@@ -134,14 +138,14 @@ public void test_addLjava_lang_StringI() {
 	}
 
 	assertEquals(0, list.getItemCount());
-	
+
 	list.add("", 0);
 	assertArrayEquals(":a:", new String[] {""}, list.getItems());
 	list.add("some \n text", 1);
 	assertArrayEquals(":b:", new String[] {"", "some \n text"}, list.getItems());
 	list.add("some text", 0);
 	assertArrayEquals(":c:", new String[] {"some text", "", "some \n text" }, list.getItems());
-	
+
 	try {
 		list.add(null, 0);
 		fail("No exception thrown string == null");
@@ -175,10 +179,10 @@ public void test_addSelectionListenerLorg_eclipse_swt_events_SelectionListener()
 		exceptionThrown = true;
 	}
 	assertTrue("Expected exception not thrown for listener == null", exceptionThrown);
-	
+
 	list.addSelectionListener(listener);
 	list.select(0);
-	assertTrue(":a:", listenerCalled == false);
+	assertFalse(":a:", listenerCalled);
 	list.removeSelectionListener(listener);
 	exceptionThrown = false;
 	try {
@@ -188,11 +192,11 @@ public void test_addSelectionListenerLorg_eclipse_swt_events_SelectionListener()
 		exceptionThrown = true;
 	}
 	assertTrue("Expected exception not thrown for listener == null", exceptionThrown);
-	
+
 	// test single-selection list
 
 	setSingleList();
-	
+
 	listenerCalled = false;
 	exceptionThrown = false;
 	try {
@@ -202,10 +206,10 @@ public void test_addSelectionListenerLorg_eclipse_swt_events_SelectionListener()
 		exceptionThrown = true;
 	}
 	assertTrue("Expected exception not thrown for listener == null", exceptionThrown);
-	
+
 	list.addSelectionListener(listener);
 	list.select(0);
-	assertTrue(":a:", listenerCalled == false);
+	assertFalse(":a:", listenerCalled);
 	list.removeSelectionListener(listener);
 	exceptionThrown = false;
 	try {
@@ -215,7 +219,26 @@ public void test_addSelectionListenerLorg_eclipse_swt_events_SelectionListener()
 		exceptionThrown = true;
 	}
 	assertTrue("Expected exception not thrown for listener == null", exceptionThrown);
-	
+}
+
+@Test
+public void test_addSelectionListenerWidgetSelectedAdapterLorg_eclipse_swt_events_SelectionListener() {
+	listenerCalled = false;
+	SelectionListener listener = SelectionListener.widgetSelectedAdapter(e -> listenerCalled = true);
+
+	list.addSelectionListener(listener);
+	list.select(0);
+	assertFalse(":a:", listenerCalled);
+	list.removeSelectionListener(listener);
+
+	// test single-selection list
+
+	setSingleList();
+
+	list.addSelectionListener(listener);
+	list.select(0);
+	assertFalse(":a:", listenerCalled);
+	list.removeSelectionListener(listener);
 }
 
 @Override
@@ -249,7 +272,7 @@ public void test_deselect$I() {
 	assert(":ddd:", list.isSelected(1));
 	assert(":ddd:", list.isSelected(1));*/
 
-	
+
 	setSingleList();
 	list.setItems(items);
 	list.setSelection(new String[] { "item3" });
@@ -285,7 +308,7 @@ public void test_deselectAll() {
 	list.deselectAll();
 	assertArrayEquals(empty, list.getSelection());
 
-	
+
 	setSingleList();
 
 	list.setItems(items);
@@ -311,11 +334,11 @@ public void test_deselectI() {
 	list.deselect(5);
 	assertArrayEquals(items2, list.getSelection());
 	list.deselect(1);
-	assertEquals(false, list.isSelected(1));
+	assertFalse(list.isSelected(1));
 	list.deselect(1);
-	assertEquals(false, list.isSelected(1));
+	assertFalse(list.isSelected(1));
+
 
-	
 	setSingleList();
 	list.setItems(items2);
 	list.setSelection(new String[] { "item3" });
@@ -323,11 +346,11 @@ public void test_deselectI() {
 	list.deselect(5);
 	assertArrayEquals(new String[] { "item3" }, list.getSelection());
 	list.deselect(2);
-	assertEquals(false, list.isSelected(2));
+	assertFalse(list.isSelected(2));
 	list.deselect(1);
-	assertEquals(false, list.isSelected(1));
+	assertFalse(list.isSelected(1));
 	list.deselect(1);
-	assertEquals(false, list.isSelected(1));
+	assertFalse(list.isSelected(1));
 }
 
 @Test
@@ -380,12 +403,12 @@ public void test_deselectII() {
 		":ddddd:",
 		list.getSelectionIndices(), new int[] { 0, 1, 2, 3 });
 
-	
+
 	setSingleList();
-	
+
 	list.setItems(items2);
 	list.deselectAll();
-	
+
 	list.select(0);
 
 	list.deselect(-3, -2);
@@ -397,7 +420,7 @@ public void test_deselectII() {
 	list.deselect(-1, -1);
 	assertArrayEquals(":e:", list.getSelectionIndices(), new int[] { 0 });
 
-	
+
 	list.setSelection(new String[] { "item3" });
 	assertArrayEquals(list.getSelection(), new String[] { "item3" });
 
@@ -445,7 +468,7 @@ public void test_getItemCount() {
 	list.removeAll();
 	assertEquals(0, list.getItemCount());
 
-	
+
 	setSingleList();
 	assertEquals(0, list.getItemCount());
 	list.setItems(items);
@@ -469,7 +492,7 @@ public void test_getItemHeight() {
 	FontData fontData = list.getFont().getFontData()[0];
 	int lineHeight;
 	Font font;
-	
+
 	font = new Font(list.getDisplay(), fontData.getName(), 8, fontData.getStyle());
 	list.setFont(font);
 	lineHeight = list.getItemHeight();
@@ -498,10 +521,10 @@ public void test_getItemI() {
 		fail("No exception thrown for index < 0");
 	} catch (IllegalArgumentException e) {
 	}
-	
+
 	assertEquals(list.getItem(3), "item3");
 
-	
+
 	setSingleList();
 	list.setItems(items);
 	try {
@@ -509,13 +532,13 @@ public void test_getItemI() {
 		fail("No exception thrown");
 	} catch (IllegalArgumentException e) {
 	}
-	
+
 	try {
 		list.getItem(-1);
 		fail("No exception thrown for index < 0");
 	} catch (IllegalArgumentException e) {
 	}
-	
+
 	//assert(":a:", list.getItem(5)==null);
 	assertEquals("item3", list.getItem(3));
 
@@ -572,7 +595,7 @@ public void test_getSelection() {
 	assertEquals(list.getSelection().length, 2);
 	selItems = new String[] {items[0], items[1]};
 	assertArrayEquals(list.getSelection(), selItems);
-	
+
 	list.setSelection(1, 1);
 	assertEquals(list.getSelection().length, 1);
 	assertEquals(list.getSelection()[0], items[1]);
@@ -586,7 +609,7 @@ public void test_getSelection() {
 	list.setSelection(bogus_items);
 	assertArrayEquals(empty, list.getSelection());
 
-	// test single-selection lists 
+	// test single-selection lists
 
 	setSingleList();
 
@@ -632,7 +655,7 @@ public void test_getSelectionCount() {
 	}
 	assertEquals(list.getSelectionCount(), 0);
 
-	
+
 	setSingleList();
 
 	list.setItems(items);
@@ -649,7 +672,7 @@ public void test_getSelectionIndex() {
 	list.setSelection(items);
 	assertEquals(0, list.getSelectionIndex());
 
-	
+
 	setSingleList();
 
 	list.setItems(items);
@@ -708,7 +731,7 @@ public void test_getSelectionIndices() {
 	list.setSelection(sel);
 	assertArrayEquals(new int[] { 1, 2 }, list.getSelectionIndices());
 
-	
+
 	setSingleList();
 
 	list.setItems(items);
@@ -732,12 +755,17 @@ public void test_getSelectionIndices() {
 
 @Test
 public void test_getTopIndex() {
-	String[] items = { "text1", "text2", "text3" };
+	String[] items = new String [10];
+	for (int i = 0; i < 10; i++) {
+		items[i] = "text" + String.valueOf(i);
+	}
 
 	list.setItems(items);
 	assertEquals(0, list.getTopIndex());
 
-	
+	list.setTopIndex(5);
+	assertEquals(5, list.getTopIndex());
+
 	setSingleList();
 
 	list.setItems(items);
@@ -764,26 +792,26 @@ public void test_indexOfLjava_lang_String() {
 	list.setItems(items2);
 	assertEquals(list.indexOf("text2"), 1);
 
-	
+
 	setSingleList();
 
 	list.setItems(items);
 	assertEquals(-1, list.indexOf("text3", 4));
 
-	
+
 	assertEquals(2, list.indexOf("text3"));
 
-	
+
 	assertEquals(-1, list.indexOf("text4"));
 
-	
+
 	try {
 		list.indexOf(null);
 		fail("No exception thrown");
 	} catch (IllegalArgumentException e) {
 	}
 
-	
+
 	assertEquals(1, list.indexOf("text2"));
 }
 
@@ -807,7 +835,7 @@ public void test_indexOfLjava_lang_StringI() {
 		fail("No exception thrown for string == null");
 	} catch (IllegalArgumentException e) {
 	}
-	
+
 	setSingleList();
 
 	list.setItems(items2);
@@ -832,18 +860,18 @@ public void test_isSelectedI() {
 	assertTrue(list.isSelected(0));
 	assertTrue(list.isSelected(1));
 	assertTrue(list.isSelected(2));
-	assertEquals(false, list.isSelected(3));
+	assertFalse(list.isSelected(3));
+
 
-	
 	setSingleList();
 
 	list.setItems(items);
 	list.setSelection(items);
 	if (SwtTestUtil.fCheckSWTPolicy) {
-		assertEquals(false, list.isSelected(0));
+		assertFalse(list.isSelected(0));
 		assertTrue(list.isSelected(1));
-		assertEquals(false, list.isSelected(2));
-		assertEquals(false, list.isSelected(3));
+		assertFalse(list.isSelected(2));
+		assertFalse(list.isSelected(3));
 	}
 
 }
@@ -855,7 +883,7 @@ public void test_remove$I() {
 		fail("No exception thrown for indices == null");
 	} catch (IllegalArgumentException e) {
 	}
-	
+
 	String[] items = { "text0", "text1", "text2", "text3" };
 
 	list.setItems(items);
@@ -913,7 +941,7 @@ public void test_remove$I() {
 	list.remove(new int[] { 1, 3 });
 	assertArrayEquals(":e:", list.getItems(), new String[] { "text0", "text2" });
 
-	
+
 	setSingleList();
 
 	try {
@@ -922,21 +950,21 @@ public void test_remove$I() {
 		fail("No exception thrown for indices == null");
 	} catch (IllegalArgumentException e) {
 	}
-	
+
 	list.setItems(items);
 	assertEquals(4, list.getItemCount());
 
 	list.remove(new int[] { 1, 3 });
 	assertArrayEquals(":f:", list.getItems(), new String[] { "text0", "text2" });
 
-	
+
 	list.setItems(items);
 	assertEquals(4, list.getItemCount());
 
 	list.remove(new int[] { 3, 1 });
 	assertArrayEquals(":g:", list.getItems(), new String[] { "text0", "text2" });
 
-	
+
 	list.setItems(items);
 	assertEquals(4, list.getItemCount());
 
@@ -945,17 +973,17 @@ public void test_remove$I() {
 		list.remove(new int[] { 4, 1});
 		fail("No exception thrown");
 	} catch (IllegalArgumentException e) {
-	} 
+	}
 	assertArrayEquals(":h:", list.getItems(), items);
 
 	try {
 		list.remove(new int[] { 3, 1, -1 });
 		fail("No exception thrown");
 	} catch (IllegalArgumentException e) {
-	} 
+	}
 	assertArrayEquals(":h:", list.getItems(), items);
 
-	
+
 	list.setItems(items);
 	assertEquals(4, list.getItemCount());
 
@@ -967,7 +995,7 @@ public void test_remove$I() {
 
 	assertArrayEquals(":i:", items, list.getItems());
 
-	
+
 	assertEquals(4, list.getItemCount());
 
 	list.remove(new int[] { 1, 1, 1 });
@@ -988,7 +1016,7 @@ public void test_removeAll() {
 	list.removeAll();
 	assertEquals(list.getItemCount(), 0);
 
-	
+
 	setSingleList();
 	list.setItems(items);
 	assertEquals(4, list.getItemCount());
@@ -996,7 +1024,7 @@ public void test_removeAll() {
 	list.removeAll();
 	assertEquals(0, list.getItemCount());
 
-	
+
 	setSingleList();
 	assertEquals(0, list.getItemCount());
 	list.removeAll();
@@ -1055,13 +1083,13 @@ public void test_removeI() {
 	assertEquals(list.getItem(0), "text3");
 	list.remove(0);
 	assertEquals(list.getItemCount(), 0);
-	
+
 	list.setItems(items);
 	list.remove(1, 2);
 	assertEquals(list.getItemCount(), 1);
 	assertEquals(list.getItem(0), "text1");
 
-	
+
 	setSingleList();
 	list.setItems(items);
 	assertEquals(3, list.getItemCount());
@@ -1069,7 +1097,7 @@ public void test_removeI() {
 		list.remove(3);
 		fail("No exception thrown");
 	} catch (IllegalArgumentException e) {
-	} 
+	}
 	assertEquals(3, list.getItemCount());
 	/////////////////////////////////////////////////
 	try {
@@ -1077,7 +1105,7 @@ public void test_removeI() {
 		fail("No exception thrown");
 	} catch (IllegalArgumentException e) {
 	}
-	
+
 	assertEquals(3, list.getItemCount());
 	////////////////////////////////////////////////
 	list.remove(1);
@@ -1095,7 +1123,7 @@ public void test_removeI() {
 	assertEquals(list.getItem(0), "text3");
 	list.remove(0);
 	assertEquals(list.getItemCount(), 0);
-	
+
 }
 
 @Test
@@ -1157,7 +1185,7 @@ public void test_removeII() {
 	list.remove(2, 0);
 	assertEquals(3, list.getItemCount());
 
-	
+
 	setSingleList();
 
 	list.setItems(items);
@@ -1179,19 +1207,19 @@ public void test_removeII() {
 
 	assertEquals(3, list.getItemCount());
 	//////////////////////////////////////////////////////////////
-	
+
 	try {
 		list.remove(-1, 1);
 		fail("No exception thrown for start index < 0");
 	} catch (IllegalArgumentException e) {
 	}
 	assertEquals(3, list.getItemCount());
-	
+
 	list.remove(1, 2);
 	assertEquals(1, list.getItemCount());
 	assertEquals("text1", list.getItem(0));
 
-	
+
 	list.setItems(items);
 	assertEquals(3, list.getItemCount());
 
@@ -1237,7 +1265,7 @@ public void test_removeLjava_lang_String() {
 	list.remove("text2");
 	assertEquals(list.getItemCount(), 2);
 
-	
+
 	setSingleList();
 	list.setItems(items);
 	assertEquals(4, list.getItemCount());
@@ -1256,7 +1284,7 @@ public void test_removeLjava_lang_String() {
 	}
 	assertEquals(4, list.getItemCount());
 
-	
+
 	assertEquals(4, list.getItemCount());
 
 	list.remove("text3");
@@ -1274,7 +1302,7 @@ public void test_select$I() {
 		fail("No exception thrown");
 	} catch (IllegalArgumentException e) {
 	}
-	
+
 	String[] items = { "item0", "item1", "item2", "item3" };
 	list.setItems(items);
 
@@ -1319,12 +1347,12 @@ public void test_select$I() {
 	select$I_helper(-1, 3, new int[] { 0, 1, 2, 3 });
 	select$I_helper(-1, 30, new int[] { 0, 1, 2, 3, 4 });
 
-	
+
 	/*--- Single-select ---*/
-	
+
 	setSingleList();
 	list.setItems(items);
-	
+
 	try {
 		list.select((int[]) null);
 		fail("No exception thrown");
@@ -1345,7 +1373,7 @@ public void test_select$I() {
 
 	list.select(new int[] { 3 });
 	assertArrayEquals(list.getSelectionIndices(), new int[] { 3 });
-	
+
 	list.select(new int[] { 4 });
 	assertArrayEquals(list.getSelectionIndices(), new int[] { 4 });
 
@@ -1377,7 +1405,7 @@ public void test_selectAll() {
 	list.selectAll();
 	assertEquals(list.getSelectionCount(), 4);
 
-	
+
 	setSingleList();
 
 	list.setItems(items);
@@ -1411,7 +1439,7 @@ public void test_selectI() {
 	list.deselectAll();
 	list.select(0);
 	assertArrayEquals("select(0):", list.getSelectionIndices(), new int[] { 0 });
-	
+
 	list.deselectAll();
 	list.select(-1);
 	assertArrayEquals("select(-1):", list.getSelectionIndices(), new int[] {});
@@ -1442,7 +1470,7 @@ public void test_selectI() {
 	list.deselectAll();
 	list.select(0);
 	assertArrayEquals("select(0):", list.getSelectionIndices(), new int[] { 0 });
-	
+
 	list.deselectAll();
 	list.select(-1);
 	assertArrayEquals("select(-1):", list.getSelectionIndices(), new int[] {});
@@ -1510,7 +1538,7 @@ public void test_selectII() {
 	list.select(3, 5);
 	assertArrayEquals(list.getSelectionIndices(), new int[] { 1, 2, 3, 4 });
 
-	
+
 	setSingleList();
 	list.select(0, 0);
 	assertArrayEquals("empty list", list.getSelectionIndices(), new int[] {});
@@ -1529,10 +1557,10 @@ public void test_selectII() {
 
 	list.deselectAll();
 	assertEquals(0, list.getSelectionCount());
-	
+
 	list.select(1000, 2000);
 	assertEquals(0, list.getSelectionCount());
-	
+
 	list.deselectAll();
 	assertEquals(0, list.getSelectionCount());
 
@@ -1591,7 +1619,7 @@ public void test_setFontLorg_eclipse_swt_graphics_Font() {
 	FontData fontData = list.getFont().getFontData()[0];
 	int lineHeight;
 	Font font;
-	
+
 	font = new Font(list.getDisplay(), fontData.getName(), 8, fontData.getStyle());
 	list.setFont(font);
 	lineHeight = list.getItemHeight();
@@ -1608,7 +1636,7 @@ public void test_setFontLorg_eclipse_swt_graphics_Font() {
 @Test
 public void test_setItemILjava_lang_String() {
 	String[] items = { "item0", "item1", "item2", "item3" };
-	
+
 	assertEquals(list.getItemCount(), 0);
 	int[] cases = { -10, 0, 10 };
 	for (int i = 0; i < cases.length; i++) {
@@ -1642,7 +1670,7 @@ public void test_setItemILjava_lang_String() {
 	list.setItems(items);
 	list.setItem(1, "new1");
 	assertArrayEquals(new String[] { "item0", "new1", "item2", "item3" }, list.getItems());
-	
+
 	setSingleList();
 	assertEquals(0, list.getItemCount());
 	for (int i = 0; i < cases.length; i++) {
@@ -1654,7 +1682,7 @@ public void test_setItemILjava_lang_String() {
 
 	}
 
-	
+
 	setSingleList();
 	for (int i = 0; i < cases.length; i++) {
 		try {
@@ -1666,7 +1694,7 @@ public void test_setItemILjava_lang_String() {
 
 	assertEquals(0, list.getItemCount());
 
-	
+
 	setSingleList();
 	for (int i = 0; i < cases2.length; i++) {
 		try {
@@ -1677,23 +1705,23 @@ public void test_setItemILjava_lang_String() {
 
 		assertEquals(0, list.getItemCount());
 	}
-	
+
 	list.setItems(items);
 	list.setItem(1, "new1");
 	assertArrayEquals(new String[] { "item0", "new1", "item2", "item3" }, list.getItems());
-	
+
 }
 
 @Test
 public void test_setItems$Ljava_lang_String() {
 	try {
-		list.setItems(null);
+		list.setItems((String[])null);
 		fail("No exception thrown for items == null");
 	} catch (IllegalArgumentException e) {
 	}
 
 	// TODO An SWTError should never happen and should not
-	// be part of the test case.  List should throw an 
+	// be part of the test case.  List should throw an
 	// SWTException.
 //	String[][] cases = { { null }, {
 //			"dsada", null, "dsdasdasd" }
@@ -1718,21 +1746,21 @@ public void test_setItems$Ljava_lang_String() {
 	}
 
 	try {
-		list.setItems(null);
+		list.setItems((String[])null);
 		fail("No exception thrown");
 	} catch (IllegalArgumentException e) {
 	}
 
-	
+
 	setSingleList();
 	for (int i = 0; i < itemArr.length; i++) {
 		list.setItems(itemArr[i]);
 		assertArrayEquals("case:" + i, itemArr[i], list.getItems());
 	}
 
-	
+
 	try {
-		list.setItems(null);
+		list.setItems((String[])null);
 		fail("No exception thrown");
 	} catch (IllegalArgumentException e) {
 	}
@@ -1752,14 +1780,14 @@ public void test_setSelection$I() {
 		fail("No exception thrown for MULTI: setSelection((int[]) null)");
 	} catch (IllegalArgumentException e) {
 	}
-	
+
 	list.setSelection(new int [] {2});
 	assertArrayEquals("MULTI: setSelection(new int [] {2})", list.getSelectionIndices(), new int[] {2});
 	assertEquals("MULTI: setSelection(new int [] {2}) getFocusIndex()", list.getFocusIndex(), 2);
 
 	list.setSelection(new int [] {number});
 	assertArrayEquals("MULTI: setSelection(new int [] {number})", list.getSelectionIndices(), new int[0]);
-	
+
 	list.setSelection(new int [] {3, 1, 5, 2});
 	assertArrayEquals("MULTI: setSelection(new int [] {3, 1, 5, 2})", list.getSelectionIndices(), new int[] {1, 2, 3, 5});
 
@@ -1768,14 +1796,14 @@ public void test_setSelection$I() {
 
 	list.setSelection(new int [] {-1, number});
 	assertArrayEquals("MULTI: setSelection(new int [] {-1, number})", list.getSelectionIndices(), new int[0]);
-	
+
 	list.setSelection(new int [] {number - 1, number});
 	assertArrayEquals("MULTI: setSelection(new int [] {number - 1, number})", list.getSelectionIndices(), new int[] {number - 1});
 	assertEquals("MULTI: setSelection(new int [] {number - 1, number}) getFocusIndex()", list.getFocusIndex(), number - 1);
 
 	list.setSelection(new int [] {-1, 0});
 	assertArrayEquals("MULTI: setSelection(new int [] {-1, 0})", list.getSelectionIndices(), new int[] {0});
-	
+
 	list.setSelection(new int [] {0, 1, 2, 3, 5});
 	assertArrayEquals("MULTI: setSelection(new int [] {0, 1, 2, 3, 5})", list.getSelectionIndices(), new int [] {0, 1, 2, 3, 5});
 
@@ -1785,22 +1813,22 @@ public void test_setSelection$I() {
 	}
 	list.setSelection(indices);
 	assertArrayEquals("MULTI: setSelection(indices)", indices, list.getSelectionIndices());
-	
+
 	list.setSelection(new int [] {number, number});
 	assertArrayEquals("MULTI: setSelection(new int [] {number, number})", new int[0], list.getSelectionIndices());
-	
+
 	list.setSelection(new int [] {number - 1, number - 1});
 	assertArrayEquals("MULTI: setSelection(new int [] {number - 1, number - 1})", list.getSelectionIndices(), new int[] {number - 1});
 	assertEquals("MULTI: setSelection(new int [] {number - 1, number - 1}) getFocusIndex()", list.getFocusIndex(), number - 1);
-	
+
 	list.setSelection(new int [] {0, number, 1});
 	assertArrayEquals("MULTI: setSelection(new int [] {0, number, 1})", list.getSelectionIndices(), new int[] {0, 1});
-	
+
 	list.setSelection(new int [] {number - 1, 0, number - 2});
 	assertArrayEquals("MULTI: setSelection(new int [] {number - 1, 0, number - 2})", list.getSelectionIndices(), new int[] {0, number - 2, number - 1});
-	
+
 	list.removeAll();
-	
+
 	list.setSelection(new int [0]);
 	assertArrayEquals("EMPTY MULTI: setSelection(new int [0])", list.getSelectionIndices(), new int[0]);
 	assertEquals("EMPTY MULTI: setSelection(new int [0]) getFocusIndex()", list.getFocusIndex(), -1);
@@ -1810,7 +1838,7 @@ public void test_setSelection$I() {
 		fail("No exception thrown for EMPTY MULTI: setSelection((int[]) null)");
 	} catch (IllegalArgumentException e) {
 	}
-	
+
 	list.setSelection(new int [] {0});
 	assertArrayEquals("EMPTY MULTI: setSelection(new int [] {0})", list.getSelectionIndices(), new int[0]);
 	assertEquals("EMPTY MULTI: setSelection(new int [] {0}) getFocusIndex()", list.getFocusIndex(), -1);
@@ -1822,16 +1850,16 @@ public void test_setSelection$I() {
 	list.setSelection(new int [] {0, 0});
 	assertArrayEquals("EMPTY MULTI: setSelection(new int [] {0, 0})", list.getSelectionIndices(), new int[0]);
 	assertEquals("EMPTY MULTI: setSelection(new int [] {0, 0}) getFocusIndex()", list.getFocusIndex(), -1);
-	
+
 	list.setSelection(new int [] {-1, 0});
 	assertArrayEquals("EMPTY MULTI: setSelection(new int [] {-1, 0})", list.getSelectionIndices(), new int[0]);
 	assertEquals("EMPTY MULTI: setSelection(new int [] {-1, 0}) getFocusIndex()", list.getFocusIndex(), -1);
-	
+
 	list.setSelection(new int [] {0, -1});
 	assertArrayEquals("EMPTY MULTI: setSelection(new int [] {0, -1})", list.getSelectionIndices(), new int[0]);
 	assertEquals("EMPTY MULTI: setSelection(new int [] {0, -1}) getFocusIndex()", list.getFocusIndex(), -1);
-	
-	
+
+
 	setSingleList();
 	for (int i = 0; i < number; i++)
 		list.add("fred" + i);
@@ -1844,14 +1872,14 @@ public void test_setSelection$I() {
 		fail("No exception thrown for SINGLE: setSelection((int[]) null)");
 	} catch (IllegalArgumentException e) {
 	}
-	
+
 	list.setSelection(new int [] {2});
 	assertArrayEquals("SINGLE: setSelection(new int [] {2})", list.getSelectionIndices(), new int[] {2});
 	assertEquals("SINGLE: setSelection(new int [] {2}) getFocusIndex()", list.getFocusIndex(), 2);
 
 	list.setSelection(new int [] {number});
 	assertArrayEquals("SINGLE: setSelection(new int [] {number})", list.getSelectionIndices(), new int[0]);
-	
+
 	list.setSelection(new int [] {1, 0});
 	assertArrayEquals("SINGLE: setSelection(new int [] {1, 0})", list.getSelectionIndices(), new int[] {});
 
@@ -1860,34 +1888,34 @@ public void test_setSelection$I() {
 
 	list.setSelection(new int [] {-1, number});
 	assertArrayEquals("SINGLE: setSelection(new int [] {-1, number})", list.getSelectionIndices(), new int[0]);
-	
+
 	list.setSelection(new int [] {number - 1, number});
 	assertArrayEquals("SINGLE: setSelection(new int [] {number - 1, number})", list.getSelectionIndices(), new int[] {});
 
 	list.setSelection(new int [] {-1, 0});
 	assertArrayEquals("SINGLE: setSelection(new int [] {-1, 0})", list.getSelectionIndices(), new int[] {});
-	
+
 	indices = new int [number];
 	for (int i = 0; i < number; i++) {
 		indices[i] = i;
 	}
 	list.setSelection(indices);
 	assertArrayEquals("SINGLE: setSelection(indices)", list.getSelectionIndices(), new int[] {});
-	
+
 	list.setSelection(new int [] {number, number});
 	assertArrayEquals("SINGLE: setSelection(new int [] {number, number})", list.getSelectionIndices(), new int[0]);
-	
+
 	list.setSelection(new int [] {number - 1, number - 1});
 	assertArrayEquals("SINGLE: setSelection(new int [] {number - 1, number - 1})", list.getSelectionIndices(), new int[] {});
-	
+
 	list.setSelection(new int [] {0, number, 1});
 	assertArrayEquals("SINGLE: setSelection(new int [] {0, number, 1})", list.getSelectionIndices(), new int[] {});
-	
+
 	list.setSelection(new int [] {number - 1, 0, number - 2});
 	assertArrayEquals("SINGLE: setSelection(new int [] {number - 1, 0, number - 2})", list.getSelectionIndices(), new int[] {});
-	
+
 	list.removeAll();
-	
+
 	list.setSelection(new int [0]);
 	assertArrayEquals("EMPTY SINGLE: setSelection(new int [0])", list.getSelectionIndices(), new int[0]);
 	assertEquals("EMPTY SINGLE: setSelection(new int [0]) getFocusIndex()", list.getFocusIndex(), -1);
@@ -1897,7 +1925,7 @@ public void test_setSelection$I() {
 		fail("No exception thrown for EMPTY SINGLE: setSelection((int[]) null)");
 	} catch (IllegalArgumentException e) {
 	}
-	
+
 	list.setSelection(new int [] {0});
 	assertArrayEquals("EMPTY SINGLE: setSelection(new int [] {0})", list.getSelectionIndices(), new int[0]);
 	assertEquals("EMPTY SINGLE: setSelection(new int [] {0}) getFocusIndex()", list.getFocusIndex(), -1);
@@ -1909,11 +1937,11 @@ public void test_setSelection$I() {
 	list.setSelection(new int [] {0, 0});
 	assertArrayEquals("EMPTY SINGLE: setSelection(new int [] {0, 0})", list.getSelectionIndices(), new int[0]);
 	assertEquals("EMPTY SINGLE: setSelection(new int [] {0, 0}) getFocusIndex()", list.getFocusIndex(), -1);
-	
+
 	list.setSelection(new int [] {-1, 0});
 	assertArrayEquals("EMPTY SINGLE: setSelection(new int [] {-1, 0})", list.getSelectionIndices(), new int[0]);
 	assertEquals("EMPTY SINGLE: setSelection(new int [] {-1, 0}) getFocusIndex()", list.getFocusIndex(), -1);
-	
+
 	list.setSelection(new int [] {0, -1});
 	assertArrayEquals("EMPTY SINGLE: setSelection(new int [] {0, -1})", list.getSelectionIndices(), new int[0]);
 	assertEquals("EMPTY SINGLE: setSelection(new int [] {0, -1}) getFocusIndex()", list.getFocusIndex(), -1);
@@ -1936,44 +1964,44 @@ public void test_setSelection$Ljava_lang_String() {
 		fail("No exception thrown");
 	} catch (IllegalArgumentException e) {
 	}
-	
+
 	list.setSelection(new String [] {"fred 2"});
 	assertArrayEquals(list.getSelection(), new String [] {"fred 2"});
 	assertEquals(list.getFocusIndex(), 2);
 
 	list.setSelection(new String [] {"fred " + number});
 	assertArrayEquals(list.getSelection(), new String [0]);
-	
+
 	list.setSelection(new String [] {"fred 1", "fred 0"});
 	assertArrayEquals(list.getSelection(), new String [] {"fred 0", "fred 1"});
 
 	list.setSelection(new String [] {"fred -1", "fred " + number});
 	assertArrayEquals(list.getSelection(), new String [0]);
-	
+
 	list.setSelection(new String [] {"fred " + (number - 1), "fred " + number});
 	assertArrayEquals(list.getSelection(), new String [] {"fred " + (number - 1)});
 
 	list.setSelection(new String [] {"fred -1", "fred 0"});
 	assertArrayEquals(list.getSelection(), new String [] {"fred 0"});
-	
+
 	String[] items = new String [number];
 	for (int i = 0; i < number; i++) {
 		items[i] = "fred " + i;
 	}
 	list.setSelection(items);
 	assertArrayEquals(list.getSelection(), items);
-	
+
 	list.setSelection(new String [] {"fred " + number, "fred " + number});
 	assertArrayEquals(list.getSelection(), new String [0]);
-	
+
 	list.setSelection(new String [] {"fred " + (number - 1), "fred " + (number - 1)});
 	assertArrayEquals(list.getSelection(), new String[] {"fred " + (number - 1)});
-	
+
 	list.setSelection(new String [] {"fred 0", "fred " + number, "fred 1"});
 	assertArrayEquals(list.getSelection(), new String[] {"fred 0", "fred 1"});
-	
+
 	list.removeAll();
-		
+
 	list.setSelection(new String [0]);
 	assertArrayEquals(list.getSelection(), new String[0]);
 	assertEquals(list.getFocusIndex(), -1);
@@ -1983,7 +2011,7 @@ public void test_setSelection$Ljava_lang_String() {
 		fail("No exception thrown");
 	} catch (IllegalArgumentException e) {
 	}
-		
+
 	list.setSelection(new String [] {"fred 0"});
 	assertArrayEquals(list.getSelection(), new String[0]);
 	assertEquals(list.getFocusIndex(), -1);
@@ -1992,7 +2020,7 @@ public void test_setSelection$Ljava_lang_String() {
 	assertArrayEquals(list.getSelection(), new String[0]);
 	assertEquals(list.getFocusIndex(), -1);
 
-	
+
 	setSingleList();
 	for (int i = 0; i < number; i++)
 		list.add("fred " + i);
@@ -2005,20 +2033,20 @@ public void test_setSelection$Ljava_lang_String() {
 		fail("No exception thrown");
 	} catch (IllegalArgumentException e) {
 	}
-	
+
 	list.setSelection(new String [] {"fred 2"});
 	assertArrayEquals(list.getSelection(), new String[] {"fred 2"});
 	assertEquals(list.getFocusIndex(), 2);
 
 	list.setSelection(new String [] {"fred " + number});
 	assertArrayEquals(list.getSelection(), new String[0]);
-	
+
 	list.setSelection(new String [] {"fred 1", "fred 0"});
 	assertArrayEquals(list.getSelection(), new String[] {});
 
 	list.setSelection(new String [] {"fred -1", "fred " + number});
 	assertArrayEquals(list.getSelection(), new String[0]);
-	
+
 	list.setSelection(new String [] {"fred " + (number - 1)});
 	assertArrayEquals(list.getSelection(), new String[] {"fred " + (number - 1)});
 
@@ -2028,18 +2056,18 @@ public void test_setSelection$Ljava_lang_String() {
 	}
 	list.setSelection(items);
 	assertArrayEquals(list.getSelection(), new String[] {});
-	
+
 	list.setSelection(new String [] {"fred " + number, "fred " + number});
 	assertArrayEquals(list.getSelection(), new String[0]);
-	
+
 	list.setSelection(new String [] {"fred " + (number - 1), "fred " + (number - 1)});
 	assertArrayEquals(list.getSelection(), new String[] {});
-	
+
 	list.setSelection(new String [] {"fred 0", "fred " + number, "fred 1"});
 	assertArrayEquals(list.getSelection(), new String[] {});
-	
+
 	list.removeAll();
-		
+
 	list.setSelection(new String [0]);
 	assertArrayEquals(list.getSelection(), new String[0]);
 	assertEquals(list.getFocusIndex(), -1);
@@ -2049,7 +2077,7 @@ public void test_setSelection$Ljava_lang_String() {
 		fail("No exception thrown");
 	} catch (IllegalArgumentException e) {
 	}
-	
+
 	list.setSelection(new String [] {"fred 0"});
 	assertArrayEquals(list.getSelection(), new String[0]);
 	assertEquals(list.getFocusIndex(), -1);
@@ -2069,37 +2097,37 @@ public void test_setSelectionI() {
 	list.setSelection(2);
 	assertArrayEquals(list.getSelectionIndices(), new int[] {2});
 	assertEquals(list.getFocusIndex(), 2);
-	
+
 	list.setSelection(-5);
 	assertArrayEquals(list.getSelectionIndices(), new int[0]);
-	
+
 	list.setSelection(0);
 	assertArrayEquals(list.getSelectionIndices(), new int[] {0});
 	assertEquals(list.getFocusIndex(), 0);
-	
+
 	list.setSelection(number);
 	assertArrayEquals(list.getSelectionIndices(), new int[0]);
-	
+
 	list.setSelection(number - 1);
 	assertArrayEquals(list.getSelectionIndices(), new int[] {number - 1});
-	
+
 	list.setSelection(-1);
 	assertArrayEquals(list.getSelectionIndices(), new int[0]);
-	
+
 	list.removeAll();
-	
+
 	list.setSelection(-2);
 	assertArrayEquals(list.getSelectionIndices(), new int[0]);
 	assertEquals(list.getFocusIndex(), -1);
-		
+
 	list.setSelection(0);
 	assertArrayEquals(list.getSelectionIndices(), new int[0]);
 	assertEquals(list.getFocusIndex(), -1);
-		
+
 	list.setSelection(-1);
 	assertArrayEquals(list.getSelectionIndices(), new int[0]);
 	assertEquals(list.getFocusIndex(), -1);
-	
+
 	setSingleList();
 
 	for (int i = 0; i < number; i++) {
@@ -2109,34 +2137,34 @@ public void test_setSelectionI() {
 	list.setSelection(2);
 	assertArrayEquals(list.getSelectionIndices(), new int[] {2});
 	assertEquals(list.getFocusIndex(), 2);
-	
+
 	list.setSelection(-5);
 	assertArrayEquals(list.getSelectionIndices(), new int[0]);
-	
+
 	list.setSelection(0);
 	assertArrayEquals(list.getSelectionIndices(), new int[] {0});
 	assertEquals(list.getFocusIndex(), 0);
-	
+
 	list.setSelection(number);
 	assertArrayEquals(list.getSelectionIndices(), new int[0]);
-	
+
 	list.setSelection(number - 1);
 	assertArrayEquals(list.getSelectionIndices(), new int[] {number - 1});
 	assertEquals(list.getFocusIndex(), number - 1);
-	
+
 	list.setSelection(-1);
 	assertArrayEquals(list.getSelectionIndices(), new int[0]);
-	
+
 	list.removeAll();
-	
+
 	list.setSelection(0);
 	assertArrayEquals(list.getSelectionIndices(), new int[0]);
 	assertEquals(list.getFocusIndex(), -1);
-		
+
 	list.setSelection(-1);
 	assertArrayEquals(list.getSelectionIndices(), new int[0]);
 	assertEquals(list.getFocusIndex(), -1);
-		
+
 	list.setSelection(-2);
 	assertArrayEquals(list.getSelectionIndices(), new int[0]);
 	assertEquals(list.getFocusIndex(), -1);
@@ -2150,42 +2178,42 @@ public void test_setSelectionII() {
 		items[i] = "fred" + i;
 
 	list.setItems(items);
-	
+
 	list.setSelection(1, 2);
 	assertArrayEquals(list.getSelectionIndices(), new int[] {1, 2});
-	
+
 	list.setSelection(-3, -2);
 	assertArrayEquals(list.getSelectionIndices(), new int[0]);
-	
+
 	list.setSelection(0, 1);
 	assertArrayEquals(list.getSelectionIndices(), new int[] {0, 1});
-	
+
 	list.setSelection(-2, -1);
 	assertArrayEquals(list.getSelectionIndices(), new int[0]);
-	
+
 	list.setSelection(number - 2, number - 1);
 	assertArrayEquals(list.getSelectionIndices(), new int[] {number - 2, number - 1});
-	
+
 	list.setSelection(number - 1, number);
 	assertArrayEquals(list.getSelectionIndices(), new int[] {number - 1});
-	
+
 	list.setSelection(-1, 0);
 	assertArrayEquals(list.getSelectionIndices(), new int[] {0});
-	
+
 	list.setSelection(number, number + 1);
 	assertArrayEquals(list.getSelectionIndices(), new int[0]);
-	
+
 	list.setSelection(0, 0);
 	assertArrayEquals(list.getSelectionIndices(), new int[] {0});
 	assertEquals(list.getFocusIndex(), 0);
-	
+
 	list.setSelection(2, 1);
 	assertArrayEquals(list.getSelectionIndices(), new int[0]);
-	
+
 	list.setSelection(number - 1, number - 1);
 	assertArrayEquals(list.getSelectionIndices(), new int[] {number - 1});
 	assertEquals(list.getFocusIndex(), number - 1);
-	
+
 	list.setSelection(-1, -1);
 	assertArrayEquals(list.getSelectionIndices(), new int[0]);
 
@@ -2194,74 +2222,74 @@ public void test_setSelectionII() {
 	list.setSelection(-2, -1);
 	assertArrayEquals(list.getSelectionIndices(), new int[0]);
 	assertEquals(list.getFocusIndex(), -1);
-		
+
 	list.setSelection(-1, 0);
 	assertArrayEquals(list.getSelectionIndices(), new int[0]);
 	assertEquals(list.getFocusIndex(), -1);
-		
+
 	list.setSelection(0, 1);
 	assertArrayEquals(list.getSelectionIndices(), new int[0]);
 	assertEquals(list.getFocusIndex(), -1);
-		
+
 	list.setSelection(1, 0);
 	assertArrayEquals(list.getSelectionIndices(), new int[0]);
 	assertEquals(list.getFocusIndex(), -1);
-		
+
 	list.setSelection(0, -1);
 	assertArrayEquals(list.getSelectionIndices(), new int[0]);
 	assertEquals(list.getFocusIndex(), -1);
 
-	
+
 	setSingleList();
 	list.setItems(items);
-	
+
 	list.setSelection(0, 0);
 	assertArrayEquals(list.getSelectionIndices(), new int[] {0});
 	assertEquals(list.getFocusIndex(), 0);
-	
+
 	list.setSelection(1, 1);
 	assertArrayEquals(list.getSelectionIndices(), new int[] {1});
 	assertEquals(list.getFocusIndex(), 1);
-	
+
 	list.setSelection(4, 4);
 	assertArrayEquals(list.getSelectionIndices(), new int[] {4});
 	assertEquals(list.getFocusIndex(), 4);
-	
+
 	list.setSelection(number - 1, number - 1);
 	assertArrayEquals(list.getSelectionIndices(), new int[] {number - 1});
 	assertEquals(list.getFocusIndex(), number - 1);
-	
+
 	list.setSelection(number, number);
 	assertArrayEquals(list.getSelectionIndices(), new int[] {});
-	
+
 	list.setSelection(-3, -2);
 	assertArrayEquals(list.getSelectionIndices(), new int[] {});
-	
+
 	list.setSelection(0, 1);
 	assertArrayEquals(list.getSelectionIndices(), new int[] {});
-	
+
 	list.setSelection(-2, -1);
 	assertArrayEquals(list.getSelectionIndices(), new int[] {});
-	
+
 	list.setSelection(number - 2, number - 1);
 	assertArrayEquals(list.getSelectionIndices(), new int[] {});
-	
+
 	list.setSelection(number - 1, number);
 	assertArrayEquals(list.getSelectionIndices(), new int[] {});
-	
+
 	list.setSelection(-1, 0);
 	assertArrayEquals(list.getSelectionIndices(), new int[] {});
-	
+
 	list.setSelection(number, number + 1);
 	assertArrayEquals(list.getSelectionIndices(), new int[] {});
-	
+
 	list.setSelection(2, 1);
 	assertArrayEquals(list.getSelectionIndices(), new int[] {});
-	
+
 	list.setSelection(number - 1, number - 1);
 	assertArrayEquals(list.getSelectionIndices(), new int[] {number - 1});
 	assertEquals(list.getFocusIndex(), number - 1);
-	
+
 	list.setSelection(-1, -1);
 	assertArrayEquals(list.getSelectionIndices(), new int[] {});
 }
@@ -2284,7 +2312,7 @@ public void test_setTopIndexI() {
 		assertEquals("MULTI: setTopIndex(i=" + i + ")", i, list.getTopIndex());
 	}
 
-	
+
 	setSingleList();
 	list.setTopIndex(3);
 	assertEquals("SINGLE: setTopIndex(3) in empty list", 0, list.getTopIndex());
@@ -2304,7 +2332,7 @@ public void test_showSelection() {
 	list.setSelection(items);
 	list.showSelection();
 
-	
+
 	setSingleList();
 
 	list.setItems(items);
@@ -2322,7 +2350,7 @@ protected void deselectII_helper(
 	int start,
 	int end,
 	int[] expectedIndices) {
-		
+
 	list.setItems(items);
 	list.setSelection(items);
 
