@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2017 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -506,34 +509,68 @@ public void test_setActive() {
 	}
 	/* Create shell2 and make it active. */
 	Shell shell2 = new Shell();
+	shell2.setText("Shell2");
 	shell2.open();
 
 	/* Test setActive for visible shell. */
 	shell.setVisible(true);
 	shell.setActive();
+	if (SwtTestUtil.isCocoa) { //workaround for Bug 536564
+		drainEventQueue(shell.getDisplay(), 5000);
+	}
 	assertTrue("visible shell was not made active", shell.getDisplay().getActiveShell() == shell);
 
 	/* Test setActive for visible dialog shell. */
 	shell2.setActive();
+	testShell.setBounds(shell.getBounds());
 	testShell.setVisible(true);
 	testShell.setActive();
+	if (SwtTestUtil.isCocoa) { //workaround for Bug 536564
+		drainEventQueue(shell.getDisplay(), 2000);
+	}
 	assertTrue("visible dialog shell was not made active", testShell.getDisplay().getActiveShell() == testShell);
 
 	/* Test setActive for non-visible shell. */
 	shell2.setActive();
 	shell.setVisible(false);
 	shell.setActive();
+	shell2.setText("Shell2: Not active");
+	if (SwtTestUtil.isCocoa) { //workaround for Bug 536564
+		drainEventQueue(shell.getDisplay(), 2000);
+	}
 	assertTrue("non-visible shell was made active", shell.getDisplay().getActiveShell() != shell);
 
 	/* Test setActive for non-visible dialog shell. */
 	shell2.setActive();
 	testShell.setVisible(false);
 	testShell.setActive();
+	shell2.setText("Shell2: Not active");
+	if (SwtTestUtil.isCocoa) { //workaround for Bug 536564
+		drainEventQueue(shell.getDisplay(), 2000);
+	}
 	assertTrue("non-visible dialog shell was made active", testShell.getDisplay().getActiveShell() != testShell);
 
 	shell2.dispose();
 }
 
+private static void drainEventQueue(Display display, int millis) {
+	if (millis == 0) {
+		while (!display.isDisposed() && display.readAndDispatch()) {
+		}
+		return;
+	}
+	long end = System.currentTimeMillis() + millis;
+	while (!display.isDisposed() && System.currentTimeMillis() < end) {
+		if (!display.readAndDispatch ()) {
+			try {
+				Thread.sleep(10);
+			} catch (InterruptedException e) {
+				e.printStackTrace();
+			}
+		}
+	}
+}
+
 @Override
 @Test
 public void test_setEnabledZ() {
@@ -718,6 +755,22 @@ public void test_setSizeII() {
 
 }
 
+@Test
+public void test_setSizeCustomResize() {
+	/*
+	 * Bug 319612: test the case where Shells with SWT.RESIZE and SWT.ON_TOP
+	 * were shrinking in size. Only applicable to GTK (both GTK2 and GTK3).
+	 */
+	if (SwtTestUtil.isGTK) {
+		Shell testShell = new Shell (shell, SWT.RESIZE | SWT.ON_TOP);
+		Point size = new Point (200, 200);
+		testShell.setSize(size);
+		testShell.open();
+		assertEquals(size, testShell.getSize());
+		testShell.dispose();
+	}
+}
+
 @Override
 @Test
 public void test_setSizeLorg_eclipse_swt_graphics_Point() {
