@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2016 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2017 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -11,6 +14,7 @@
 package org.eclipse.swt.tests.junit;
 
 import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 
@@ -78,7 +82,7 @@ public void test_addSelectionListenerLorg_eclipse_swt_events_SelectionListener()
 	exceptionThrown = false;
 	slider.addSelectionListener(listener);
 	slider.setSelection(0);
-	assertTrue(":a:", listenerCalled == false);
+	assertFalse(":a:", listenerCalled);
 	slider.removeSelectionListener(listener);
 	try {
 		slider.removeSelectionListener(null);
@@ -89,6 +93,17 @@ public void test_addSelectionListenerLorg_eclipse_swt_events_SelectionListener()
 	assertTrue("Expected exception not thrown", exceptionThrown);
 }
 
+@Test
+public void test_addSelectionListenerWidgetSelectedAdapterLorg_eclipse_swt_events_SelectionListener() {
+	listenerCalled = false;
+	SelectionListener listener = SelectionListener.widgetSelectedAdapter(e -> listenerCalled = true);
+
+	slider.addSelectionListener(listener);
+	slider.setSelection(0);
+	assertFalse(":a:", listenerCalled);
+	slider.removeSelectionListener(listener);
+}
+
 @Override
 @Test
 public void test_computeSizeIIZ() {
@@ -102,7 +117,7 @@ public void test_getIncrement() {
 	{
 	  slider.setIncrement(cases[i]);
 	  assertTrue("case: " + String.valueOf(i), slider.getIncrement()==cases[i]);
-	} 
+	}
 }
 
 @Test
@@ -140,14 +155,14 @@ public void test_getPageIncrement() {
 	{
 	  slider.setPageIncrement(cases[i]);
 	  assertTrue("case: " + String.valueOf(i), slider.getPageIncrement()==cases[i]);
-	} 
+	}
 }
 
 @Test
 public void test_getSelection() {
 	slider.setSelection(10);
 	assertTrue(":a:", slider.getSelection()== 10);
-	
+
 }
 
 @Override
@@ -529,7 +544,7 @@ return new int[][] {
 {97, 97, 100, 97, 3},
 {98, 98, 100, 98, 2},
 {99, 99, 100, 99, 1},
-{100, 10, 100, 50, 10}, 
+{100, 10, 100, 50, 10},
 {101, 10, 100, 50, 10},
 {102, 10, 100, 50, 10},
 {103, 10, 100, 50, 10},
