@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2018 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -11,6 +14,7 @@
 package org.eclipse.swt.tests.junit;
 
 import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertNull;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
@@ -469,17 +473,17 @@ public void test_setBackgroundLorg_eclipse_swt_graphics_Color() {
 
 @Test
 public void test_setCheckedZ() {
-	assertEquals(false, tableItem.getChecked());
+	assertFalse(tableItem.getChecked());
 
 	tableItem.setChecked(true);
-	assertEquals(false, tableItem.getChecked());
+	assertFalse(tableItem.getChecked());
  	Table t = new Table(shell, SWT.CHECK);
 	TableItem ti = new TableItem(t, SWT.NULL);
 	ti.setChecked(true);
 	assertTrue(ti.getChecked());
 
 	ti.setChecked(false);
-	assertEquals(false, ti.getChecked());
+	assertFalse(ti.getChecked());
 	t.dispose();
 }
 
@@ -624,11 +628,11 @@ public void test_setForegroundLorg_eclipse_swt_graphics_Color() {
 public void test_setGrayedZ() {
 	Table newTable = new Table(shell, SWT.CHECK);
 	TableItem tItem = new TableItem(newTable,0);
-	assertEquals(false, tItem.getGrayed());
+	assertFalse(tItem.getGrayed());
 	tItem.setGrayed(true);
 	assertTrue(tItem.getGrayed());
 	tItem.setGrayed(false);
-	assertEquals(false, tItem.getGrayed());
+	assertFalse(tItem.getGrayed());
 	newTable.dispose();
 }
 
