@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2016 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2018 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -11,15 +14,20 @@
 package org.eclipse.swt.tests.junit;
 
 import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertNull;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 
+import java.lang.reflect.Field;
+import java.util.function.BooleanSupplier;
+
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.events.DisposeListener;
 import org.eclipse.swt.graphics.GC;
 import org.eclipse.swt.widgets.Control;
+import org.eclipse.swt.widgets.Display;
 import org.eclipse.swt.widgets.Event;
 import org.eclipse.swt.widgets.Listener;
 import org.eclipse.swt.widgets.Shell;
@@ -54,14 +62,22 @@ public void tearDown() {
 	if (widget != null) {
 		assertEquals(disposedIntentionally, widget.isDisposed());
 	}
+	Display display = null;
 	if (!disposedIntentionally) {
-		assertEquals(false, shell.isDisposed());
+		assertFalse(shell.isDisposed());
+		display = shell.getDisplay();
 	}
 	shell.dispose();
 	if (widget != null) {
 		assertTrue(widget.isDisposed());
+		if(SwtTestUtil.isLinux && display != null) {
+			assertNotExists(getWidgetTable(display), widget);
+		}
 	}
 	assertTrue(shell.isDisposed());
+	if(SwtTestUtil.isLinux && display != null) {
+		assertNotExists(getWidgetTable(display), shell);
+	}
 }
 @Test
 public void test_ConstructorLorg_eclipse_swt_widgets_WidgetI() {
@@ -94,7 +110,7 @@ public void test_getDisplay() {
 }
 @Test
 public void test_isDisposed() {
-	assertEquals(false, widget.isDisposed());
+	assertFalse(widget.isDisposed());
 }
 @Test
 public void test_notifyListenersILorg_eclipse_swt_widgets_Event() {
@@ -207,6 +223,40 @@ protected String getClassName() {
     return clazz;
 }
 
+protected void processEvents(int timeoutMs, BooleanSupplier condition) throws InterruptedException {
+	if (condition == null) {
+		condition = () -> false;
+	}
+	long targetTimestamp = System.currentTimeMillis() + timeoutMs;
+	while (!condition.getAsBoolean()) {
+		if (!shell.getDisplay().readAndDispatch()) {
+			if (System.currentTimeMillis() < targetTimestamp) {
+				Thread.sleep(50);
+			} else {
+				return;
+			}
+		}
+	}
+}
 
+protected void assertNotExists(Widget[] widgetTable, Widget w) {
+	for (Widget widget : widgetTable) {
+		if(widget == w) {
+			fail("Widget table leaks: " + w);
+		}
+	}
+}
+
+protected static Widget[] getWidgetTable(Display display) {
+	try {
+		Field field = Display.class.getDeclaredField("widgetTable");
+		field.setAccessible(true);
+		Widget[] widgetTable = (Widget[]) field.get(display);
+		return widgetTable;
+	} catch (Throwable t) {
+		t.printStackTrace();
+		return null;
+	}
+}
 
 }
